/* 
 * UserGesture.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.app;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import com.eXtropia.auth.User;

/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */
public class UserGesture {
    public static final String REQUEST_PARAM_NAME = "gesture";

    public static final String ACTION_REQUEST_PARAM_NAME = "action";
    public static final String VIEW_REQUEST_PARAM_NAME = "view";
    public static final String TARGET_VIEW_REQUEST_PARAM_NAME = "targetView";
    public static final String BASE_VIEW_REQUEST_PARAM_NAME = "baseView";
    public static final String MULTIPART_FORM_DATA = "multipart/form-data";

    private String _action;
    private Map _data;
    private String _baseView;
    private String _view;
    private String _targetView;
    private Date _requestDate;
    protected User _user;

    public UserGesture() {
        _requestDate = new Date();
    }

    public String getAction() {
        return _action;
    }

    public Map getData() {
        return _data;
    }

    public Date getRequestDate() {
        return _requestDate;
    }

    public Map getRequestParams() {
        Map map = new HashMap();
        if (_action != null)
            map.put(ACTION_REQUEST_PARAM_NAME, _action);
        if (_view != null)
            map.put(VIEW_REQUEST_PARAM_NAME, _view);
        if (_targetView != null)
            map.put(TARGET_VIEW_REQUEST_PARAM_NAME, _targetView);
        if (_baseView != null)
            map.put(BASE_VIEW_REQUEST_PARAM_NAME, _baseView);
        return map;
    }

    public String getBaseView() {
        return _baseView;
    }

    public String getTargetView() {
        return _targetView;
    }

    public User getUser() {
        return _user;
    }

    public String getView() {
        return _view;
    }

    public void setAction(String action) {
        _action = action;
    }

    public void setBaseView(String baseView) {
        _baseView = baseView;
    }

    public void setData(Map data) {
        _data = data;
    }

    public void setTargetView(String targetView) {
        _targetView = targetView;
    }

    public void setUser(User user) {
        _user = user;
    }

    public void setView(String view) {
        _view = view;
    }

    public boolean validate() {
        return true;
    }
}