/*
 * UserGestureFactory.java
 * Copyright (C) 2000 -2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.app.servlet;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Enumeration;
import java.util.HashMap;

import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;

import com.eXtropia.app.MultipartUserGesture;
import com.eXtropia.app.UserGesture;
import com.eXtropia.config.Config;
import com.eXtropia.config.ConfigException;
import com.eXtropia.config.Configurable;
import com.oreilly.servlet.multipart.FilePart;
import com.oreilly.servlet.multipart.MultipartParser;
import com.oreilly.servlet.multipart.ParamPart;
import com.oreilly.servlet.multipart.Part;

/**
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */
public class UserGestureFactory implements Configurable {
    public void configure(Config config) throws ConfigException {

    }

    public UserGesture create(ServletRequest request) {
        UserGesture g;
        String action = null;

        if (request.getContentType() == null || request.getContentType().indexOf(UserGesture.MULTIPART_FORM_DATA) < 0) {
            g = new UserGesture();

            g.setTargetView(request.getParameter(UserGesture.TARGET_VIEW_REQUEST_PARAM_NAME));
            g.setView(request.getParameter(UserGesture.VIEW_REQUEST_PARAM_NAME));
            g.setBaseView(request.getParameter(UserGesture.BASE_VIEW_REQUEST_PARAM_NAME));

            String currentRequestParam;
            for (Enumeration requestParamNames = request.getParameterNames(); requestParamNames.hasMoreElements();) {
                currentRequestParam = (String) requestParamNames.nextElement();
                if (currentRequestParam.startsWith(UserGesture.ACTION_REQUEST_PARAM_NAME)) {
                    action = currentRequestParam.substring(UserGesture.ACTION_REQUEST_PARAM_NAME.length());
                    if (action != null && action.length() > 0)
                        if (action.endsWith(".x") || action.endsWith(".y")) {
                            action = action.substring(0, action.length() - 2);
                        }
                    g.setAction(action);
                    break;
                }
            }
            if (action == null && request.getParameter("defaultAction") != null) {
                action = request.getParameter("defaultAction");
                g.setAction(action);
            }
        } else // multipart data
            {
            MultipartUserGesture mg = new MultipartUserGesture();
            HashMap hm = new HashMap();

            try {
                MultipartParser mp = new MultipartParser((HttpServletRequest) request, 10 * 1024 * 1024); // 10MB
                Part part;

                while ((part = mp.readNextPart()) != null) {
                    String name = part.getName();
                    if (part.isParam()) {
                        // it's a parameter part
                        ParamPart paramPart = (ParamPart) part;
                        String value = paramPart.getStringValue();
                        String key = paramPart.getName();
                        if (key.equals(UserGesture.TARGET_VIEW_REQUEST_PARAM_NAME))
                            mg.setTargetView(value);
                        else if (key.equals(UserGesture.VIEW_REQUEST_PARAM_NAME))
                            mg.setView(value);
                        else if (key.equals(UserGesture.BASE_VIEW_REQUEST_PARAM_NAME))
                            mg.setBaseView(value);
                        else if (name.startsWith(UserGesture.ACTION_REQUEST_PARAM_NAME)) {
                            action = key.substring(UserGesture.ACTION_REQUEST_PARAM_NAME.length());
                            if (action != null && action.length() > 0)
                                if (action.endsWith(".x") || action.endsWith(".y")) {
                                    action = action.substring(0, action.length() - 2);
                                }
                            mg.setAction(action);
                            break;
                        } else
                            hm.put(key, value);
                    } else if (part.isFile()) {
                        // it's a file part
                        FilePart filePart = (FilePart) part;
                        String fileName = filePart.getFileName();

                        if (fileName != null) {
                            // the part actually contained a file
                            ByteArrayOutputStream bs = new ByteArrayOutputStream();
                            long size = filePart.writeTo(bs);
                            hm.put(filePart.getName(), bs);
                            hm.put(filePart.getName() + "_filename", fileName);
                        } else {
                            // the field did not contain a file
                        }
                    } // else if (part.isFile())
                } // while ((part = mp.readNextPart()) != null)
            } catch (IOException lEx) {
                System.out.println("Exception Uploading File " + lEx.getMessage());
            }
            mg.setMultipartData(hm);
            g = mg;
        }
        //System.out.println("Action: " + g.getAction());
        return g;
    }
}