/* 
 * AuthManager.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.auth;

import java.util.Map;

import com.eXtropia.config.Configurable;
import com.eXtropia.util.ExtropiaException;

/** 
 * AuthManager: manage authentication and authoriaztion services for
 * clients.
 *
 * The AuthManager interface is the uniform front-end to all
 * authentication and authorization services in the com.eXtropia.auth
 * and related package.  Clients should rely on this interface instead
 * of actual implementation classes for functionality.
 *
 * @author Carlo Ravagli (carlo.ravagli@extropia.com)
 * @author Parijat Mishra (parijat@extropia.com)
 * @version 0.2
 *
 * Internal Revision: $Revision: 1.14 $
 */
public interface AuthManager extends Configurable {
    /**
     * The XML tag under which an AuthManager's config may be found
     * in a config file.
     */
    public static final String CFG_BASE = "AuthManager";

    /**
    * If a servlet context shares a AuthManager, it will be found under this attribute name
    */
    public static final String SERVLET_CONTEXT_ATTRIBUTE_NAME = "com.eXtropia.auth.AuthManager.SERVLET_CONTEXT_ATTRIBUTE";

    /**
     * The identifier used by AuthManager to store user objects in sessions.
     * Client code may access the validated user from a session object
     *	using this identifier.
     */
    public static final String KEY_USER = "com.eXtropia.auth.AuthManager.User";

    public User authenticate(Object request, Object response) throws AuthException, ExtropiaException;
    public User register(Object id, Object password, Map regInfo) throws AuthException, ExtropiaException;
    public User getUser(Object id) throws AuthException, ExtropiaException;
    public User getUserIfLoggedIn(Object request);
    public User[] getMembers(String group) throws AuthException, ExtropiaException;
}
