/* 
 * AuthManagerInitServlet.java
 * Copyright (C) 2000  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.auth;

import com.eXtropia.app.servlet.ExtropiaServlet;
import com.eXtropia.config.Config;
import com.eXtropia.config.ConfigException;

/** 
 * <code>AuthManagerInitServlet</code> is the initialization class for the
 * Auth Engine.
 *
 * @author  <a href="mailto:tim@eXtropia.com">Tim Parker</a>
 * @author  <a href="mailto:st_lim@eXtropia.com">Lim Swee Tat</a>
 * @version $Id: AuthManagerInitServlet.java,v 1.9 2002/11/16 08:45:09 rayfd Exp $
 */
public abstract class AuthManagerInitServlet extends ExtropiaServlet {
    /** 
     * Creates new AuthManagerInitServlet
     */
    public AuthManagerInitServlet() {
    }

    /* (non-Javadoc)
     * @see com.eXtropia.config.Configurable#configure(com.eXtropia.config.Config)
     */
    public void configure(Config config) throws ConfigException {
        String appName = config.getName();
        String appInstance = config.getValue();
        super.configureDoctor(config, appInstance, appName);

        if (!config.getName().equals(AuthManager.CFG_BASE)) {
            throw new ConfigException("AuthManagerInitServlet config base: " + AuthManager.CFG_BASE);
        }

        try {
            AuthManager authManager = (AuthManager) config.createAndConfigureInstance();
            getServletContext().setAttribute(AuthManager.SERVLET_CONTEXT_ATTRIBUTE_NAME, authManager);
        } catch (ClassCastException e) {
            throw new ConfigException(AuthManager.CFG_BASE + " is not of type AuthManager");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
} // AuthManagerInitServlet.java
