/* 
 * FormVarActionManager.java
 * Copyright (C) 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.auth.authmanager.formvar;

import java.util.Map;

import com.eXtropia.app.servlet.Action;
import com.eXtropia.app.servlet.ActionManager;
import com.eXtropia.auth.AuthException;
import com.eXtropia.auth.Authenticator;
import com.eXtropia.auth.User;
import com.eXtropia.config.Config;
import com.eXtropia.config.ConfigException;
import com.eXtropia.util.ExtropiaException;
import com.eXtropia.view.ViewManager;

/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */
public class FormVarActionManager extends ActionManager {
    public static final String CFG_REGISTRATION_STEPS = "RegistrationSteps";

    protected Authenticator _authenticator;
    protected Integer _registrationSteps;

    public FormVarActionManager(String appName, String appInstance, Authenticator authenticator, ViewManager viewManager) {
        super(appName, appInstance, null, viewManager);
        _authenticator = authenticator;
    }

    public void configureConcrete(Config config) throws ConfigException {
    }

    public Action createAction(Config config) throws ConfigException {
        Action action = super.createAction(config);
        if (action instanceof AuthAction) {
            AuthAction authAction = (AuthAction) action;
            authAction.setAuthenticator(_authenticator);
            authAction.setRegistrationSteps(_registrationSteps);
        }
        return action;
    }

    public User getUser(Object id) throws AuthException, ExtropiaException {
        return _authenticator.getUser(id);
    }

    public User[] getMembers(String group) throws AuthException, ExtropiaException {
        return _authenticator.getMembers(group);
    }

    public User register(Object id, Object password, Map regInfo) throws AuthException, ExtropiaException {
        return _authenticator.register(id, password, regInfo);
    }
}