/* 
 * FormVarAuthManager.java
 * Copyright (C) 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.auth.authmanager.formvar;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import com.eXtropia.app.UserGestureResponse;
import com.eXtropia.app.servlet.ActionManager;
import com.eXtropia.auth.AuthException;
import com.eXtropia.auth.AuthManager;
import com.eXtropia.auth.Authenticator;
import com.eXtropia.auth.User;
import com.eXtropia.config.Config;
import com.eXtropia.config.ConfigException;
import com.eXtropia.diagnostics.Diagnostics;
import com.eXtropia.util.ExtropiaException;
import com.eXtropia.view.ViewManager;

/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */
public class FormVarAuthManager implements AuthManager {
    protected Authenticator _authenticator;
    protected FormVarActionManager _actionManager;

    public FormVarAuthManager() {
    }

    public User authenticate(Object req, Object res) throws AuthException, ExtropiaException {
        HttpServletRequest request = (HttpServletRequest) req;
        HttpServletResponse response = (HttpServletResponse) res;
        UserGestureResponse gr = _actionManager.execute(request, response);
        //if we get here, there is no user in the session
        if (gr != null && gr instanceof AuthManagerUserGestureResponse) {
            AuthManagerUserGestureResponse gestureResponse = (AuthManagerUserGestureResponse) gr;
            User user = gestureResponse.getUser();
            return user;
        } else
            return null;
    }

    public void configure(Config config) throws ConfigException {
        ViewManager viewManager;
        String appName = config.getName();
        String appInstance = config.getValue();

        if (config.hasChild(Diagnostics.CFG_BASE)) {
            Diagnostics.configureFactory(config.getChild(Diagnostics.CFG_BASE));
        }

        if (config.hasChild(Authenticator.CFG_BASE)) {
            try {
                _authenticator = (Authenticator) config.getChild(Authenticator.CFG_BASE).createAndConfigureInstance();
            } catch (ClassCastException cce) {
                throw new ConfigException(Authenticator.CFG_BASE + " is not of type " + Authenticator.class.getName());
            }
        } else
            throw new ConfigException("No " + Authenticator.CFG_BASE + " specified in " + AuthManager.CFG_BASE);

        if (config.hasChild(ViewManager.CFG_BASE)) {
            viewManager = new ViewManager(appName, appInstance);
            viewManager.configure(config.getChild(ViewManager.CFG_BASE));
        } else
            throw new ConfigException("No " + ViewManager.CFG_BASE + " specified in " + AuthManager.CFG_BASE);

        if (config.hasChild(ActionManager.CFG_BASE)) {
            _actionManager = new FormVarActionManager(appName, appInstance, _authenticator, viewManager);
            _actionManager.configure(config.getChild(ActionManager.CFG_BASE));
        } else
            throw new ConfigException("No " + ActionManager.CFG_BASE + " specified in " + AuthManager.CFG_BASE);
    }

    public User getUser(Object id) throws AuthException, ExtropiaException {
        return _authenticator.getUser(id);
    }

    public User[] getMembers(String group) throws AuthException, ExtropiaException {
        return _authenticator.getMembers(group);
    }

    public User getUserIfLoggedIn(Object r) {
        HttpServletRequest request = (HttpServletRequest) r;
        HttpSession session = request.getSession(false);
        if (session != null)
            return (User) session.getAttribute(KEY_USER);
        else
            return null;
    }

    public User register(Object id, Object password, Map regInfo) throws AuthException, ExtropiaException {
        return _actionManager.register(id, password, regInfo);
    }
}