/* 
 * Login.java
 * Copyright (C) 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.auth.authmanager.formvar;

import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import com.eXtropia.app.UserGesture;
import com.eXtropia.app.UserGestureResponse;
import com.eXtropia.auth.AuthManager;
import com.eXtropia.auth.InvalidLoginDetailsException;
import com.eXtropia.auth.User;
import com.eXtropia.datahandler.DataHandlerException;
import com.eXtropia.util.ExtropiaException;
/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */
public class Login extends AuthAction {
    public Login() {
        setRequiresAuthentication(false);
    }

    public UserGestureResponse execute(UserGesture g, ServletRequest request) throws ExtropiaException {
        AuthManagerUserGestureResponse response = new AuthManagerUserGestureResponse();
        HttpSession session = ((HttpServletRequest) request).getSession();
        User user;

        //check to see if this is a relogin
        if (request.getParameter(AuthManagerUserGesture.REQUEST_PARAM_AUTH_RELOGIN) != null) {
            session.removeAttribute(AuthManager.KEY_USER);
        } else {
            user = (User) session.getAttribute(AuthManager.KEY_USER);
            if (user != null) {
                response.setUser(user);
                return response;
            }
        }

        AuthManagerUserGesture gesture = (AuthManagerUserGesture) g;
        Object username = gesture.getUsername();
        Object password = gesture.getPassword();
        //System.out.println("Validating: " + username + "/" + password);
        if (username == null || password == null) {
            return getInvalidDetailsResponse(gesture, request);
        } else {
            try {
                user = _authenticator.authenticate(username, password);
                if (user == null)
                    throw new InvalidLoginDetailsException();
            } catch (InvalidLoginDetailsException ilde) {
                return getInvalidDetailsResponse(gesture, request);
            }
            if (user != null)
                session.setAttribute(AuthManager.KEY_USER, user);
            response.setUser(user);
            return response;
        }
    }

    protected AuthManagerUserGestureResponse getInvalidDetailsResponse(AuthManagerUserGesture g, ServletRequest request) {
        AuthManagerUserGestureResponse response = new AuthManagerUserGestureResponse();
        if (request.getParameter(AuthManagerUserGesture.REQUEST_PARAM_AUTH_RELOGIN) != null) {
            DataHandlerException dhe = new DataHandlerException("rule");
            DataHandlerException dhe1 = new DataHandlerException("logic");
            DataHandlerException dhe2 = new DataHandlerException("login");
            DataHandlerException dhe3 = new DataHandlerException("InvalidDetails");
            dhe.addChild(dhe1);
            dhe1.addChild(dhe2);
            dhe2.addChild(dhe3);
            response.setDataHandlerException(dhe);
        }
        response.setView(g.getView());
        return response;
    }
}