/* 
 * UpdateProfile.java
 * Copyright (C) 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.auth.authmanager.formvar;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import com.eXtropia.app.UserGesture;
import com.eXtropia.app.UserGestureResponse;
import com.eXtropia.auth.AuthManager;
import com.eXtropia.auth.User;
import com.eXtropia.util.ExtropiaException;

/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */
public class UpdateProfile extends AuthAction {
    public UpdateProfile() {
        setRequiresAuthentication(true);
    }

    public UserGestureResponse execute(UserGesture g, ServletRequest request) throws ExtropiaException {
        AuthManagerUserGestureResponse response = new AuthManagerUserGestureResponse();
        HttpSession session = ((HttpServletRequest) request).getSession(false);
        if (session == null)
            throw new ExtropiaException("Can't update user profile without a session");
        User user = (User) session.getAttribute(AuthManager.KEY_USER);
        if (user == null) {
            //System.out.println("Not currently logged in");
            throw new ExtropiaException("Can't update user profile when not logged in");
        }

        Map attributes = new HashMap();
        Map.Entry entry;
        for (Iterator i = g.getData().entrySet().iterator(); i.hasNext();) {
            entry = (Map.Entry) i.next();
            if (!(entry.getKey().equals(AuthManagerUserGesture.REQUEST_PARAM_AUTH_USERNAME) || entry.getKey().equals(AuthManagerUserGesture.REQUEST_PARAM_AUTH_PASSWORD))) {
                if (entry.getKey().equals(AuthManagerUserGesture.REQUEST_PARAM_AUTH_NEW_PASSWORD)) {
                    user.setPassword(entry.getValue());
                } else if (((String) entry.getKey()).startsWith(AuthManagerUserGesture.REQUEST_PARAM_AUTH_PREFIX)) {
                    attributes.put((String) entry.getKey(), entry.getValue());
                }
            }
        }
        user.setAttributes(attributes);
        response.setUser(user);
        return response;
    }
}