/* 
 * DataHandler.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.datahandler;

import java.util.ArrayList;
import java.util.List;

import com.eXtropia.config.Config;
import com.eXtropia.config.ConfigException;
import com.eXtropia.config.Configurable;

/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 
 */
public abstract class DataHandler implements Configurable {
    public static final String CFG_BASE = "DataHandler";
    public static final String CFG_ID = "Id";
    public static final String CFG_NAME = "Name";
    public static final String CFG_TRANSFORM_ID = "TransformId";

    protected String _name;
    protected String[] _ids;
    protected String _transformId;

    /*public DataHandler()
    {
    }*/

    public DataHandler(String name) {
        setName(name);
    }

    public void configure(Config config) throws ConfigException {
        if (!config.getName().equals(CFG_BASE))
            throw new ConfigException("Incorrect Config passed to " + CFG_BASE + ": " + config.getName());

        if (config.hasChild(CFG_NAME))
            setName(config.getChild(CFG_NAME).getValue());

        if (config.hasChild(CFG_TRANSFORM_ID)) {
            setTransformId(config.getChild(CFG_TRANSFORM_ID).getValue());
        }

        Config[] children = config.getChildren();
        List ids = new ArrayList();
        for (int i = 0; i < children.length; i++) {
            if (children[i].getName().equals(CFG_ID)) {
                ids.add(children[i].getValue());
            }
        }

        if (ids.size() > 0)
            setIds((String[]) ids.toArray(new String[ids.size()]));
    }

    public String[] getIds() {
        return _ids;
    }

    public String getName() {
        return _name;
    }

    public String getTransformId() {
        return _transformId == null ? _ids[0] : _transformId;
    }

    public void setIds(String[] ids) {
        _ids = ids;
    }

    public void setName(String name) {
        _name = name;
    }

    public void setTransformId(String id) {
        _transformId = id;
    }

    public abstract Object validate(Object[] values) throws DataHandlerException;
}