/* 
 * DataHandlerException.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.datahandler;

import java.util.HashMap;
import java.util.Map;
import java.util.StringTokenizer;

import com.eXtropia.util.ExtropiaException;

/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */
public class DataHandlerException extends ExtropiaException {
    protected String _resourceName;
    protected DataHandlerException _parent;

    protected String[] _ids;
    protected Object[] _values;
    protected Map _childExceptions = new HashMap();
    protected Map _exceptionData;

    public DataHandlerException(String resourceName) {
        _resourceName = resourceName;
    }

    public DataHandlerException(String resourceName, String error) {
        super(error);
        _resourceName = resourceName;
    }

    public DataHandlerException(String resourceName, String id, Object value) {
        _resourceName = resourceName;
        _ids = new String[] { id };
        _values = new Object[] { value };
    }

    public DataHandlerException(String resourceName, String error, String id, Object value) {
        this(resourceName, error, new String[] { id }, new Object[] { value });
    }

    public DataHandlerException(String resourceName, String[] ids, Object[] values) {
        _resourceName = resourceName;
        _ids = ids;
        _values = values;
    }

    public DataHandlerException(String resourceName, String error, String[] ids, Object[] values) {
        super(error);
        _resourceName = resourceName;
        _ids = ids;
        _values = values;
    }

    public void setExceptionData(Map m) {
        _exceptionData = m;
    }
    public Map getExceptionData() {
        return _exceptionData;
    }

    public void addChild(DataHandlerException dhe) {
        _childExceptions.put(dhe._resourceName, dhe);
        dhe._parent = this;
        super.addChild(dhe);
    }

    public String getId() {
        try {
            return _ids[0];
        } catch (ArrayIndexOutOfBoundsException e) {
            return null;
        }
    }

    public String[] getIds() {
        return _ids;
    }

    public DataHandlerException getParent() {
        return _parent;
    }

    public String getResourceName() {
        String result;
        result = _resourceName;
        DataHandlerException parent = _parent;
        while (parent != null) {
            result = parent._resourceName + '.' + result;
            parent = parent._parent;
        }
        String error = getMessage();
        if (error != null)
            result += '.' + error;
        return result;
    }

    public Object getValue() {
        try {
            return _values[0];
        } catch (ArrayIndexOutOfBoundsException e) {
            return null;
        }
    }

    public Object[] getValues() {
        return _values;
    }

    public boolean isMultipleId() {
        return !(_ids == null || _ids.length == 1);
    }

    public DataHandlerException getFirstError(String indexString) {
        DataHandlerException dhe = this;
        String currentIndex;
        for (StringTokenizer st = new StringTokenizer(indexString, "."); st.hasMoreTokens();) {
            currentIndex = st.nextToken();
            if (dhe._childExceptions.containsKey(currentIndex))
                dhe = (DataHandlerException) dhe._childExceptions.get(currentIndex);
            else
                return null;
        }
        //now get the first leaf error that resides under this
        while (dhe.hasChildren()) {
            dhe = (DataHandlerException) dhe.getChildren()[0];
        }
        return dhe;
    }
}