/* 
 * DataHandlerManager.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.datahandler;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import com.eXtropia.config.Config;
import com.eXtropia.config.ConfigException;
import com.eXtropia.config.Configurable;

/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */
public abstract class DataHandlerManager implements Configurable {
    public static final String CFG_BASE = "DataHandlerManager";
    public static final String CFG_DATA_ACCESSOR = "DataAccessor";

    protected Rule _rule;
    protected String[] _referencedIds;
    protected Class _dataAccessor;

    /** Creates new DataHandlerManager */
    public DataHandlerManager() {

    }

    public void configure(Config config) throws ConfigException {
        if (!config.getName().equals(CFG_BASE))
            throw new ConfigException("Invalid config passed to " + CFG_BASE + ": " + config.getName());

        //configure the top level rule
        if (config.hasChild(Rule.CFG_BASE))
            setRule((Rule) config.getChild(Rule.CFG_BASE).createAndConfigureInstance());
        else
            throw new ConfigException("No " + Rule.CFG_BASE + " specified in " + CFG_BASE + " config");
    }

    private void determineRequiredIds(Rule rule, Set set) {
        Rule[] childRules = rule.getRules();
        if (childRules != null) {
            for (int i = 0; i < childRules.length; i++) {
                //System.out.println("Checking rule: " + childRules[i].getName());
                determineRequiredIds(childRules[i], set);
            }
        }
        DataHandler[] dataHandlers = rule.getDataHandlers();
        String[] ids;
        if (dataHandlers != null) {
            for (int i = 0; i < dataHandlers.length; i++) {
                ids = dataHandlers[i].getIds();
                for (int j = 0; j < ids.length; j++) {
                    if (ids[j] == null)
                        throw new IllegalArgumentException("Datahandler requires a null key: " + dataHandlers[i].getName());
                    //System.out.println("DataHandler " + dataHandlers[i].getName() + " requires id " + ids[j]);
                    set.add(ids[j]);
                }
            }
        }
        return;
    }

    public Map execute(Object object) throws DataHandlerException, IllegalStateException {
        Map data = new HashMap();
        for (int i = 0; i < _referencedIds.length; i++)
            data.put(_referencedIds[i], getOriginalData(_referencedIds[i], object));
        try {
            _rule.execute(data);
        } catch (DataHandlerException dhe) {
            //save successfully transformed data back to the object
            setOriginalData(data, object);
            throw dhe;
        }
        setOriginalData(data, object);
        return data;
    }

    private void setOriginalData(Map data, Object originalData) {
        Map.Entry entry;
        for (Iterator i = data.entrySet().iterator(); i.hasNext();) {
            entry = (Map.Entry) i.next();
            setOriginalData((String) entry.getKey(), entry.getValue(), originalData);
        }
    }

    public abstract Object getOriginalData(String key, Object originalData) throws IllegalArgumentException;
    public abstract void setOriginalData(String key, Object value, Object originalData) throws IllegalArgumentException;

    public void setRule(Rule rule) {
        _rule = rule;
        Set referencedIds = new HashSet();
        determineRequiredIds(_rule, referencedIds);
        _referencedIds = (String[]) referencedIds.toArray(new String[referencedIds.size()]);
    }
}