/* 
 * ComparableHandler.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.datahandler.handler;

import com.eXtropia.config.Config;
import com.eXtropia.config.ConfigException;
import com.eXtropia.datahandler.DataHandler;
import com.eXtropia.datahandler.DataHandlerException;

/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */
public class ComparableHandler extends DataHandler {
    public static final String CFG_TYPE = "Type";

    public static final int TYPE_LESS_THAN = 3318169; /* less */
    public static final int TYPE_LESS_EQUAL = 371449147; /* lessequal */
    public static final int TYPE_GREATER_THAN = 283601914; /* greater */
    public static final int TYPE_GREATER_EQUAL = -1369217158; /* greaterequal */
    public static final int TYPE_EQUAL = 96757556; /* equal */
    public static final int TYPE_NOT_EQUAL = 1582008385; /* notequal */
    protected int _type = TYPE_EQUAL;

    /** Creates new DateComparison */
    public ComparableHandler() {
        super("comparable");
    }

    public void configure(Config config) throws ConfigException {
        super.configure(config);
        if (config.hasChild(CFG_TYPE)) {
            int hash = config.getChild(CFG_TYPE).getValue().toLowerCase().hashCode();
            switch (hash) {
                case TYPE_LESS_THAN :
                case TYPE_LESS_EQUAL :
                case TYPE_EQUAL :
                case TYPE_GREATER_EQUAL :
                case TYPE_GREATER_THAN :
                case TYPE_NOT_EQUAL :
                    setType(config.getChild(CFG_TYPE).getValue().toLowerCase().hashCode());
                    break;
                default :
                    throw new ConfigException("Unknown " + CFG_TYPE + ": " + config.getChild(CFG_TYPE).getValue());
            }
        }
    }

    public void setIds(String[] ids) {
        if (ids.length != 2)
            throw new IllegalArgumentException("Need exactly two ids");
        else
            super.setIds(ids);
    }

    public void setType(int type) {
        _type = type;
    }

    public Object validate(Object[] values) throws DataHandlerException {
        Comparable v1 = (Comparable) values[0];
        Comparable v2 = (Comparable) values[1];
        //System.out.println("Comparing: " + v1 + ", " + v2 + ": " + v1.compareTo(v2) + ", " + _type);
        if (v1 == null || v2 == null)
            return null;
        int comparison = v1.compareTo(v2);
        if (comparison < 0) {
            if (_type == TYPE_EQUAL || _type == TYPE_GREATER_EQUAL || _type == TYPE_GREATER_THAN)
                throw new DataHandlerException(_name);
        } else if (comparison == 0) {
            if (_type == TYPE_LESS_THAN || _type == TYPE_GREATER_THAN || _type == TYPE_NOT_EQUAL)
                throw new DataHandlerException(_name);
        } else {
            if (_type == TYPE_LESS_THAN || _type == TYPE_LESS_EQUAL || _type == TYPE_EQUAL)
                throw new DataHandlerException(_name);
        }
        return null;
    }
}