/* 
 * EmailHandler.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.datahandler.handler;

import org.apache.oro.text.regex.Pattern;
import org.apache.oro.text.regex.PatternCompiler;
import org.apache.oro.text.regex.PatternMatcher;
import org.apache.oro.text.regex.Perl5Compiler;
import org.apache.oro.text.regex.Perl5Matcher;

import com.eXtropia.config.Config;
import com.eXtropia.config.ConfigException;
import com.eXtropia.datahandler.DataHandler;
import com.eXtropia.datahandler.DataHandlerException;

/** 
 *
 *
 * 	The following regular expression is used to validate email addresses:
 *   (
 *		^\s*$								#blank
 *		|										#or
 *		^\s*									#some whitespace
 * 		\w{1}								#one word character
 * 		[\w-.=+~!#$%^&\/]* 	#any email symbols
 *		\@									#one @ symbol
 *		[\w-_.]+						# Any word chars, hypehns, _ or .
 *		\.\w{2,3}						# End with a period and 2-3 word chars
 * 		\s*$								# Any amount of whitespace to the end 
 *   )
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 0.1
 */
public class EmailHandler extends DataHandler {
    public static final String CFG_EMAIL_OK = "RequireEmail";
    public static final String NAME = "email";

    private boolean _checkingForEmail = true;
    private Pattern _pattern;
    private PatternMatcher _matcher = new Perl5Matcher();

    public EmailHandler() {
        super(NAME);
        PatternCompiler compiler = new Perl5Compiler();
        try {
            _pattern = compiler.compile("(^\\s*$|^\\s*\\w{1}[\\w-.=+~!#$%^&\\/]*\\@[\\w-_.]+\\.\\w{2,3}\\s*$)");
        } catch (Exception e) {
            throw new RuntimeException("Failed to create pattern to validate email addresses");
        }
    }

    public void configure(Config config) throws ConfigException {
        //System.out.println("HELLO");
        super.configure(config);
        //check to see if we are looking to check not a for an email address!
        if (config.hasChild(CFG_EMAIL_OK)) {
            setCheckForEmail(config.getChild(CFG_EMAIL_OK).getBooleanValue());
        }
    }

    public void setCheckForEmail(boolean checkForEmail) {
        _checkingForEmail = checkForEmail;
    }

    public Object validate(Object[] values) throws DataHandlerException {
        for (int i = 0; i < values.length; i++) {
            if (values[i] != null) {
                boolean isEmail = _matcher.matches(values[i].toString(), _pattern);
                if ((_checkingForEmail && !isEmail) || (!_checkingForEmail && isEmail)) {
                    throw new DataHandlerException(_name, _ids[i], values[i]);
                }
            }
        }
        return null;
    }
}