/* 
 * HtmlHandler.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.datahandler.handler;

import org.apache.oro.text.regex.Pattern;
import org.apache.oro.text.regex.PatternCompiler;
import org.apache.oro.text.regex.PatternMatcher;
import org.apache.oro.text.regex.Perl5Compiler;
import org.apache.oro.text.regex.Perl5Matcher;
import org.apache.oro.text.regex.StringSubstitution;
import org.apache.oro.text.regex.Util;

import com.eXtropia.config.Config;
import com.eXtropia.config.ConfigException;
import com.eXtropia.datahandler.DataHandler;

/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 0.1
 */
public class HtmlHandler extends DataHandler {
    public static final int ACTION_UNSPECIFIED = 0;
    public static final int ACTION_ESCAPE_HTML = 1;

    public static final String CFG_ACTION = "Action";
    public static final String CFG_ESCAPE_HTML = "EscapeHTML";

    private int _action = ACTION_UNSPECIFIED;
    private PatternMatcher _matcher = new Perl5Matcher();
    private Pattern _lessThanPattern;
    private Pattern _greaterThanPattern;

    public HtmlHandler() {
        super("html");
        PatternCompiler compiler = new Perl5Compiler();
        try {
            _lessThanPattern = compiler.compile("<");
            _greaterThanPattern = compiler.compile(">");
        } catch (Exception e) {
            throw new RuntimeException("Failed to create pattern to validate email addresses");
        }
    }

    public void configure(Config config) throws ConfigException {
        super.configure(config);
        Config actionConfig = null;
        actionConfig = config.getChild(CFG_ACTION);
        if (actionConfig == null) {
            throw new ConfigException("No Action specified in the configuration");
        }
        if (actionConfig.getValue().equals(CFG_ESCAPE_HTML)) {
            _action = ACTION_ESCAPE_HTML;
        } else {
            throw new ConfigException("Action specified is not valid");
        }
    }

    public Object validate(Object[] values) {
        int valuesLength = values.length;
        for (int i = 0; i < valuesLength; i++) {
            String strValue;
            switch (_action) {
                case ACTION_ESCAPE_HTML :
                    strValue = values[i].toString();
                    strValue = Util.substitute(_matcher, _lessThanPattern, new StringSubstitution("&lt;"), strValue, Util.SUBSTITUTE_ALL);
                    strValue = Util.substitute(_matcher, _greaterThanPattern, new StringSubstitution("&gt;"), strValue, Util.SUBSTITUTE_ALL);
                    values[i] = strValue;
                case ACTION_UNSPECIFIED :
                    throw new IllegalStateException("Action for transform has not been specified");
                default :
                    }
        }
        return null;
    }
}
