/* 
 * IntegerHandler.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.datahandler.handler;

import com.eXtropia.config.Config;
import com.eXtropia.config.ConfigException;
import com.eXtropia.datahandler.DataHandler;
import com.eXtropia.datahandler.DataHandlerException;

/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 0.1
 */
public class IntegerHandler extends DataHandler {
    public static final String CFG_LOW_BOUND = "LowBound";
    public static final String CFG_HIGH_BOUND = "HighBound";

    public static final String ERROR_TOO_LOW = "LessThan";
    public static final String ERROR_TOO_HIGH = "GreaterThan";
    public static final String ERROR_OUT_OF_BOUNDS = "OutOfBounds";

    public static final String NAME = "integer";

    private static final int CHECK_BOTH = 3;
    private static final int CHECK_LOW = 2;
    private static final int CHECK_HIGH = 1;
    private static final int CHECK_NO_BOUNDS = 0;

    private int _checkBounds = CHECK_NO_BOUNDS;
    private int _lowBound, _highBound;

    public IntegerHandler() {
        super(NAME);
    }

    public void configure(Config config) throws ConfigException {
        super.configure(config);
        try {
            setLowBound(Integer.parseInt(config.getChild(CFG_LOW_BOUND).getValue()));
        } catch (Exception e) {
        }
        try {
            setHighBound(Integer.parseInt(config.getChild(CFG_HIGH_BOUND).getValue()));
        } catch (Exception e) {
        }
    }

    public Object validate(Object[] values) throws DataHandlerException {
        for (int i = 0; i < values.length; i++) {
            if (values[i] != null) {
                try {
                    int myInt = Integer.parseInt(values[i].toString());
                    switch (_checkBounds) {
                        case CHECK_BOTH :
                            if (myInt < _lowBound || myInt > _highBound)
                                throw new DataHandlerException(_name, ERROR_OUT_OF_BOUNDS, _ids[i], values[i]);
                            break;
                        case CHECK_LOW :
                            if (myInt < _lowBound)
                                throw new DataHandlerException(_name, ERROR_TOO_LOW, _ids[i], values[i]);
                            break;
                        case CHECK_HIGH :
                            if (myInt > _highBound)
                                throw new DataHandlerException(_name, ERROR_TOO_HIGH, _ids[i], values[i]);
                            break;
                    }
                    return new Integer(myInt);
                } catch (NumberFormatException nfe) {
                    throw new DataHandlerException(_name, _ids[i], values[i]);
                }
            }
        }
        return null;
    }

    public void setHighBound(int highBound) {
        _highBound = highBound;
        _checkBounds = _checkBounds | CHECK_HIGH;
    }

    public void setLowBound(int lowBound) {
        _lowBound = lowBound;
        _checkBounds = _checkBounds | CHECK_LOW;
    }
}