/*
 * MoneyHandler.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.datahandler.handler;

import java.math.BigDecimal;

import com.eXtropia.config.Config;
import com.eXtropia.config.ConfigException;
import com.eXtropia.datahandler.DataHandler;
import com.eXtropia.datahandler.DataHandlerException;

/**
 * Describe class <code>MoneyHandler</code> here.
 *
 * @author <a href="mailto:st_lim@eXtropia.com">Lim Swee Tat</a>
 * @version $Id: MoneyHandler.java,v 1.4 2002/11/16 08:45:11 rayfd Exp $
 */
public class MoneyHandler extends DataHandler {
    public static final String CFG_LOW_BOUND = "LowBound";
    public static final String CFG_HIGH_BOUND = "HighBound";

    public static final String ERROR_TOO_LOW = "LessThan";
    public static final String ERROR_TOO_HIGH = "GreaterThan";
    public static final String ERROR_OUT_OF_BOUNDS = "OutOfBounds";
    public static final String ERROR_REQUIRES_ROUNDING = "RequiresRounding";

    public static final String NAME = "money";

    private static final int CHECK_BOTH = 3;
    private static final int CHECK_LOW = 2;
    private static final int CHECK_HIGH = 1;
    private static final int CHECK_NO_BOUNDS = 0;

    private int _checkBounds = CHECK_NO_BOUNDS;
    private BigDecimal _lowBound, _highBound;

    /**
     * Creates a new <code>MoneyHandler</code> instance.
     *
     */
    public MoneyHandler() {
        super(NAME);
    }

    /**
     * Describe <code>configure</code> method here.
     *
     * @param config a <code>Config</code> value
     * @exception ConfigException if an error occurs
     */
    public void configure(Config config) throws ConfigException {
        super.configure(config);
        try {
            setLowBound(config.getChild(CFG_LOW_BOUND).getValue());
        } catch (Exception e) {
        }
        try {
            setHighBound(config.getChild(CFG_HIGH_BOUND).getValue());
        } catch (Exception e) {
        }
    }

    public Object validate(Object[] values) throws DataHandlerException {
        for (int i = 0; i < values.length; i++) {
            if (values[i] != null) {
                try {
                    BigDecimal myDec = new BigDecimal(values[i].toString());
                    try {
                        myDec.setScale(2);
                    } catch (ArithmeticException e) {
                        throw new DataHandlerException(_name, ERROR_REQUIRES_ROUNDING, _ids[i], values[i]);
                    }

                    switch (_checkBounds) {
                        case CHECK_BOTH :
                            if ((myDec.max(_highBound)).equals(myDec) || (myDec.min(_lowBound)).equals(myDec))
                                throw new DataHandlerException(_name, ERROR_OUT_OF_BOUNDS, _ids[i], values[i]);
                            break;
                        case CHECK_LOW :
                            if ((myDec.min(_lowBound)).equals(myDec))
                                throw new DataHandlerException(_name, ERROR_TOO_LOW, _ids[i], values[i]);
                            break;
                        case CHECK_HIGH :
                            if ((myDec.max(_highBound)).equals(myDec))
                                throw new DataHandlerException(_name, ERROR_TOO_HIGH, _ids[i], values[i]);
                            break;
                    }
                    return myDec;
                } catch (NumberFormatException nfe) {
                    throw new DataHandlerException(_name, _ids[i], values[i]);
                }
            }
        }
        return null;
    }

    public void setHighBound(String highBound) {
        _highBound = new BigDecimal(highBound);
        _checkBounds = _checkBounds | CHECK_HIGH;
    }

    public void setLowBound(String lowBound) {
        _lowBound = new BigDecimal(lowBound);
        _checkBounds = _checkBounds | CHECK_LOW;
    }
}
