/* 
 * RegularExpressionHandler.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.datahandler.handler;

import org.apache.oro.text.regex.Pattern;
import org.apache.oro.text.regex.PatternCompiler;
import org.apache.oro.text.regex.PatternMatcher;
import org.apache.oro.text.regex.Perl5Compiler;
import org.apache.oro.text.regex.Perl5Matcher;

import com.eXtropia.config.Config;
import com.eXtropia.config.ConfigException;
import com.eXtropia.datahandler.DataHandler;
import com.eXtropia.datahandler.DataHandlerException;

/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */
public class RegularExpressionHandler extends DataHandler {
    public static final String CFG_REGEX = "RegularExpression";

    private Pattern _pattern;
    private PatternMatcher _matcher = new Perl5Matcher();

    public RegularExpressionHandler() {
        super("regex");
    }

    public void configure(Config config) throws ConfigException {
        super.configure(config);

        if (config.hasChild(CFG_REGEX)) {
            try {
                setRegularExpression(config.getChild(CFG_REGEX).getValue());
            } catch (Exception e) {
                throw new ConfigException(e.getMessage());
            }
        } else
            throw new ConfigException("No " + CFG_REGEX + " specified in configuration of " + CFG_BASE + ": " + _name);
    }

    public void setRegularExpression(String regex) throws IllegalArgumentException {
        PatternCompiler compiler = new Perl5Compiler();

        try {
            _pattern = compiler.compile(regex);
        } catch (Exception e) {
            throw new IllegalArgumentException("Failed to create a regular expression from the one supplied (" + regex + "): " + e.getMessage());
        }
    }

    public Object validate(Object[] values) throws DataHandlerException {
        for (int i = 0; i < values.length; i++) {
            if (values[i] != null && !_matcher.matches(values[i].toString(), _pattern))
                throw new DataHandlerException(_name, _ids[i], values[i]);
        }
        return null;
    }
}