/* 
 * StringHandler.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.datahandler.handler;

import com.eXtropia.config.Config;
import com.eXtropia.config.ConfigException;
import com.eXtropia.datahandler.DataHandler;
import com.eXtropia.datahandler.DataHandlerException;

/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */
public class StringHandler extends DataHandler {
    public static final String NAME = "string";
    public static final String ERROR_MAXLENGTH = "maxlength";
    public static final String ERROR_MINLENGTH = "minlength";
    public static final String ERROR_LENGTH = "length";

    public static final String CFG_WHITE_SPACE_NULL = "WhiteSpaceNull";
    public static final String CFG_MAX_LENGTH = "MaxLength";
    public static final String CFG_MIN_LENGTH = "MinLength";
    public static final String CFG_LENGTH = "Length";

    protected boolean _whiteSpaceIsNull = false;
    protected int _maxLength = -1;
    protected int _minLength = -1;
    protected int _length = -1;

    /** Creates new StringHandler */
    public StringHandler() {
        super(NAME);
    }

    public void configure(Config config) throws ConfigException {
        super.configure(config);
        if (config.hasChild(CFG_WHITE_SPACE_NULL))
            _whiteSpaceIsNull = config.getChild(CFG_WHITE_SPACE_NULL).getBooleanValue();

        if (config.hasChild(CFG_MAX_LENGTH))
            _maxLength = config.getChild(CFG_MAX_LENGTH).getIntValue();

        if (config.hasChild(CFG_MIN_LENGTH))
            _minLength = config.getChild(CFG_MIN_LENGTH).getIntValue();
    }

    public Object validate(Object[] values) throws DataHandlerException {
        String str;
        for (int i = 0; i < values.length; i++) {
            //System.out.println("Validating: [" + values[i] + "]");
            if (values[i] == null)
                continue;

            str = values[i].toString();
            if (_whiteSpaceIsNull) {
                str = str.trim();
                if (str.length() == 0) {
                    values[i] = null;
                    continue;
                }
            }
            if (_maxLength > -1 && str.length() > _maxLength)
                throw new DataHandlerException(_name, ERROR_MAXLENGTH, _ids[i], str);

            if (_minLength > -1 && str.length() < _minLength)
                throw new DataHandlerException(_name, ERROR_MINLENGTH, _ids[i], str);

            if (_length > -1 && str.length() != _length)
                throw new DataHandlerException(_name, ERROR_LENGTH, _ids[i], str);
        }
        return null;
    }
}