/* 
 * WordHandler.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.datahandler.handler;

import java.util.HashSet;

import org.apache.oro.text.regex.Pattern;
import org.apache.oro.text.regex.PatternCompiler;
import org.apache.oro.text.regex.PatternMatcher;
import org.apache.oro.text.regex.Perl5Compiler;
import org.apache.oro.text.regex.Perl5Matcher;

import com.eXtropia.config.Config;
import com.eXtropia.config.ConfigException;
import com.eXtropia.datahandler.DataHandler;
import com.eXtropia.datahandler.DataHandlerException;

/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 0.1
 */
public class WordHandler extends DataHandler {
    public static final String CFG_LIST = "List";
    public static final String CFG_WORD_IN_LIST = "InList"; // rather than not in list
    public static final String CFG_LIST_ITEM = "ListItem";
    public static final String CFG_LIST_IGNORE_CASE = "IgnoreCase";

    public static final String ERROR_NOT_IN_SET = "NotInSet";

    private boolean _useWordSet = false;
    private boolean _inWordSet = true;
    private boolean _ignoreCase = false;
    private HashSet _originalWordSet = new HashSet();
    private HashSet _caseInsensitiveWordSet = new HashSet();
    private HashSet _wordSetToUse = (_ignoreCase ? _caseInsensitiveWordSet : _originalWordSet);

    private Pattern _pattern;
    private PatternMatcher _matcher = new Perl5Matcher();

    public WordHandler() {
        super("word");
        PatternCompiler compiler = new Perl5Compiler();
        try {
            _pattern = compiler.compile("(^\\s*$|^\\s*[\\w-]+\\s*$)");
        } catch (Exception e) {
        }
    }

    public void configure(Config config) throws ConfigException {
        super.configure(config);
        try {
            Config listConfig = config.getChild(CFG_LIST);
            //gather up the list if we have one and set it
            Config[] listChildren = listConfig.getChildren();
            for (int i = 0; i < listChildren.length; i++) {
                if (listChildren[i].getName().equals(CFG_LIST_ITEM)) {
                    _originalWordSet.add(listChildren[i].getValue());
                    _caseInsensitiveWordSet.add(listChildren[i].getValue().toLowerCase());
                }
            }
            if (!_wordSetToUse.isEmpty()) {
                _useWordSet = true;
                try {
                    _inWordSet = Boolean.valueOf(listConfig.getChild(CFG_WORD_IN_LIST).getValue()).booleanValue();
                } catch (Exception e) {
                } // ah well, use the default value
                try {
                    _ignoreCase = Boolean.valueOf(listConfig.getChild(CFG_LIST_IGNORE_CASE).getValue()).booleanValue();
                    _wordSetToUse = (_ignoreCase ? _caseInsensitiveWordSet : _originalWordSet);
                } catch (Exception e) {
                } // ah well, use default value
            }
        } catch (Exception e) {
        } //ah well, no list specified
    }

    public Object validate(Object[] values) throws DataHandlerException {
        if (values[0] == null) {
            return null;
        }
        if (!_matcher.matches(values[0].toString(), _pattern)) {
            throw new DataHandlerException(_name, _ids[0], values[0]);
        } else if (_useWordSet && !values.toString().trim().equals("")) {
            values[0] = _ignoreCase ? values[0].toString().toLowerCase() : values[0];
            if ((_inWordSet && !_wordSetToUse.contains(values[0])) || (!_inWordSet && _wordSetToUse.contains(values[0]))) {
                throw new DataHandlerException(_name, ERROR_NOT_IN_SET, _ids[0], values[0]);
            }
        }
        return null;
    }
}