/* 
 * Criterion.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.datasource;

import java.util.LinkedList;

/** 
 *
 * @author Carlo Ravagli
 * @version 0.1
 */
public class Criterion {
    public static final int AND = 0;
    public static final int OR = 1;

    protected int _type;
    protected LinkedList _conditions = new LinkedList();
    protected LinkedList _subCriteria = new LinkedList();

    /** Creates new Criterion */
    public Criterion() {
        _type = AND;
    }

    public Criterion(int type) {
        _type = type;
    }

    public Criterion(Condition condition) {
        this();
        add(condition);
    }

    public Criterion(Condition[] conditions) {
        this();
        add(conditions);
    }

    public Criterion(Criterion subCriterion) {
        this();
        add(subCriterion);
    }

    public Criterion(Criterion[] subCriteria) {
        this();
        add(subCriteria);
    }

    public Criterion(Condition condition, int type) {
        this(type);
        add(condition);
    }

    public Criterion(Condition[] conditions, int type) {
        this(type);
        add(conditions);
    }

    public Criterion(Criterion subCriterion, int type) {
        this(type);
        add(subCriterion);
    }

    public Criterion(Criterion[] subCriteria, int type) {
        this(type);
        add(subCriteria);
    }

    public void setType(int type) {
        _type = type;
    }

    public int getType() {
        return _type;
    }

    public void add(Condition condition) {
        if (condition != null)
            _conditions.add(condition);
    }

    public void add(Condition[] conditions) {
        if (conditions != null);
        {
            for (int i = 0; i < conditions.length; i++) {
                if (conditions[i] != null)
                    _conditions.add(conditions[i]);
            }
        }
    }

    public Condition[] getConditions() {
        return _conditions.size() > 0 ? (Condition[]) _conditions.toArray(new Condition[_conditions.size()]) : null;
    }

    public void setConditions(Condition[] conditions) {
        _conditions.clear();
        add(conditions);
    }

    public void add(Criterion subCriterion) {
        if (subCriterion != null)
            _subCriteria.add(subCriterion);
    }

    public void add(Criterion[] subCriteria) {
        if (subCriteria != null);
        {
            for (int i = 0; i < subCriteria.length; i++) {
                if (subCriteria[i] != null)
                    _subCriteria.add(subCriteria[i]);
            }
        }
    }

    public Criterion[] getSubCriteria() {
        return _subCriteria.size() > 0 ? (Criterion[]) _subCriteria.toArray(new Criterion[_subCriteria.size()]) : null;
    }

    public void setSubCriteria(Criterion[] subCriteria) {
        _subCriteria.clear();
        add(subCriteria);
    }
}