/* 
 * DataSource.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.datasource;

import java.util.Map;

import javax.sql.RowSet;

import com.eXtropia.config.Config;
import com.eXtropia.config.ConfigException;
import com.eXtropia.config.Configurable;

/** 
 *
 * @author Carlo Ravagli
 * @version 1.0
 */
public abstract class DataSource implements Configurable {
    public static final String CFG_BASE = "DataSource";
    //public abstract void configure( Config config ) throws ConfigException;

    public abstract int add(Map rowValues) throws DataSourceException;

    public abstract int delete() throws DataSourceException;
    public abstract int delete(Condition condition) throws DataSourceException;
    public abstract int delete(Criterion criterion) throws DataSourceException;

    public abstract int update(Map rowValues, Condition condition) throws DataSourceException;
    public abstract int update(Map rowValues, Criterion criterion) throws DataSourceException;

    public abstract RowSet search(Query query) throws DataSourceException;

    public abstract int count() throws DataSourceException;
    public abstract int count(Condition condition) throws DataSourceException;
    public abstract int count(Criterion criterion) throws DataSourceException;

    public abstract String[] getKeyFields() throws DataSourceException;
    public abstract String[] getFields() throws DataSourceException;

    public abstract Criterion generateCriterion(String keyword) throws DataSourceException;
    public abstract Criterion generateCriterion(String keyword, String[] searchFields) throws DataSourceException;

    public abstract Criterion generateCriterion(String[] keywords, int criterionType) throws DataSourceException;
    public abstract Criterion generateCriterion(String[] keywords, int criterionType, String[] searchFields) throws DataSourceException;

    /*public static final DataSource create(Config config) throws ConfigException
    {
    	if( !config.getName().equals(CFG_BASE) )
    		throw new ConfigException("Cannot create a DataSource with configuartion: " + config.getName() );
    	
    	try
    	{
    		DataSource ds = (DataSource)Class.forName(config.getType()).newInstance();
    		ds.configure( config );
    		return ds;
    	}
    	catch( ClassCastException cce )
    	{
    		throw new ConfigException( "Unable to create DataSource. Invalid DataSource type: " + config.getType() );
    	}
    	catch( InstantiationException ie )
    	{
    		throw new ConfigException( "Failed to instantiate the specified DataSource object: " + config.getType());
    	}
    	catch( IllegalAccessException iae )
    	{
    		throw new ConfigException( "Illegal access to specified DataSource class: " + config.getType());
    	}
    	catch( ClassNotFoundException cnfe )
    	{
    		throw new ConfigException( "Unable to find specified DataSource class: " + config.getType());
    	}
    }*/
    public void configure(Config config) throws ConfigException {
    }
}