/* 
 * Query.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.datasource;

/** 
 *
 * @author  Carlo Ravagli (carlo.ravagli@extropia.com)
 * @version 0.1
 */
public class Query extends Object {
    private Criterion _criterion;
    private FieldOrder[] _orders;
    private String[] _returnFields;
    private int _offset;
    private int _limit;
    private boolean _isDistinct;

    /** Creates new Query */
    public Query() {
        _criterion = null;
        _orders = null;
        _returnFields = null;
        _offset = 0;
        _limit = 0;
        _isDistinct = false;
    }

    public Query(Criterion criterion) {
        this();
        setCriterion(criterion);
    }

    public Query(Condition condition) {
        this(new Criterion(condition));
    }

    /**
     * returns the distinct status of the query
     * @return true if rows returned should be distinct, false otherwise
     */
    public boolean isDistinct() {
        return _isDistinct;
    }

    /**
     * sets whether all the rows returned by this
     * query should be distinct or not
     * @param distinct true if distinct, false otherwise
     */
    public void setDistinct(boolean distinct) {
        _isDistinct = distinct;
    }

    public void setCriterion(Criterion criterion) {
        _criterion = criterion;
    }

    public Criterion getCriterion() {
        return _criterion;
    }

    public void setFieldOrder(FieldOrder[] order) {
        _orders = order;
    }

    public FieldOrder[] getFieldOrder() {
        return _orders;
    }

    public void setReturnFields(String[] returnFields) {
        _returnFields = returnFields != null && returnFields.length > 0 ? returnFields : null;
    }

    public String[] getReturnFileds() {
        return _returnFields;
    }

    public void setOffset(int offset) throws IllegalArgumentException {
        if (offset < 0)
            throw new IllegalArgumentException();

        _offset = offset;
    }

    public int getOffset() {
        return _offset;
    }

    public void setLimit(int limit) throws IllegalArgumentException {
        if (limit < 0)
            throw new IllegalArgumentException();
        _limit = limit;
    }

    public int getLimit() {
        return _limit;
    }
}
