/* 
 * DiagnosticsEventQueue.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.diagnostics;

import java.util.LinkedList;

/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 0.1
 */
public class DiagnosticsEventQueue implements Runnable {
    private LinkedList _queue;
    private boolean _logInline;
    private boolean _traceInline;

    /** Creates new DiagnosticsEventQueue */
    public DiagnosticsEventQueue(boolean logInline, boolean traceInline) {
        _queue = new LinkedList();
        _logInline = logInline;
        _traceInline = traceInline;
        if (!(_logInline && _traceInline)) {
            Thread t = new Thread(this);
            t.setPriority(Thread.MIN_PRIORITY);
            t.setDaemon(true);
            t.start();
        }
    }

    public void run() {
        try {
            _queue.wait();
        } catch (Exception e) {
        }
        DiagnosticsEvent event;
        //while there are events fire them
        while (true) {
            try {
                synchronized (_queue) {
                    if (_queue.isEmpty()) {
                        _queue.wait();
                    }
                    event = (DiagnosticsEvent) _queue.removeFirst();
                }
                if (event._type == DiagnosticsEvent.LOG) {
                    event.getDoctor().handleEvent((LogEvent) event);
                } else {
                    event.getDoctor().handleEvent((TraceEvent) event);
                }
            } catch (Exception e) {
            }
        }
    }

    public void addEvent(LogEvent event) {
        if (_logInline) {
            event.getDoctor().handleEvent(event);
        } else {
            addToQueue(event);
        }
    }

    public void addEvent(TraceEvent event) {
        if (_traceInline) {
            event.getDoctor().handleEvent(event);
        } else {
            addToQueue(event);
        }
    }

    private void addToQueue(DiagnosticsEvent event) {
        synchronized (_queue) {
            _queue.addLast(event);
            if (_queue.size() == 1) {
                _queue.notifyAll();
            }
        }
    }
}