/* 
 * EmailListener.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.diagnostics;

import java.util.Vector;

import com.eXtropia.config.Config;
import com.eXtropia.config.ConfigException;
import com.eXtropia.mail.Mailer;
import com.eXtropia.mail.Message;

/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 0.1
 */
public class EmailListener extends ListenerSupport implements LogListener {
    public static final String CFG_RECIPIENT = "Recipient";
    public static final String CFG_FROM = "From";

    private static final short TYPE_PARA = 1;
    private static final short LEVEL_PARA = 2;
    private static final short MESSAGE_PARA = 3;

    private String[] _contentTemplate = new String[] { "The following alert has been raised: ", "Type: ", "Level: ", "Message: ", "Please investigate as soon as possible" };
    private String[] _recipients;
    private String _from;

    /** Creates new EmailListener */
    public EmailListener() {
    }

    public void configure(Config config) throws ConfigException {
        super.configure(config);
        //need to get a list of email addresses that we send the mail to
        Vector recipients = new Vector();
        Config[] children = config.getChildren();
        for (int i = 0; i < children.length; i++) {
            if (children[i].getName().equals(CFG_RECIPIENT)) {
                recipients.addElement(children[i].getValue());
            } else if (children[i].getName().equals(CFG_FROM)) {
                if (_from == null) {
                    _from = children[i].getValue();
                } else {
                    throw new ConfigException("EmailListener: Can't specify more than one '" + CFG_FROM + "' email address");
                }
            }
        }
        ConfigException ce = new ConfigException("EmailListener: Insufficient parameters supplied");
        if (_from == null) {
            ce.addChild(new ConfigException("No from email address specified"));
        }
        if (recipients.size() < 1) {
            ce.addChild(new ConfigException("No recipients specified"));
        } else {
            _recipients = new String[recipients.size()];
            recipients.copyInto(_recipients);
        }
        if (ce.hasChildren()) {
            throw ce;
        }
    }

    public void log(LogEvent logEvent) {
        if (okLogLevel(logEvent.getLevel())) {
            Message message = new Message();
            message.setFrom(_from);
            message.setRecipients(_recipients, Message.TO_RECIPIENT);
            message.setSubject("ALERT: " + logEvent.getLevelName() + ": " + logEvent.getFilterString() + " " + logEvent.getMessage());
            String[] myContent = (String[]) _contentTemplate.clone();
            myContent[TYPE_PARA] += logEvent.getFilterString();
            myContent[LEVEL_PARA] += logEvent.getLevelName();
            myContent[MESSAGE_PARA] += logEvent.getMessage();
            message.setContent(myContent);
            try {
                Mailer.send(message);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }
}