/* 
 * FileListener.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.diagnostics;

import java.io.FileWriter;
import java.io.PrintWriter;

import com.eXtropia.config.Config;
import com.eXtropia.config.ConfigException;

/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 0.1
 */
public class FileListener extends ListenerSupport implements LogListener, TraceListener {
    public static final String CFG_FILENAME = "File";
    //public static final String CFG_WRITE_LOG_HEADER = "WriteLogHeader";
    public static final int FILE_WRITE_RETRIES = 5;

    private PrintWriter _out;
    private String _fileName;
    //private boolean _writeLogHeader = true;

    private int retries = 0;
    public FileListener() {

    }

    public void configure(Config config) throws ConfigException {
        //get out the file name
        super.configure(config);
        _fileName = config.getChild(CFG_FILENAME).getValue();
        // GB: Added useful feature to write message straight to the log without
        // extra hdr info
        //String writeLogHeader = config.getChild(CFG_WRITE_LOG_HEADER).getValue();
        //if (writeLogHeader != null) {
        //	_writeLogHeader = (new Boolean(writeLogHeader)).booleanValue();
        try {
            if (!tryFileAgain()) {
                throw new Exception();
            }
        } catch (Exception e) {
            throw new ConfigException("Failed to open specified output file: " + _fileName);
        }
    }

    public void log(LogEvent logEvent) {
        if (okLogLevel(logEvent.getLevel())) {
            _out.println(formatMessage(logEvent.getMessage(), logEvent.getLevelName(), logEvent.getMap()));
            if (_out.checkError() && tryFileAgain())
                _out.println(formatMessage(logEvent.getMessage(), logEvent.getLevelName(), logEvent.getMap()));
        }
    }

    public void trace(TraceEvent traceEvent) {
        _out.println(formatMessage(traceEvent.getMessage(), traceEvent.getLevelName(), null));
        if (traceEvent.getException() != null)
            traceEvent.getException().printStackTrace(_out);

        if (_out.checkError() && tryFileAgain()) {
            _out.println(formatMessage(traceEvent.getMessage(), traceEvent.getLevelName(), null));
            if (traceEvent.getException() != null)
                traceEvent.getException().printStackTrace(_out);
        }
    }

    private boolean tryFileAgain() {
        try {
            if (retries < FILE_WRITE_RETRIES) {
                _out = new PrintWriter(new FileWriter(_fileName, true));
                retries++;
            }
            return true;
        } catch (Exception e2) {
            return false;
        }
    }

    protected void finalize() {
        try {
            _out.close();
        } catch (Exception e) {
        }
    }
}