/* 
 * LevelBoundListener.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.diagnostics;

import com.eXtropia.config.Config;
import com.eXtropia.config.ConfigException;

/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 0.1
 */
public abstract class LevelBoundListener {
    public static final String CFG_MIN_LEVEL = "MinLevel";
    public static final String CFG_MAX_LEVEL = "MaxLevel";

    protected short _traceMinLevel = TraceEvent.MIN_LEVEL;
    protected short _traceMaxLevel = TraceEvent.MAX_LEVEL;
    protected short _logMinLevel = LogEvent.MIN_LEVEL;
    protected short _logMaxLevel = LogEvent.MIN_LEVEL;
    /** Creates new LevelBoundListener */
    public LevelBoundListener() {

    }

    public void configure(Config config) throws ConfigException {
        //do trace
        try {
            _traceMinLevel = selectShortValue(config, CFG_MIN_LEVEL, TraceEvent.LEVEL_NAMES, TraceEvent.MIN_LEVEL);
            _traceMaxLevel = selectShortValue(config, CFG_MAX_LEVEL, TraceEvent.LEVEL_NAMES, TraceEvent.MAX_LEVEL);
        } catch (Exception e) {
            _traceMinLevel = TraceEvent.MIN_LEVEL;
            _traceMaxLevel = TraceEvent.MAX_LEVEL;
        }

        //do log
        try {
            _logMinLevel = selectShortValue(config, CFG_MIN_LEVEL, LogEvent.LEVEL_NAMES, LogEvent.MIN_LEVEL);
            _logMaxLevel = selectShortValue(config, CFG_MAX_LEVEL, LogEvent.LEVEL_NAMES, LogEvent.MAX_LEVEL);
        } catch (Exception e) {
            _logMinLevel = LogEvent.MIN_LEVEL;
            _logMaxLevel = LogEvent.MAX_LEVEL;
        }
    }

    private short selectShortValue(Config config, String selector, String[] values, short defaultValue) {
        try {
            String value = config.getChild(selector).getValue();
            if (value == null)
                return defaultValue;
            for (short i = 0; i < values.length; i++) {
                if (value.equals(values[i])) {
                    return i;
                }
            }
        } catch (Exception e) {
        }
        return defaultValue;
    }

    public void setTraceMaxLevel(short level) {
        _traceMaxLevel = level;
    }

    public void setTraceMinLevel(short level) {
        _traceMinLevel = level;
    }

    public void setLogMaxLevel(short level) {
        _logMaxLevel = level;
    }

    public void setLogMinLevel(short level) {
        _logMinLevel = level;
    }

    protected boolean okLogLevel(short level) {
        return level >= _logMinLevel && level <= _logMaxLevel;
    }

    protected boolean okTraceLevel(short level) {
        return level >= _traceMinLevel && level <= _traceMaxLevel;
    }

}