/*
 *  SmsListener.java
 *  Copyright (C) 2000 - 2001  eXtropia.com
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.diagnostics;

import com.eXtropia.config.Config;
import com.eXtropia.config.ConfigException;
import com.eXtropia.sms.Sms;
import com.eXtropia.sms.dispatcher.SmsDispatchException;
import com.eXtropia.sms.dispatcher.SmsDispatcherManager;

/**
 * @author     Carlo Ravagli (carlo.ravagli@eXtropia.com)
 * @created    June 19, 2002
 * @version    0.1
 */
public class SmsListener extends LevelBoundListener implements LogListener {
    /**  Description of the Field */
    public final static String CFG_RECIPIENTS = "Recipients";
    /**  Description of the Field */
    public final static String CFG_RECIPIENT = "Recipient";
    /**  Description of the Field */
    public final static String CFG_FROM = "From";

    private boolean _configured = false;
    private String[] _recipients;
    private String _from;

    private SmsDispatcherManager _dispatcher;

    /**  Creates new EmailListener */
    public SmsListener() {
    }

    /**
     *  Description of the Method
     *
     * @param  config               Description of the Parameter
     * @exception  ConfigException  Description of the Exception
     */
    public void configure(Config config) throws ConfigException {
        if (_configured) {
            throw new ConfigException("Already configured");
        }

        if (!config.getName().equals(Diagnostics.CFG_LOGLISTENER)) {
            throw new ConfigException("Invalid config specfied - " + config.getName());
        }

        super.configure(config);

        Config childConfig = config.getChild(CFG_RECIPIENTS);
        if (childConfig == null) {
            throw new ConfigException("Failed to configure SmsListener: " + CFG_RECIPIENTS + " not specified");
        }

        Config[] recipientsConfig = childConfig.getChildren();
        if (recipientsConfig == null || recipientsConfig.length == 0) {
            throw new ConfigException("No recipients specifed for SmsListener");
        }

        _recipients = new String[recipientsConfig.length];

        for (int i = 0; i < recipientsConfig.length; i++) {
            if (!recipientsConfig[i].getName().equals(CFG_RECIPIENT) || recipientsConfig[i].getValue() == null) {
                throw new ConfigException("Invalid Sms recipient config: " + recipientsConfig[i].getName());
            }
            _recipients[i] = recipientsConfig[i].getValue();
        }

        childConfig = config.getChild(CFG_FROM);
        if (childConfig != null && childConfig.getValue() != null) {
            _from = childConfig.getValue();
        }

        childConfig = config.getChild(SmsDispatcherManager.CFG_BASE);
        if (childConfig == null) {
            throw new ConfigException("No SMS dispatcher manager config specifed");
        }

        _dispatcher = SmsDispatcherManager.create(childConfig);
    }

    /**
     *  Description of the Method
     *
     * @param  logEvent  Description of the Parameter
     */
    public void log(LogEvent logEvent) {
        if (okLogLevel(logEvent.getLevel())) {
            Sms msg = new Sms();
            msg.setFrom(_from);
            msg.setText(logEvent.getLevelName() + " ALERT: " + logEvent.getMessage());
            for (int i = 0; i < _recipients.length; i++) {
                msg.setTo(_recipients[i]);
                try {
                    _dispatcher.send(msg);
                } catch (SmsDispatchException e) {
                    e.printStackTrace();
                }
            }
        }
    }
}
