/*
 * MessageTemplate.java
 *
 * Created on October 3, 2000, 11:30 AM
 */

package com.eXtropia.sms;

import java.text.MessageFormat;
import java.text.ParseException;
import java.util.Map;
import java.util.Vector;

import com.eXtropia.config.Config;
import com.eXtropia.config.ConfigException;
import com.eXtropia.config.Configurable;

/** 
 *
 * @author Carlo Ravagli
 * @version 1.0
 */
public class TemplatedSms extends Sms implements Configurable {
    public static final String CFG_MSG_PATTERN = "MsgPattern";
    public static final String CFG_MSG_TYPE = "MsgType";

    // Can't extend MessageFormat cos it has final format method
    // that is called from base class Format therefore I
    // have wrap it instead.

    private String _name;
    private MessageFormat _mf;
    private String[] _mappings;

    /** Creates new MessageTemplate */
    public TemplatedSms() {

    }

    public String getName() {
        return _name;
    }

    public void setName(String name) {
        _name = name;
    }

    public synchronized void configure(Config config) throws ConfigException {
        if (_mf != null)
            throw new ConfigException("Already configured");

        _name = config.getValue();

        Config typeConfig = config.getChild(CFG_MSG_TYPE);
        if (typeConfig != null && typeConfig.getValue() != null)
            setType(Integer.parseInt(typeConfig.getValue()));

        Config formatConfig = config.getChild(CFG_MSG_PATTERN);

        if (formatConfig == null || formatConfig.getValue() == null)
            throw new ConfigException("Message pattern not specified for template");

        String pattern = formatConfig.getValue();
        StringBuffer formatPattern = new StringBuffer();
        Vector mappings = new Vector();

        for (int i = 0; i < pattern.length(); i++) {
            formatPattern.append(pattern.charAt(i));
            if (pattern.charAt(i) == '{' && (i == 0 || pattern.charAt(i - 1) != '\'')) {
                StringBuffer argName = new StringBuffer();
                while (pattern.charAt(i + 1) != '}' && pattern.charAt(i + 1) != ',')
                    argName.append(pattern.charAt(++i));

                int mappingIndex = mappings.indexOf(argName.toString());
                if (mappingIndex < 0) {
                    formatPattern.append(mappings.size());
                    mappings.add(argName.toString());
                } else
                    formatPattern.append(mappingIndex);

            }
        }

        _mappings = new String[mappings.size()];
        mappings.copyInto(_mappings);
        _mf = new MessageFormat(formatPattern.toString());
    }

    public void setText(Map args) throws ParseException {
        if (_mf == null)
            throw new IllegalStateException("Failed to genterate message from template: Template not configured");

        Object[] argsArray = new Object[_mappings.length];

        for (int i = 0; i < _mappings.length; i++) {
            argsArray[i] = args.get(_mappings[i]);
            if (argsArray[i] == null)
                throw new IllegalArgumentException("Failed to generate msg from template: arg missing - " + _mappings[i]);
        }

        setText(_mf.format(argsArray));
    }
}
