/*
 *  SmsDispatchWrapper.java
 *  Copyright (C) 2000 - 2001  eXtropia.com
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.sms.dispatcher;

import java.util.HashMap;
import java.util.Map;

import com.eXtropia.config.Config;
import com.eXtropia.config.ConfigException;
import com.eXtropia.config.Configurable;
import com.eXtropia.sms.Sms;
import com.eXtropia.sms.TemplatedSms;
import com.eXtropia.util.ExtropiaException;

/**
 *  Description of the Class
 *
 * @author     st_lim
 * @created    June 19, 2002
 * @version    $Id: SmsDispatchWrapper.java,v 1.8 2002/11/16 08:45:13 rayfd Exp $
 */
public class SmsDispatchWrapper implements Configurable {
    /**  Description of the Field */
    public final static String CFG_BASE = "SmsDispatcher";
    /**  Description of the Field */
    public final static String CFG_TEMPLATES = "Templates";
    /**  Description of the Field */
    public final static String CFG_TEMPLATE = "Template";

    private HashMap _templates = new HashMap();
    private SmsDispatcherManager _dispatcher;
    private boolean _configured;

    /**  Creates new MessageDispatcher */
    public SmsDispatchWrapper() {
    }

    /**
     *  Description of the Method
     *
     * @param  config               Description of the Parameter
     * @exception  ConfigException  Description of the Exception
     */
    public synchronized void configure(Config config) throws ConfigException {
        if (_configured) {
            throw new ConfigException("Already configured");
        }

        if (!config.getName().equals(CFG_BASE)) {
            throw new ConfigException("Invalid config specfied - " + config.getName());
        }

        Config dispatcherConfig = config.getChild(SmsDispatcherManager.CFG_BASE);
        if (dispatcherConfig == null) {
            throw new ConfigException("No SMS dispatcher manager config specifed");
        }

        _dispatcher = SmsDispatcherManager.create(dispatcherConfig);

        Config templateConfig = config.getChild(CFG_TEMPLATES);
        if (templateConfig != null) {
            Config[] templatesConfig = templateConfig.getChildren();

            for (int i = 0; templatesConfig != null && i < templatesConfig.length; i++) {
                TemplatedSms sms = new TemplatedSms();
                sms.configure(templatesConfig[i]);
                _templates.put(sms.getName(), sms);
            }
        }
        _configured = true;
    }

    /**
     *  Description of the Method
     *
     * @param  to                        Description of the Parameter
     * @param  msg                       Description of the Parameter
     * @exception  SmsDispatchException  Description of the Exception
     */
    public void send(String to, String msg) throws SmsDispatchException {
        send("0", to, msg, Sms.TYPE_NONE);
    }

    /**
     *  Description of the Method
     *
     * @param  to                        Description of the Parameter
     * @param  msg                       Description of the Parameter
     * @exception  SmsDispatchException  Description of the Exception
     */
    public void send(String[] to, String msg) throws SmsDispatchException {
        send("0", to, msg, Sms.TYPE_NONE);
    }

    /**
     *  Description of the Method
     *
     * @param  to                        Description of the Parameter
     * @param  msg                       Description of the Parameter
     * @param  type                      Description of the Parameter
     * @exception  SmsDispatchException  Description of the Exception
     */
    public void send(String to, String msg, int type) throws SmsDispatchException {
        send("0", to, msg, type);
    }

    /**
     *  Description of the Method
     *
     * @param  to                        Description of the Parameter
     * @param  msg                       Description of the Parameter
     * @param  type                      Description of the Parameter
     * @exception  SmsDispatchException  Description of the Exception
     */
    public void send(String[] to, String msg, int type) throws SmsDispatchException {
        send("0", to, msg, type);
    }

    /**
     *  Description of the Method
     *
     * @param  to                        Description of the Parameter
     * @param  template                  Description of the Parameter
     * @param  args                      Description of the Parameter
     * @exception  SmsDispatchException  Description of the Exception
     */
    public void send(String to, String template, Map args) throws SmsDispatchException {
        send("0", to, template, args);
    }

    /**
     *  Description of the Method
     *
     * @param  to                        Description of the Parameter
     * @param  template                  Description of the Parameter
     * @param  args                      Description of the Parameter
     * @exception  SmsDispatchException  Description of the Exception
     */
    public void send(String[] to, String template, Map args) throws SmsDispatchException {
        send("0", to, template, args);
    }

    /**
     *  Description of the Method
     *
     * @param  from                      Description of the Parameter
     * @param  to                        Description of the Parameter
     * @param  msg                       Description of the Parameter
     * @exception  SmsDispatchException  Description of the Exception
     */
    public void send(String from, String to, String msg) throws SmsDispatchException {
        send(from, to, msg, Sms.TYPE_NONE);
    }

    /**
     *  Description of the Method
     *
     * @param  from                      Description of the Parameter
     * @param  to                        Description of the Parameter
     * @param  msg                       Description of the Parameter
     * @exception  SmsDispatchException  Description of the Exception
     */
    public void send(String from, String[] to, String msg) throws SmsDispatchException {
        send(from, to, msg, Sms.TYPE_NONE);
    }

    /**
     *  Description of the Method
     *
     * @param  from                      Description of the Parameter
     * @param  to                        Description of the Parameter
     * @param  msg                       Description of the Parameter
     * @param  type                      Description of the Parameter
     * @exception  SmsDispatchException  Description of the Exception
     */
    public void send(String from, String to, String msg, int type) throws SmsDispatchException {
        _dispatcher.send(new Sms(to, from, msg, type));
    }

    /**
     *  Description of the Method
     *
     * @param  from                      Description of the Parameter
     * @param  to                        Description of the Parameter
     * @param  msg                       Description of the Parameter
     * @param  type                      Description of the Parameter
     * @exception  SmsDispatchException  Description of the Exception
     */
    public void send(String from, String[] to, String msg, int type) throws SmsDispatchException {
        SmsDispatchException exception = null;
        for (int i = 0; i < to.length; i++) {
            try {
                send(from, to[i], msg, type);
            } catch (ExtropiaException e) {
                if (exception == null) {
                    exception = new SmsDispatchException("Failed to send sms to all recipients");
                }
                exception.addChild(e);
            }
        }
        if (exception != null) {
            throw exception;
        }
    }

    /**
     *  Description of the Method
     *
     * @param  from                      Description of the Parameter
     * @param  to                        Description of the Parameter
     * @param  template                  Description of the Parameter
     * @param  args                      Description of the Parameter
     * @exception  SmsDispatchException  Description of the Exception
     */
    public void send(String from, String to, String template, Map args) throws SmsDispatchException {
        TemplatedSms sms = (TemplatedSms) _templates.get(template);
        if (sms == null) {
            throw new SmsDispatchException("Failed to send sms: invalid template - " + template);
        }

        try {
            sms = (TemplatedSms) sms.clone();
            sms.setFrom(from);
            sms.setTo(to);
            sms.setText(args);
            _dispatcher.send(sms);
        } catch (ExtropiaException e) {
            throw new SmsDispatchException("Failed to send sms", e);
        } catch (Exception e) {
            throw new SmsDispatchException("Failed to genterate sms from " + template + " template: " + e.getMessage());
        }
    }

    /**
     *  Description of the Method
     *
     * @param  from                      Description of the Parameter
     * @param  to                        Description of the Parameter
     * @param  template                  Description of the Parameter
     * @param  args                      Description of the Parameter
     * @exception  SmsDispatchException  Description of the Exception
     */
    public void send(String from, String[] to, String template, Map args) throws SmsDispatchException {
        TemplatedSms sms = (TemplatedSms) _templates.get(template);
        if (sms == null) {
            throw new SmsDispatchException("Failed to send sms: invalid template - " + template);
        }

        try {
            sms = (TemplatedSms) sms.clone();
            sms.setFrom(from);
            sms.setText(args);
        } catch (Exception e) {
            throw new SmsDispatchException("Failed to genterate sms from " + template + " template: " + e.getMessage());
        }

        SmsDispatchException exception = null;
        for (int i = 0; i < to.length; i++) {
            try {
                sms.setTo(to[i]);
                _dispatcher.send(sms);
            } catch (ExtropiaException e) {
                if (exception == null) {
                    exception = new SmsDispatchException("Failed to send sms to all recipients");
                }
                exception.addChild(e);
            } catch (Exception e) {
                if (exception == null) {
                    exception = new SmsDispatchException("Failed to send sms to all recipients");
                }
                exception.addChild(new SmsDispatchException(e.getClass().getName() + ": " + e.getMessage()));
            }
        }
        if (exception != null) {
            throw exception;
        }
    }
}
