/*
 * SoapSmsDispatcher.java
 * Copyright (C) 2000  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.sms.dispatcher;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.Iterator;
import java.util.Vector;

import org.apache.soap.Constants;
import org.apache.soap.Fault;
import org.apache.soap.SOAPException;
import org.apache.soap.encoding.SOAPMappingRegistry;
import org.apache.soap.encoding.soapenc.BeanSerializer;
import org.apache.soap.rpc.Call;
import org.apache.soap.rpc.Parameter;
import org.apache.soap.rpc.Response;
import org.apache.soap.transport.http.SOAPHTTPConnection;
import org.apache.soap.util.xml.Deserializer;
import org.apache.soap.util.xml.QName;
import org.apache.soap.util.xml.Serializer;

import com.eXtropia.config.Config;
import com.eXtropia.config.ConfigException;
import com.eXtropia.sms.Sms;
import com.eXtropia.soap.encoding.ExceptionSerializer;

/**
 * Class SoapSmsDispatcher
 * This is basically the SoapSmsDispatcherManager, but this is now just a dispatcher.
 * 8)
 *
 * @author <a href="st_lim@eXtropia.com">Lim Swee Tat</a>
 * @version $Id: SoapSmsDispatcher.java,v 1.4 2002/11/16 08:45:13 rayfd Exp $
 */
public class SoapSmsDispatcher implements SmsDispatcher {
    public static final String CFG_BASE = "SoapSmsDispatcher";
    public static final String NS_URI_SOAP_SMS = "http://www.eXtropia.com/soap/encoding/sms";

    public static final String CFG_SOAPSERVER_URL = "SoapServerUrl";
    public static final String CFG_SOAP_URN = "SoapUrn";
    public static final String CFG_TIMEOUT = "Timeout";
    public static final String CFG_PROXY_HOST = "ProxyHost";
    public static final String CFG_PROXY_PORT = "ProxyPort";
    public static final String CFG_USERNAME = "Username";
    public static final String CFG_PASSWORD = "Password";

    public static final String CFG_MAPPINGS = "Mappings";
    public static final String CFG_MAPPING = "Mapping";
    public static final String CFG_MAPPING_QNAME = "Qname";
    public static final String CFG_MAPPING_NAMESPACE = "Namespace";
    public static final String CFG_MAPPING_CLASS = "Class";
    public static final String CFG_MAPPING_SER = "Serializer";
    public static final String CFG_MAPPING_DESER = "Deserializer";

    private URL _url;
    private String _urn;
    private SOAPMappingRegistry _smr;

    private int _timeout;
    private String _proxyHost;
    private int _proxyPort;
    private String _username;
    private String _password;

    public SoapSmsDispatcher() {
        _smr = new SOAPMappingRegistry();

        BeanSerializer smsSer = new BeanSerializer();
        _smr.mapTypes(Constants.NS_URI_SOAP_ENC, new QName(NS_URI_SOAP_SMS, "Sms"), Sms.class, smsSer, smsSer);

        ExceptionSerializer exceptionSer = new ExceptionSerializer();
        _smr.mapTypes(Constants.NS_URI_SOAP_ENC, new QName("http://www.eXtropia.com/soap/encoding", "Exception"), Exception.class, exceptionSer, exceptionSer);

    }

    public void configure(Config config) throws ConfigException {
        if (_url != null || _urn != null)
            throw new ConfigException("Soap Dispatcher is already configured");

        if (!config.hasChild(CFG_SOAPSERVER_URL) || !config.hasChild(CFG_SOAP_URN))
            throw new ConfigException("Soap Dispatcher requies soap server url and soap urn config");

        try {
            _url = new URL(config.getChild(CFG_SOAPSERVER_URL).getValue());
            _urn = config.getChild(CFG_SOAP_URN).getValue();
        } catch (MalformedURLException e) {
            throw new ConfigException("Invalid Soap URL: " + e.getMessage());
        }

        if (config.hasChild(CFG_TIMEOUT)) {
            _timeout = Integer.parseInt(config.getChild(CFG_TIMEOUT).getValue());
        }

        if (config.hasChild(CFG_PROXY_HOST)) {
            Config proxyConfig = config.getChild(CFG_PROXY_HOST);
            _proxyHost = proxyConfig.getValue();

            if (proxyConfig.hasChild(CFG_PROXY_PORT)) {
                _proxyPort = Integer.parseInt(proxyConfig.getChild(CFG_PROXY_PORT).getValue());
            }
        }

        if (config.hasChild(CFG_USERNAME)) {
            Config authConfig = config.getChild(CFG_USERNAME);
            _username = authConfig.getValue();

            if (authConfig.hasChild(CFG_PASSWORD)) {
                _password = authConfig.getChild(CFG_PASSWORD).getValue();
            }
        }

        configureSoapRegistry(config.getChild(CFG_MAPPINGS));
    }

    protected void configureSoapRegistry(Config config) throws ConfigException {
        if (config == null)
            return;

        try {
            Config[] mappings = config.getChildren();
            for (int i = 0; i < mappings.length; i++) {
                QName qname;
                Class javaType;
                String namespace = null;
                Serializer ser = null;
                Deserializer deser = null;

                if (!mappings[i].hasChild(CFG_MAPPING_QNAME))
                    throw new ConfigException("Invalid mapping, " + CFG_MAPPING_QNAME + " not specifed");

                Config qnameConfig = mappings[i].getChild(CFG_MAPPING_QNAME);

                if (qnameConfig.hasChild(CFG_MAPPING_NAMESPACE))
                    namespace = qnameConfig.getChild(CFG_MAPPING_NAMESPACE).getValue();

                qname = new QName(namespace != null ? namespace : NS_URI_SOAP_SMS, qnameConfig.getValue());

                javaType = Class.forName(mappings[i].getChildValue(CFG_MAPPING_CLASS, true));
                if (mappings[i].hasChild(CFG_MAPPING_SER)) {
                    ser = (Serializer) Class.forName(mappings[i].getChildValue(CFG_MAPPING_SER)).newInstance();
                }

                if (mappings[i].hasChild(CFG_MAPPING_DESER)) {
                    deser = (Deserializer) Class.forName(mappings[i].getChildValue(CFG_MAPPING_DESER)).newInstance();
                }

                _smr.mapTypes(Constants.NS_URI_SOAP_ENC, qname, javaType, ser, deser);
            }
        } catch (Exception e) {
            throw new ConfigException("Failed to configure soap registry: " + e.getMessage());
        }
    }

    private Call createCall(String method) {
        Call call = new Call();
        SOAPHTTPConnection st = new SOAPHTTPConnection();

        if (_proxyHost != null) {
            st.setProxyHost(_proxyHost);
            st.setProxyPort(_proxyPort);
        }

        if (_username != null) {
            st.setUserName(_username);
            st.setPassword(_password);
        }

        st.setTimeout(_timeout);

        call.setSOAPTransport(st);
        call.setSOAPMappingRegistry(_smr);
        call.setTargetObjectURI(_urn);
        call.setMethodName(method);
        call.setEncodingStyleURI(Constants.NS_URI_SOAP_ENC);

        return call;
    }

    private void executeCall(Call call) throws SmsDispatchException {
        Response resp;
        try {
            resp = call.invoke(_url, call.getMethodName());
        } catch (SOAPException e) {
            System.out.println("Stacking");
            e.printStackTrace();
            throw new SmsDispatchException("SOAP Exception occured while performing " + call.getMethodName() + ": " + e.getMessage());
        }

        if (resp.generatedFault()) {
            Fault fault = resp.getFault();
            Vector detail = fault.getDetailEntries();
            if (detail != null) {
                for (Iterator iter = detail.iterator(); iter.hasNext();) {
                    Object e = iter.next();
                    if (e instanceof SmsDispatchException)
                        throw (SmsDispatchException) e;

                    if (e instanceof RuntimeException)
                        throw (RuntimeException) e;
                }
            }

            throw new SmsDispatchException("SOAP Fault occured while performing " + call.getMethodName() + ": " + fault.getFaultString());
        }
    }

    public void send(Sms sms) throws SmsDispatchException {
        Call call = createCall("send");
        Vector params = new Vector();
        params.addElement(new Parameter("sms", sms.getClass(), sms, null));
        call.setParams(params);
        executeCall(call);
    }

}
