/* 
 * DateSerializer.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.soap.encoding;

import java.io.IOException;
import java.io.Writer;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;

import org.apache.soap.encoding.soapenc.SoapEncUtils;
import org.apache.soap.rpc.SOAPContext;
import org.apache.soap.util.Bean;
import org.apache.soap.util.xml.DOMUtils;
import org.apache.soap.util.xml.Deserializer;
import org.apache.soap.util.xml.NSStack;
import org.apache.soap.util.xml.QName;
import org.apache.soap.util.xml.Serializer;
import org.apache.soap.util.xml.XMLJavaMappingRegistry;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/** 
 *
 * @author  Carlo Ravagli (carlo.ravagli@extropia.com)
 * @version 
 */
public class DateSerializer implements Serializer, Deserializer {
    public static final String XSD_DATE = "date";
    public static final String XSD_TIME = "time";
    public static final String XSD_TIMEINSTANT = "timeInstant";

    private static final SimpleDateFormat dateFormatter = new SimpleDateFormat("yyyy-MM-dd");
    private static final SimpleDateFormat timeFormatter = new SimpleDateFormat("HH:mm:ss");
    private static final SimpleDateFormat timeInstantFormatter = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");

    public DateSerializer() {
    }

    public void marshall(String inScopeEncStyle, Class javaType, Object src, Object context, Writer sink, NSStack nsStack, XMLJavaMappingRegistry xjmr, SOAPContext ctx) throws IOException {
        java.util.Date date = (java.util.Date) src;
        nsStack.pushScope();

        if (date == null) {
            SoapEncUtils.generateNullStructure(inScopeEncStyle, javaType, context, sink, nsStack, xjmr);
        } else {
            DateFormat formatter;

            SoapEncUtils.generateStructureHeader(inScopeEncStyle, javaType, context, sink, nsStack, xjmr);

            if (javaType.equals(java.util.Date.class) || javaType.equals(java.sql.Date.class))
                formatter = dateFormatter;
            else if (javaType.equals(java.sql.Time.class))
                formatter = timeFormatter;
            else if (javaType.equals(java.sql.Timestamp.class))
                formatter = timeInstantFormatter;
            else
                throw new IllegalArgumentException("Failed to marshall date type " + javaType.getName() + " : Unsupported type.");

            sink.write(formatter.format(date) + "</" + context + '>');
        }

        nsStack.popScope();
    }

    public Bean unmarshall(String inScopeEncStyle, final QName elementType, final Node src, final XMLJavaMappingRegistry xjmr, final SOAPContext ctx) {
        java.util.Date date = null;

        if (!SoapEncUtils.isNull((Element) src)) {
            try {
                String type = elementType.getLocalPart();
                String dateString = DOMUtils.getChildCharacterData((Element) src);

                if (type.equals(XSD_DATE))
                    date = new java.sql.Date(dateFormatter.parse(dateString).getTime());
                else if (type.equals(XSD_TIME))
                    date = new java.sql.Time(timeFormatter.parse(dateString).getTime());
                else if (type.equals(XSD_TIMEINSTANT))
                    date = new java.sql.Timestamp(timeInstantFormatter.parse(dateString).getTime());
                else
                    throw new IllegalArgumentException("Failed to unmarshall <" + src.getNodeName() + ">: unrecognised type " + type);
            } catch (ParseException pe) {
                throw new IllegalArgumentException("Failed to parse <" + src.getNodeName() + ">: " + pe.getMessage());
            }
        }
        return new Bean(java.util.Date.class, date);
    }
}