/*
 * DecimalSerializer.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.soap.encoding;

import java.io.IOException;
import java.io.Writer;
import java.math.BigDecimal;

import org.apache.soap.Constants;
import org.apache.soap.encoding.soapenc.SoapEncUtils;
import org.apache.soap.rpc.SOAPContext;
import org.apache.soap.util.Bean;
import org.apache.soap.util.xml.DOMUtils;
import org.apache.soap.util.xml.Deserializer;
import org.apache.soap.util.xml.NSStack;
import org.apache.soap.util.xml.QName;
import org.apache.soap.util.xml.Serializer;
import org.apache.soap.util.xml.XMLJavaMappingRegistry;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 *
 * @author  Carlo Ravagli (carlo.ravagli@extropia.com)
 * @version
 */
public class DecimalSerializer implements Serializer, Deserializer {
    public static final String XSD_DECIMAL = "decimal";

    public DecimalSerializer() {
    }

    public void marshall(String inScopeEncStyle, Class javaType, Object src, Object context, Writer sink, NSStack nsStack, XMLJavaMappingRegistry xjmr, SOAPContext ctx) throws IOException {
        nsStack.pushScope();

        BigDecimal decimal = (BigDecimal) src;

        if (decimal == null) {
            SoapEncUtils.generateNullStructure(inScopeEncStyle, javaType, context, sink, nsStack, xjmr);
        } else {
            SoapEncUtils.generateStructureHeader(inScopeEncStyle, javaType, context, sink, nsStack, xjmr);

            sink.write(decimal.toString() + "</" + context + '>');
        }

        nsStack.popScope();
    }

    public Bean unmarshall(String inScopeEncStyle, final QName elementType, final Node src, final XMLJavaMappingRegistry xjmr, final SOAPContext ctx) {
        BigDecimal decimal = null;

        if (!SoapEncUtils.isNull((Element) src)) {
            String type = elementType.getLocalPart();
            String decimalString = DOMUtils.getChildCharacterData((Element) src);

            if (type.equals(XSD_DECIMAL))
                decimal = new BigDecimal(decimalString);
            else
                throw new IllegalArgumentException("Failed to unmarshall <" + src.getNodeName() + ">: not a " + Constants.NS_URI_1999_SCHEMA_XSD + ":" + XSD_DECIMAL);
        }

        return new Bean(BigDecimal.class, decimal);
    }
}