/* 
 * ExceptionSerializer.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.soap.encoding;

import java.io.IOException;
import java.io.Writer;
import java.lang.reflect.Constructor;

import org.apache.soap.Constants;
import org.apache.soap.encoding.soapenc.SoapEncUtils;
import org.apache.soap.rpc.SOAPContext;
import org.apache.soap.util.Bean;
import org.apache.soap.util.xml.DOMUtils;
import org.apache.soap.util.xml.Deserializer;
import org.apache.soap.util.xml.NSStack;
import org.apache.soap.util.xml.QName;
import org.apache.soap.util.xml.Serializer;
import org.apache.soap.util.xml.XMLJavaMappingRegistry;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/** 
 *
 * @author  Carlo Ravagli (carlo.ravagli@extropia.com)
 * @version 
 */
public class ExceptionSerializer implements Serializer, Deserializer {
    public static final String ELEMENT_TYPE = "type";
    public static final String ELEMENT_MESSAGE = "message";

    public ExceptionSerializer() {
    }

    public void marshall(String inScopeEncStyle, Class javaType, Object src, Object context, Writer sink, NSStack nsStack, XMLJavaMappingRegistry xjmr, SOAPContext ctx) throws IOException {
        Exception e = (Exception) src;
        if (e != null) {
            nsStack.pushScope();

            QName elementType = xjmr.queryElementType(javaType, Constants.NS_URI_SOAP_ENC);
            String prefix = nsStack.getPrefixFromURI(elementType.getNamespaceURI());

            if (prefix == null)
                prefix = nsStack.addNSDeclaration(elementType.getNamespaceURI());

            context = prefix + ':' + elementType.getLocalPart();
            sink.write('<' + context.toString());
            sink.write(" xmlns:" + prefix + "=\"" + elementType.getNamespaceURI() + '\"');

            if (inScopeEncStyle == null || !inScopeEncStyle.equals(Constants.NS_URI_SOAP_ENC)) {
                // Determine the prefix associated with the NS_URI_SOAP_ENV
                // namespace URI.
                String soapEnvNSPrefix = nsStack.getPrefixFromURI(Constants.NS_URI_SOAP_ENV, sink);

                sink.write(' ' + soapEnvNSPrefix + ':');
                sink.write(Constants.ATTR_ENCODING_STYLE + "=\"");
                sink.write(Constants.NS_URI_SOAP_ENC + '\"');
            }
            sink.write('>');
            writeDetail(e, sink);
            sink.write("</" + context + '>');
            nsStack.popScope();
        }
    }

    public Bean unmarshall(String inScopeEncStyle, final QName elementType, final Node src, final XMLJavaMappingRegistry xjmr, final SOAPContext ctx) {
        Exception exception = null;

        if (!SoapEncUtils.isNull((Element) src)) {
            String type = null;
            String message = null;

            for (Node childNode = src.getFirstChild(); childNode != null; childNode = childNode.getNextSibling()) {
                if (childNode.getNodeType() == Node.ELEMENT_NODE) {
                    if (childNode.getNodeName().equals(ELEMENT_TYPE))
                        type = DOMUtils.getChildCharacterData((Element) childNode);
                    else if (childNode.getNodeName().equals(ELEMENT_MESSAGE))
                        message = DOMUtils.getChildCharacterData((Element) childNode);
                }
            }
            if (type == null)
                throw new IllegalArgumentException("No exception type specified");

            try {
                if (message != null) {
                    Class exceptionClass = Class.forName(type);
                    Constructor c = exceptionClass.getConstructor(new Class[] { String.class });
                    exception = (Exception) c.newInstance(new Object[] { message });
                } else
                    exception = (Exception) Class.forName(type).newInstance();
            } catch (Exception e) {
                throw new IllegalArgumentException(e.getMessage());
            }
        }
        return new Bean(java.lang.Exception.class, exception);
    }

    protected void writeDetail(Exception e, Writer sink) throws IOException {
        sink.write('<' + ELEMENT_TYPE + '>');
        sink.write(e.getClass().getName());
        sink.write("</" + ELEMENT_TYPE + '>');
        sink.write('<' + ELEMENT_MESSAGE + '>');
        sink.write(e.getMessage());
        sink.write("</" + ELEMENT_MESSAGE + '>');
    }
}