/*
 * StructDeserializer.java 
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.soap.encoding;

import java.io.IOException;
import java.io.Writer;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.apache.soap.Constants;
import org.apache.soap.encoding.soapenc.SoapEncUtils;
import org.apache.soap.rpc.SOAPContext;
import org.apache.soap.util.Bean;
import org.apache.soap.util.xml.Deserializer;
import org.apache.soap.util.xml.NSStack;
import org.apache.soap.util.xml.QName;
import org.apache.soap.util.xml.Serializer;
import org.apache.soap.util.xml.XMLJavaMappingRegistry;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 *
 * @author  Carlo Ravagli (carlo.ravagli@extropia.com)
 * @version
 */

public class StructDeserializer implements Serializer, Deserializer {
    public StructDeserializer() {
    }

    public void marshall(String inScopeEncStyle, Class javaType, Object src, Object context, Writer sink, NSStack nsStack, XMLJavaMappingRegistry xjmr, SOAPContext ctx) throws IOException {
        Map struct = (Map) src;
        nsStack.pushScope();

        if (struct == null) {
            SoapEncUtils.generateNullStructure(inScopeEncStyle, javaType, context, sink, nsStack, xjmr);
        } else {

            SoapEncUtils.generateStructureHeader(inScopeEncStyle, javaType, context, sink, nsStack, xjmr);

            for (Iterator i = struct.entrySet().iterator(); i.hasNext();) {
                Map.Entry entry = (Map.Entry) i.next();
                if (entry.getValue() != null) {
                    xjmr.marshall(inScopeEncStyle, entry.getValue().getClass(), entry.getValue(), (String) entry.getKey(), sink, nsStack, ctx);
                } else {
                    SoapEncUtils.generateNullStructure(inScopeEncStyle, Object.class, (String) entry.getKey(), sink, nsStack, xjmr);
                }
            }

            sink.write("</" + context + '>');
        }
        nsStack.popScope();
    }

    public Bean unmarshall(String inScopeEncStyle, final QName elementType, final Node src, final XMLJavaMappingRegistry xjmr, final SOAPContext ctx) {
        if (SoapEncUtils.isNull((Element) src))
            return new Bean(Map.class, null);

        HashMap struct = new HashMap();

        for (Node childNode = src.getFirstChild(); childNode != null; childNode = childNode.getNextSibling()) {
            if (childNode.getNodeType() == Node.ELEMENT_NODE) {
                Element structItem = (Element) childNode;
                Object value = null;

                if (!SoapEncUtils.isNull(structItem)) {

                    QName soapType = SoapEncUtils.getAttributeValue(structItem, Constants.NS_URI_1999_SCHEMA_XSI, Constants.ATTR_TYPE, "struct item", true);

                    value = xjmr.unmarshall(inScopeEncStyle, soapType, childNode, ctx).value;
                }

                if (struct.put(structItem.getTagName(), value) != null)
                    throw new IllegalArgumentException("Failed to unmarshall " + Constants.NS_URI_SOAP_ENC + ":Struct <" + src.getNodeName() + ">: elements are not unique");
            }
        }
        return new Bean(Map.class, struct);
    }
}