/* 
 * CriterionDeserializer.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.soap.encoding.datasource;

import java.io.IOException;
import java.io.Writer;

import org.apache.soap.encoding.soapenc.SoapEncUtils;
import org.apache.soap.rpc.SOAPContext;
import org.apache.soap.util.Bean;
import org.apache.soap.util.xml.DOMUtils;
import org.apache.soap.util.xml.Deserializer;
import org.apache.soap.util.xml.NSStack;
import org.apache.soap.util.xml.QName;
import org.apache.soap.util.xml.Serializer;
import org.apache.soap.util.xml.XMLJavaMappingRegistry;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import com.eXtropia.datasource.Condition;
import com.eXtropia.datasource.Criterion;

/** 
 *
 * @author  Carlo Ravagli (carlo.ravagli@extropia.com)
 * @version 1.0
 */
public class CriterionDeserializer implements Serializer, Deserializer {
    public static final String CRITERION_ELEMENT_NAME = "Criterion";
    public static final String TYPE_ELEMENT_NAME = "Type";

    public CriterionDeserializer() {
    }

    public void marshall(String inScopeEncStyle, Class javaType, Object src, Object context, Writer sink, NSStack nsStack, XMLJavaMappingRegistry xjmr, SOAPContext ctx) throws IOException {
        Criterion criterion = (Criterion) src;
        nsStack.pushScope();

        if (criterion == null) {
            SoapEncUtils.generateNullStructure(inScopeEncStyle, javaType, context, sink, nsStack, xjmr);
        } else {

            SoapEncUtils.generateStructureHeader(inScopeEncStyle, javaType, context, sink, nsStack, xjmr);

            sink.write('<' + TYPE_ELEMENT_NAME + '>');
            sink.write(criterion.getType() == Criterion.AND ? "AND" : "OR");
            sink.write("</" + TYPE_ELEMENT_NAME + '>');

            Condition[] conds = criterion.getConditions();
            if (conds != null) {
                for (int i = 0; i < conds.length; i++) {
                    xjmr.marshall(inScopeEncStyle, Condition.class, conds[i], ConditionDeserializer.CONDITION_ELEMENT_NAME, sink, nsStack, ctx);
                }
            }

            Criterion[] subCrits = criterion.getSubCriteria();
            if (subCrits != null) {
                for (int i = 0; i < subCrits.length; i++) {
                    marshall(inScopeEncStyle, Criterion.class, subCrits[i], CRITERION_ELEMENT_NAME, sink, nsStack, xjmr, ctx);
                }
            }

            sink.write("</" + context + '>');
        }

        nsStack.popScope();
    }

    public Bean unmarshall(String inScopeEncStyle, final QName elementType, final Node src, final XMLJavaMappingRegistry xjmr, final SOAPContext ctx) {
        if (SoapEncUtils.isNull((Element) src))
            return new Bean(Criterion.class, null);

        Criterion criterion = new Criterion();

        for (Node childNode = src.getFirstChild(); childNode != null; childNode = childNode.getNextSibling()) {
            if (childNode.getNodeType() == Node.ELEMENT_NODE) {
                String name = childNode.getNodeName();
                if (name.equals(ConditionDeserializer.CONDITION_ELEMENT_NAME)) {
                    Condition cond = (Condition) xjmr.unmarshall(inScopeEncStyle, xjmr.queryElementType(Condition.class, inScopeEncStyle), childNode, ctx).value;

                    if (cond != null)
                        criterion.add(cond);

                } else if (name.equals(CRITERION_ELEMENT_NAME)) {
                    Criterion subCriterion = (Criterion) unmarshall(inScopeEncStyle, elementType, childNode, xjmr, ctx).value;

                    if (subCriterion != null)
                        criterion.add(subCriterion);
                } else if (name.equals(TYPE_ELEMENT_NAME)) {
                    String criterionType = DOMUtils.getChildCharacterData((Element) childNode);
                    try {
                        criterion.setType(Criterion.class.getDeclaredField(criterionType).getInt(null));
                    } catch (Exception e) {
                        throw new IllegalArgumentException(e.getMessage());
                    }
                } else
                    throw new IllegalArgumentException("Failed to unmarshall Criterion: <" + name + "> is not a valid sub element");
            }
        }

        return new Bean(Criterion.class, criterion);
    }
}