/* 
 * QuotaExceptionSerializer.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.soap.encoding.datasource.secure;

import java.io.IOException;
import java.io.Writer;

import org.apache.soap.rpc.SOAPContext;
import org.apache.soap.util.Bean;
import org.apache.soap.util.xml.DOMUtils;
import org.apache.soap.util.xml.Deserializer;
import org.apache.soap.util.xml.QName;
import org.apache.soap.util.xml.Serializer;
import org.apache.soap.util.xml.XMLJavaMappingRegistry;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import com.eXtropia.datasource.secure.QuotaException;
import com.eXtropia.soap.encoding.ExceptionSerializer;

/** 
 *
 * @author  Carlo Ravagli (carlo.ravagli@extropia.com)
 * @version 
 */
public class QuotaExceptionSerializer extends ExceptionSerializer implements Serializer, Deserializer {
    public static final String ELEMENT_QUOTA = "quota";

    public QuotaExceptionSerializer() {
    }

    public Bean unmarshall(String inScopeEncStyle, final QName elementType, final Node src, final XMLJavaMappingRegistry xjmr, final SOAPContext ctx) {
        int quota = 0;
        String message = null;

        for (Node childNode = src.getFirstChild(); childNode != null; childNode = childNode.getNextSibling()) {
            if (childNode.getNodeType() == Node.ELEMENT_NODE) {
                if (childNode.getNodeName().equals(ELEMENT_QUOTA))
                    quota = Integer.parseInt(DOMUtils.getChildCharacterData((Element) childNode));
                else if (childNode.getNodeName().equals(ELEMENT_MESSAGE))
                    message = DOMUtils.getChildCharacterData((Element) childNode);
            }
        }

        return new Bean(QuotaException.class, new QuotaException(message, quota));
    }

    protected void writeDetail(Exception e, Writer sink) throws IOException {
        super.writeDetail(e, sink);
        QuotaException qe = (QuotaException) e;
        if (qe.getQuota() > 0) {
            sink.write('<' + ELEMENT_QUOTA + '>');
            sink.write(Integer.toString(qe.getQuota()));
            sink.write("</" + ELEMENT_QUOTA + '>');
        }
    }
}