/* 
 * MapMessageFormatter.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.util;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import com.eXtropia.config.Config;
import com.eXtropia.config.ConfigException;

/**
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 
 */
public class MapMessageFormatter {
    public static final String CFG_BASE = "MessageFormatter";
    public static final String CFG_FORMAT = "Format";

    private ArrayList _formatList;
    private ArrayList _mappingsList;
    private String[] _mappings;

    public MapMessageFormatter() {

    }

    public MapMessageFormatter(String pattern) {
        setPattern(pattern);
    }

    public void configure(Config config) throws ConfigException {
        if (!config.getName().equals(CFG_BASE))
            throw new ConfigException("Incorrect config passed to " + CFG_BASE + ": " + config.getName());
        Config[] children = config.getChildren();
        if (children == null || children.length < 1) {
            throw new ConfigException("Incorrect config passed to " + CFG_BASE + ": You must specify at least one <Format> section");
        } else {
            for (int i = 0; i < children.length; i++) {
                if (children[i].getName().equals(CFG_FORMAT)) {
                    addPattern(children[i].getValue());
                }
            }
        }

        //if(config.hasChild(CFG_FORMAT))
        //		setPattern(config.getChild(CFG_FORMAT).getValue());
        //else
        //	throw new ConfigException("No " + CFG_FORMAT + " specified in " + CFG_BASE);
    }

    public String format(Map args) {
        if (_formatList == null)
            throw new IllegalStateException("Message format can't format exception without a format");

        String formattedMessage = "";
        for (int j = 0; j < _formatList.size(); j++) {
            if (_mappingsList.get(j) == null)
                throw new IllegalArgumentException("Mappings List for one of the formats had no mappings");
            Object[] argsArray = new Object[((Object[]) (_mappingsList.get(j))).length];

            for (int i = 0; i < ((Object[]) (_mappingsList.get(j))).length; i++) {
                argsArray[i] = args.get(((Object[]) (_mappingsList.get(j)))[i]);
                if (argsArray[i] == null)
                    argsArray[i] = "";
            }
            formattedMessage += ((MessageFormat) _formatList.get(j)).format(argsArray);
        }
        return formattedMessage;
    }

    public void addPattern(String pattern) {
        StringBuffer formatPattern = new StringBuffer();
        List mappings = new ArrayList();
        StringBuffer argName;
        int mappingIndex;

        for (int i = 0; i < pattern.length(); i++) {
            formatPattern.append(pattern.charAt(i));
            if (pattern.charAt(i) == '{' && (i == 0 || pattern.charAt(i - 1) != '\'')) {
                argName = new StringBuffer();
                while (pattern.charAt(i + 1) != '}' && pattern.charAt(i + 1) != ',')
                    argName.append(pattern.charAt(++i));

                mappingIndex = mappings.indexOf(argName.toString());
                if (mappingIndex < 0) {
                    formatPattern.append(mappings.size());
                    mappings.add(argName.toString());
                } else
                    formatPattern.append(mappingIndex);
            }
        }
        if (_mappingsList == null) {
            _mappingsList = new ArrayList();
        }
        _mappingsList.add(mappings == null ? null : (String[]) mappings.toArray(new String[mappings.size()]));
        try {
            if (_formatList == null) {
                _formatList = new ArrayList();
            }
            _formatList.add(new MessageFormat(formatPattern.toString()));
        } catch (IllegalArgumentException e) {
            throw new IllegalArgumentException("MessageFormat [" + formatPattern.toString() + "] could not be created for the following reason: " + e.getMessage());
        }

    } // end of addPattern

    public void setPattern(String pattern) {
        _formatList = null;
        _mappingsList = null;
        addPattern(pattern);
    }

}
