/*
 * HtmlEncoder.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.util;

import java.io.File;
import java.util.StringTokenizer;

import javax.servlet.ServletContext;

/**
 * Utility methods 
 * @author Lee Yeow Leong
 */
public class Utils {
    /**
     * construct the full path from a relative path specified
     * in relation to the web application directory
     * The relative path should begins with a "./".
     * @param relativePath the relative path name
     * @return the constructed full path name
     * @throws IllegalArgumentException if the relativePath is null,empty or having length lesser or equal to 2, 
                                        or the servlet context is null
     */
    public static String getFullPathName(ServletContext ctx, String relativePathName) {
        if (ctx == null) {
            throw new IllegalArgumentException("ServletContext cannot be null");
        }
        if (relativePathName == null || relativePathName.equals("")) {
            throw new IllegalArgumentException("Relative Path Name cannot be null or empty.");
        }
        if (relativePathName.length() <= 2) {
            // has to be because it starts with a "./"
            throw new IllegalArgumentException("Relative Path Name must be of length > 2");
        }
        String fullPath = ctx.getRealPath("/");
        String fileSeparator = File.separator;
        // removes the "./" 
        if (fullPath.endsWith("/")) {
            relativePathName = relativePathName.substring(2);
        } else {
            relativePathName = relativePathName.substring(1);
        }
        if (relativePathName.indexOf(fileSeparator) == -1) {
            StringBuffer sb = new StringBuffer();
            StringTokenizer st = new StringTokenizer(relativePathName, "/");
            while (st.hasMoreTokens()) {
                sb.append(st.nextToken());
                if (st.hasMoreTokens()) {
                    sb.append(fileSeparator);
                }
            }
            relativePathName = sb.toString();
        }
        fullPath += relativePathName;
        System.out.println(fullPath);
        return fullPath;
    }
}