/* 
 * Worker.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.util;

import java.util.LinkedList;

/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 0.1
 */
public class Worker implements Runnable {
    private int _threadPriority;
    private boolean _daemon;
    private LinkedList _workQueue = new LinkedList();
    private Thread _myThread;
    /** Creates new Worker */
    public Worker(int threadPriority, boolean daemon) {
        _threadPriority = threadPriority;
        _daemon = daemon;
    }

    public void run() {
        Runnable work;
        //while there are events fire them
        do {
            try {
                synchronized (_workQueue) {
                    if (_workQueue.isEmpty()) {
                        _workQueue.wait();
                    }
                    work = (Runnable) _workQueue.removeFirst();
                }
                work.run();
            } catch (InterruptedException e) {
                return;
            }
        } while (true);
    }

    public void addWork(Runnable work) {
        if (_myThread == null || !_myThread.isAlive()) {
            _myThread = new Thread(this);
            _myThread.setPriority(_threadPriority);
            _myThread.setDaemon(_daemon);
            _myThread.start();
        }
        synchronized (_workQueue) {
            _workQueue.addLast(work);
            if (_workQueue.size() == 1) {
                _workQueue.notifyAll();
            }
        }
    }
}