/*
 * XCrypt.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Created on December 28, 2001, 1:59 PM
 */
package com.eXtropia.util.crypt;

import java.security.GeneralSecurityException;

import com.eXtropia.config.Configurable;

/**
 * @author  Ray Djajadinata (rayfd@eXtropia.com)
 * @version 0.1
 */
public interface XCrypt extends Configurable {
    String CFG_BASE = "XCrypt";

    /**
     * Encrypts plaintext. The algorithm is specified by the implementing class.
     *
     * @param plainText   the plain text to be encrypted
     * @return            the cipher text
     * @throws GeneralSecurityException This exception covers all the exceptions that
     * are common in cryptographic operations. Implementing classes should throw
     * more specific exception.
     */
    byte[] encrypt(byte[] plainText) throws GeneralSecurityException;

    /**
     * Decrypts ciphertext. The algorithm is specified by the implementing class.
     *
     * @param cipherText    the cipher text to be decrypted
     * @return              the plain text
     * @throws GeneralSecurityException This exception covers all the exceptions that
     * are common in cryptographic operations. Implementing classes should throw
     * more specific exception.
     */
    byte[] decrypt(byte[] cipherText) throws GeneralSecurityException;

    /**
     * Encrypts plaintext and base64-encode the result. The algorithm is specified
     * by the implementing class.
     *
     * @param plainText   the plain text to be encrypted
     * @return            the base64-encoded cipher text
     * @throws GeneralSecurityException This exception covers all the exceptions that
     * are common in cryptographic operations. Implementing classes should throw
     * more specific exception.
     */
    String encryptAndBase64Encode(byte[] plainText) throws GeneralSecurityException;

    /**
     * Base64 decodes and decrypts the resulting ciphertext. The algorithm is
     * specified by the implementing class.
     *
     * @param base64CipherText  the base64-encoded cipher text to be decoded and decrypted
     * @return                  the plain text
     * @throws GeneralSecurityException This exception covers all the exceptions that
     * are common in cryptographic operations. Implementing classes should throw
     * more specific exception.
     */
    byte[] base64DecodeAndDecrypt(String base64CipherText) throws GeneralSecurityException;
}
