/* 
 * DataHandlerExceptionTag.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.view;

import java.io.IOException;
import java.util.Locale;
import java.util.Map;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import javax.servlet.ServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TagSupport;

import com.eXtropia.app.UserGestureResponse;
import com.eXtropia.app.servlet.ServletUtils;
import com.eXtropia.datahandler.DataHandlerException;
import com.eXtropia.util.MapMessageFormatter;

/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */
public class DataHandlerExceptionTag extends TagSupport {
    protected String _id;

    /** Creates new DataHandlerExceptionTag */
    public DataHandlerExceptionTag() {

    }

    public int doEndTag() throws JspException {
        ServletRequest request = (ServletRequest) pageContext.getRequest();
        UserGestureResponse gestureResponse = (UserGestureResponse) request.getAttribute(UserGestureResponse.REQUEST_PARAM_NAME);
        DataHandlerException topLevelError = gestureResponse.getDataHandlerException();
        if (topLevelError != null) {
            DataHandlerException firstError = topLevelError.getFirstError(_id);
            if (firstError != null) {
                View view = (View) request.getAttribute(ViewManager.VIEW_ATTRIBUTE_NAME);
                Locale locale = (Locale) pageContext.getSession().getAttribute(ServletUtils.SESSION_ATTRIBUTE_LOCALE);
                ResourceBundle bundle = ResourceBundle.getBundle(view.getResourceLocation(), locale);
                String error = firstError.getResourceName();
                try {
                    error = bundle.getString(error);
                    Map exceptionData = firstError.getExceptionData();
                    if (exceptionData != null) {
                        MapMessageFormatter formatter = new MapMessageFormatter(error);
                        error = formatter.format(exceptionData);
                    }
                } catch (MissingResourceException mre) {
                }
                try {
                    pageContext.getOut().write(error);
                } catch (IOException ioe) {
                    throw new JspException("IOException: " + ioe.getMessage());
                }
            }
        }
        return EVAL_PAGE;
    }

    public String getId() {
        return _id;
    }

    public void setId(String id) {
        _id = id;
    }
}