/*
 * FormNavigationTag.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.view;

import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import javax.servlet.ServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.TagSupport;

import com.eXtropia.app.UserGesture;
import com.eXtropia.app.UserGestureResponse;
import com.eXtropia.app.servlet.ServletUtils;

/**
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */

public class FormNavigationTag extends TagSupport {
    protected String _src;
    protected String _alt;
    protected String _height;
    protected String _width;
    protected String _value;
    protected String _onClick;
    protected String _action;
    protected String _targetView;
    protected String _view;
    protected String _style;
    protected boolean _default;
    protected boolean _supress = false;
    //anderson - add the extraAttributes
    protected String _extraAttributes;

    protected ServletRequest _request;
    protected UserGesture _gesture;
    protected UserGestureResponse _gestureResponse;

    public FormNavigationTag() {
        _supress = false;
    }

    private void appendHiddenInput(StringBuffer buffer, String key, String value) {
        buffer.append("<INPUT TYPE=\"HIDDEN\" NAME=\"");
        buffer.append(key);
        buffer.append("\" VALUE=\"");
        buffer.append(value);
        buffer.append("\" ALIGN=\"ABSMIDDLE\">\n");
    }

    //anderson - add the extraAttributes
    protected void appendSubmitInput(StringBuffer buffer, String action, String src, String alt, String onClick, String extraAttributes, String value, String height, String width, String style) {
        if (src == null) {
            buffer.append("<INPUT TYPE=\"SUBMIT\"");
            if (action != null) {
                buffer.append(" NAME=\"");
                buffer.append(getActionParamName());
                buffer.append(action);
                buffer.append("\"");
            }
            if (onClick != null) {
                buffer.append(" onClick=\"");
                buffer.append(onClick);
                buffer.append("\"");
            }
            buffer.append(" VALUE=\"");
            if (value != null) {
                buffer.append(value);
            } else if (alt != null) {
                buffer.append(alt);
            }
            buffer.append("\"");
            if (style != null)
                buffer.append(" STYLE=\"" + style + "\"");
            //anderson - add the extraAttributes
            if (extraAttributes != null) {
                buffer.append(" ");
                buffer.append(extraAttributes);
            }

            buffer.append(" BORDER=\"0\">");
        } else {
            buffer.append("<INPUT TYPE=\"IMAGE\" SRC=\"");
            buffer.append(src);
            buffer.append("\"");
            if (action != null) {
                buffer.append(" NAME=\"");
                buffer.append(getActionParamName());
                buffer.append(action);
                buffer.append("\"");
            }
            if (onClick != null) {
                buffer.append(" onClick=\"");
                buffer.append(onClick);
                buffer.append("\"");
            }
            if (_height != null) {
                buffer.append(" HEIGHT=\"");
                buffer.append(height);
                buffer.append("\"");
            }
            if (width != null) {
                buffer.append(" WIDTH=\"");
                buffer.append(width);
                buffer.append("\"");
            }
            if (alt != null) {
                buffer.append(" ALT=\"");
                buffer.append(alt);
                buffer.append("\"");
            }
            if (style != null)
                buffer.append(" STYLE=\"" + style + "\"");

            if (extraAttributes != null) {
                buffer.append(" ");
                buffer.append(extraAttributes);
            }

            buffer.append(" BORDER=\"0\">");
        }
        if (_default)
            appendHiddenInput(buffer, getActionParamName() + action, "1");
    }

    public int doEndTag() throws JspException {
        Map requestParams = getRequestParams();
        if (requestParams == null)
            return EVAL_PAGE;
        if (_supress)
            requestParams = new HashMap();

        Map.Entry entry;
        StringBuffer buffer = new StringBuffer(256);
        Object[] multiValue;

        //determine the text for the submit button / image
        View view = (View) pageContext.getRequest().getAttribute(ViewManager.VIEW_ATTRIBUTE_NAME);
        Locale locale = (Locale) pageContext.getSession().getAttribute(ServletUtils.SESSION_ATTRIBUTE_LOCALE);
        ResourceBundle bundle = ResourceBundle.getBundle(view.getResourceLocation(), locale);

        String alt = _alt;
        if (alt != null) {
            try {
                alt = bundle.getString(_alt);
            } catch (MissingResourceException mre) {
            }
        }
        String value = _value;
        if (value != null) {
            try {
                value = bundle.getString(_value);
            } catch (MissingResourceException mre) {
            }
        }
        boolean doneSubmit = false;

        for (Iterator iter = requestParams.entrySet().iterator(); iter.hasNext();) {
            entry = (Map.Entry) iter.next();
            if (entry.getValue() instanceof Object[]) {
                multiValue = (Object[]) entry.getValue();
                for (int i = 0; i < multiValue.length; i++) {
                    appendHiddenInput(buffer, entry.getKey().toString(), multiValue[i].toString());
                }
            } else if (entry.getKey().toString().equals(getActionParamName())) {
                //anderson - add the extraAttributes
                appendSubmitInput(buffer, (String) entry.getValue(), _src, alt, _onClick, _extraAttributes, _value, _height, _width, _style);
                doneSubmit = true;
            } else {
                appendHiddenInput(buffer, entry.getKey().toString(), entry.getValue().toString());
            }
        }
        if (!doneSubmit)
            //anderson - add the extraAttributes
            appendSubmitInput(buffer, _action, _src, alt, _onClick, _extraAttributes, _value, _height, _width, _style);
        try {
            pageContext.getOut().write(buffer.toString());
        } catch (IOException ioe) {
            throw new JspException("IOException: " + ioe.getMessage());
        }
        return EVAL_PAGE;
    }

    protected String getActionParamName() {
        return UserGesture.ACTION_REQUEST_PARAM_NAME;
    }

    public int doStartTag() throws JspException {
        return SKIP_BODY;
    }

    /**
     * @return a string of hidden form elements with the specified changes and exclustions
     */
    protected Map getRequestParams() {
        UserGesture gesture = (UserGesture) pageContext.getRequest().getAttribute(UserGesture.REQUEST_PARAM_NAME);
        Map map = gesture.getRequestParams();

        //when submitting a form, the view tag should always be set to the view that we came from
        if (_view != null) {
            map.put(UserGesture.VIEW_REQUEST_PARAM_NAME, _view);
        } else {
            View view = (View) pageContext.getRequest().getAttribute(ViewManager.VIEW_ATTRIBUTE_NAME);
            map.put(UserGesture.VIEW_REQUEST_PARAM_NAME, view.getName());
        }
        if (_targetView != null)
            map.put(UserGesture.TARGET_VIEW_REQUEST_PARAM_NAME, _targetView);
        //no action by default, sub class should add one if required
        map.remove(UserGesture.ACTION_REQUEST_PARAM_NAME);

        return map;
    }

    public String getAlt() {
        return _alt;
    }

    public String getSrc() {
        return _src;
    }

    public void setPageContext(PageContext pc) {
        super.setPageContext(pc);
        _request = pc.getRequest();
        _gesture = (UserGesture) _request.getAttribute(UserGesture.REQUEST_PARAM_NAME);
        _gestureResponse = (UserGestureResponse) _request.getAttribute(UserGestureResponse.REQUEST_PARAM_NAME);
    }

    public String getTargetView() {
        return _targetView;
    }

    public void setAlt(String alt) {
        _alt = alt;
    }

    public void setDefault(boolean d) {
        _default = d;
    }

    public void setSrc(String src) {
        _src = src;
    }

    public String getHeight() {
        return _height;
    }
    public String getWidth() {
        return _width;
    }
    public String getValue() {
        return _value;
    }
    public String getOnClick() {
        return _onClick;
    }
    public String getAction() {
        return _action;
    }
    public String getStyle() {
        return _style;
    }
    public boolean getSupress() {
        return _supress;
    }
    //anderson
    public String getExtraAttributes() {
        return _extraAttributes;
    }

    public void setHeight(String s) {
        _height = s;
    }
    public void setWidth(String s) {
        _width = s;
    }
    public void setValue(String s) {
        _value = s;
    }
    public void setOnClick(String s) {
        _onClick = s;
    }
    public void setAction(String s) {
        _action = s;
    }
    public void setStyle(String s) {
        _style = s;
    }
    public void setSupress(String s) {
        _supress = new Boolean(s).booleanValue();
    }
    public void setSupress(boolean b) {
        _supress = b;
    }
    //anderson
    public void setExtraAttributes(String s) {
        _extraAttributes = s;
    }

    public void setTargetView(String targetView) {
        _targetView = targetView;
    }

    public void setView(String view) {
        _view = view;
    }

    public void release() {
        _src = null;
        _alt = null;
        _targetView = null;
        _default = false;
        _style = null;
        _view = null;
    }
}
