/* 
 * LookAndFeel.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.view;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import com.eXtropia.config.Config;
import com.eXtropia.config.ConfigException;
import com.eXtropia.config.Configurable;

/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 0.1
 */
public class LookAndFeel implements Configurable {
    public static final String CFG_BASE = "LookAndFeel";
    public static final String ATTRIBUTE_NAME = "lookAndFeel";

    /**
     * General page parameters
     */
    private String _fontFace = "HELVETICA, ARIAL";
    private String _fontSize = "-1";
    private String _fgColor = "#000000";
    private String _bgColor = "#FFFFFF";

    private String _titleFontFace = "HELVETICA, ARIAL";
    private String _titleFontSize = "+1";
    private String _titleFgColor = "#CCCCCC";

    /**
     * Link parameters
     */
    private String _linkFontFace = null;
    private String _linkFontSize = null;
    private String _linkActiveColor = "";
    private String _linkVisitedColor = "";

    /**
     * Parameters for tables
     */
    private String _tableHeaderFgColor = "#FFFFFF";
    private String _tableHeaderBgColor = "#990000";
    private String _tableHeaderFontFace = null;
    private String _tableHeaderFontSize = "-1";

    private String _tableSubHeaderFgColor = "#000000";
    private String _tableSubHeaderBgColor = "#CCCC99";
    private String _tableSubHeaderFontFace = null;
    private String _tableSubHeaderFontSize = null;

    private String _tableRowBgColor = "#FFFFFF";
    private String _tableRowFgColor = "#000000";
    private String _tableRowFontFace = null;
    private String _tableRowFontSize = null;

    private String _tableRowPrimaryBgColor = "#FFFFFF";
    private String _tableRowSecondaryBgColor = "#CCCCCC";
    private String _tableRowPrimaryFgColor = null;
    private String _tableRowSecondaryFgColor = null;

    private String _tableBorderWidth = "1";
    private String _tableBorderColor = "#999999";

    private String _tableCellSpacing = "0";
    private String _tableCellPadding = "1";

    private String _imageBase = "/images";

    private boolean _useRollover = false;

    public LookAndFeel() {

    }

    public void configure(Config config) throws ConfigException {
        if (config.getName().equals(CFG_BASE)) {
            Method method;
            Class[] singleString = new Class[] { String.class };
            String property;
            Config[] children = config.getChildren();
            //use reflection on each of the children to set the variable accordingly
            for (int i = 0; i < children.length; i++) {
                try {
                    method = getClass().getDeclaredMethod("set" + children[i].getName().substring(0, 1).toUpperCase() + children[i].getName().substring(1), singleString);
                    method.invoke(this, new Object[] { children[i].getValue()});
                } catch (NoSuchMethodException msme) {
                    throw new ConfigException("Failed to configure LookAndFeel: unknown property: " + children[i].getName());
                } catch (InvocationTargetException ite) {
                } catch (IllegalAccessException iae) {
                }
            }
        }
    }

    public String getFontFace() {
        return _fontFace;
    }

    public String getFontSize() {
        return _fontSize;
    }

    public String getFgColor() {
        return _fgColor;
    }

    public String getBgColor() {
        return _bgColor;
    }

    public String getTitleFontFace() {
        return _titleFontFace;
    }

    public String getTitleFgColor() {
        return _titleFgColor;
    }

    public String getTitleFontSize() {
        return _titleFontSize;
    }

    public String getLinkFontFace() {
        return _linkFontFace == null ? getFontFace() : _linkFontFace;
    }

    public String getLinkFontSize() {
        return _linkFontSize == null ? getFontSize() : _linkFontSize;
    }

    public String getLinkActiveColor() {
        return _linkActiveColor;
    }

    public String getLinkVisitedColor() {
        return _linkVisitedColor;
    }

    public String getTableHeaderFgColor() {
        return _tableHeaderFgColor == null ? getFgColor() : _tableHeaderFgColor;
    }

    public String getTableHeaderBgColor() {
        return _tableHeaderBgColor == null ? getBgColor() : _tableHeaderBgColor;
    }

    public String getTableHeaderFontFace() {
        return _tableHeaderFontFace == null ? getFontFace() : _tableHeaderFontFace;
    }

    public String getTableHeaderFontSize() {
        return _tableHeaderFontSize == null ? getFontSize() : _tableHeaderFontSize;
    }

    public String getTableSubHeaderFgColor() {
        return _tableSubHeaderFgColor == null ? getTableHeaderFgColor() : _tableSubHeaderFgColor;
    }

    public String getTableSubHeaderBgColor() {
        return _tableSubHeaderBgColor == null ? getTableHeaderBgColor() : _tableSubHeaderBgColor;
    }

    public String getTableSubHeaderFontFace() {
        return _tableSubHeaderFontFace == null ? getTableHeaderFontFace() : _tableSubHeaderFontFace;
    }

    public String getTableSubHeaderFontSize() {
        return _tableSubHeaderFontSize == null ? getFontSize() : _tableSubHeaderFontSize;
    }

    public String getTableRowBgColor() {
        return _tableRowBgColor == null ? getBgColor() : _tableRowBgColor;
    }

    public String getTableRowFgColor() {
        return _tableRowFgColor == null ? getFgColor() : _tableRowFgColor;
    }

    public String getTableRowFontFace() {
        return _tableRowFontFace == null ? getFontFace() : _tableRowFontFace;
    }

    public String getTableRowFontSize() {
        return _tableRowFontSize == null ? getFontSize() : _tableRowFontSize;
    }

    public String getTableRowPrimaryBgColor() {
        return _tableRowPrimaryBgColor == null ? getTableRowBgColor() : _tableRowPrimaryBgColor;
    }

    public String getTableRowSecondaryBgColor() {
        return _tableRowSecondaryBgColor == null ? getTableRowBgColor() : _tableRowSecondaryBgColor;
    }

    public String getTableRowPrimaryFgColor() {
        return _tableRowPrimaryFgColor == null ? getTableRowFgColor() : _tableRowPrimaryFgColor;
    }

    public String getTableRowSecondaryFgColor() {
        return _tableRowSecondaryFgColor == null ? getTableRowFgColor() : _tableRowSecondaryFgColor;
    }

    public String getTableBorderColor() {
        return _tableBorderColor;
    }

    public String getTableBorderWidth() {
        return _tableBorderWidth;
    }

    public String getTableCellPadding() {
        return _tableCellPadding;
    }

    public String getTableCellSpacing() {
        return _tableCellSpacing;
    }

    public boolean getUseRollOver() {
        return _useRollover;
    }

    public void setFontFace(String fontFace) {
        _fontFace = fontFace;
    }

    public void setFontSize(String fontSize) {
        _fontSize = fontSize;
    }

    public void setFgColor(String fgColor) {
        _fgColor = fgColor;
    }

    public void setBgColor(String bgColor) {
        _bgColor = bgColor;
    }

    public void setTitleFontFace(String titleFontFace) {
        _titleFontFace = titleFontFace;
    }

    public void setTitleFontSize(String titleFontSize) {
        _titleFontFace = titleFontSize;
    }

    public void setTitleFgColor(String titleFgColor) {
        _titleFgColor = titleFgColor;
    }

    public void setLinkFontFace(String linkFontFace) {
        _linkFontFace = linkFontFace;
    }

    public void setLinkFontSize(String linkFontSize) {
        _linkFontSize = linkFontSize;
    }

    public void setLinkActiveColor(String linkActiveColor) {
        _linkActiveColor = linkActiveColor;
    }

    public void setLinkVisitedColor(String linkVisitedColor) {
        _linkVisitedColor = linkVisitedColor;
    }

    public void setTableHeaderFgColor(String tableHeaderFgColor) {
        _tableHeaderFgColor = tableHeaderFgColor;
    }

    public void setTableHeaderBgColor(String tableHeaderBgColor) {
        _tableHeaderBgColor = tableHeaderBgColor;
    }

    public void setTableHeaderFontFace(String tableHeaderFontFace) {
        _tableHeaderFontFace = tableHeaderFontFace;
    }

    public void setTableHeaderFontSize(String tableHeaderFontSize) {
        _tableHeaderFontSize = tableHeaderFontSize;
    }

    public void setTableSubHeaderFgColor(String tableSubHeaderFgColor) {
        _tableSubHeaderFgColor = tableSubHeaderFgColor;
    }

    public void setTableSubHeaderBgColor(String tableSubHeaderBgColor) {
        _tableSubHeaderBgColor = tableSubHeaderBgColor;
    }

    public void setTableSubHeaderFontFace(String tableSubHeaderFontFace) {
        _tableSubHeaderFontFace = tableSubHeaderFontFace;
    }

    public void setTableSubHeaderFontSize(String tableSubHeaderFontSize) {
        _tableSubHeaderFontSize = tableSubHeaderFontSize;
    }

    public void setTableRowBgColor(String tableRowBgColor) {
        _tableRowBgColor = tableRowBgColor;
    }

    public void setTableRowFgColor(String tableRowFgColor) {
        _tableRowFgColor = tableRowFgColor;
    }

    public void setTableRowFontFace(String tableRowFontFace) {
        _tableRowFontFace = tableRowFontFace;
    }

    public void setTableRowFontSize(String tableRowFontSize) {
        _tableRowFontSize = tableRowFontSize;
    }

    public void setTableRowPrimaryBgColor(String tableRowPrimaryBgColor) {
        _tableRowPrimaryBgColor = tableRowPrimaryBgColor;
    }

    public void setTableRowSecondaryBgColor(String tableRowSecondaryBgColor) {
        _tableRowSecondaryBgColor = tableRowSecondaryBgColor;
    }

    public void setTableRowPrimaryFgColor(String tableRowPrimaryFgColor) {
        _tableRowPrimaryFgColor = tableRowPrimaryFgColor;
    }

    public void setTableRowSecondaryFgColor(String tableRowSecondaryFgColor) {
        _tableRowSecondaryFgColor = tableRowSecondaryFgColor;
    }

    public void setTableBorderColor(String tableBorderColor) {
        _tableBorderColor = tableBorderColor;
    }

    public void setTableBorderWidth(String tableBorderWidth) {
        _tableBorderWidth = tableBorderWidth;
    }

    public void setTableCellSpacing(String tableCellSpacing) {
        _tableCellSpacing = tableCellSpacing;
    }

    public void setTableCellPadding(String tableCellPadding) {
        _tableCellPadding = tableCellPadding;
    }

    public String getImageBase() {
        return _imageBase;
    }

    public void setImageBase(String imageBase) {
        _imageBase = imageBase;
    }

    public void setUseRollOver(String useRollover) {
        _useRollover = Boolean.valueOf(useRollover).booleanValue();
    }
}