/*
 * UrlNavigationTag.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.view;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.BodyTagSupport;
import javax.servlet.jsp.tagext.Tag;

import com.eXtropia.app.UserGesture;
import com.eXtropia.app.UserGestureResponse;

/**
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */
public class UrlNavigationTag extends BodyTagSupport {
    protected String _class;
    protected String _baseView;
    protected String _view;
    protected String _targetView;
    protected String _urlSuffix;
    protected String _action;
    protected String _onClick;
    protected String _uri;
    protected String _extraAttributes;
    protected UserGesture _gesture;
    protected UserGestureResponse _gestureResponse;

    // The following was added to allow access to data
    // within the URL Navigation tag
    protected WidgetDataAccessor _dataAccessor;
    protected String _accessorParameter;
    protected String _urlName;
    protected String _realName;

    public UrlNavigationTag() {

    }

    /*  private void appendHiddenInput(StringBuffer buffer, String key, String value)
        {
            buffer.append("<INPUT TYPE=\"HIDDEN\" NAME=\"");
            buffer.append(key);
            buffer.append("\" VALUE=\"");
            buffer.append(value);
            buffer.append("\">\n");
        }
     */

    public int doEndTag() throws JspException {
        if (bodyContent == null)
            return EVAL_PAGE;
        StringBuffer linkBuffer = new StringBuffer(256);
        StringBuffer hrefBuffer = new StringBuffer(256);

        View view = (View) pageContext.getRequest().getAttribute(ViewManager.VIEW_ATTRIBUTE_NAME);
        Map requestParams = getRequestParams();
        Map.Entry entry;
        Object[] multiValue;

        for (Iterator iter = requestParams.entrySet().iterator(); iter.hasNext();) {
            entry = (Map.Entry) iter.next();
            if (entry.getValue() instanceof Object[]) {
                multiValue = (Object[]) entry.getValue();
                for (int i = 0; i < multiValue.length; i++) {
                    if (linkBuffer.length() == 0)
                        linkBuffer.append('?');
                    else
                        linkBuffer.append('&');
                    linkBuffer.append(entry.getKey().toString());
                    linkBuffer.append('=');
                    linkBuffer.append(multiValue[i].toString());
                }
            } else if (entry.getKey().toString().equals(UserGesture.ACTION_REQUEST_PARAM_NAME)) {
                if (linkBuffer.length() == 0)
                    linkBuffer.append('?');
                else
                    linkBuffer.append('&');
                linkBuffer.append(UserGesture.ACTION_REQUEST_PARAM_NAME);
                linkBuffer.append(entry.getValue().toString());
                linkBuffer.append("=1");
            } else {
                if (linkBuffer.length() == 0)
                    linkBuffer.append('?');
                else
                    linkBuffer.append('&');
                linkBuffer.append(entry.getKey().toString());
                linkBuffer.append('=');
                linkBuffer.append(entry.getValue().toString());
            }
        }

        // Added Data Accessor Parameter
        if (_dataAccessor != null && _accessorParameter != null) {
            if (linkBuffer.length() == 0)
                linkBuffer.append('?');
            else
                linkBuffer.append('&');
            //linkBuffer.append(_accessorParameter);

            // yllee
            // specifying more then one accessor parameters

            List params;
            List names;
            params = getValues(_accessorParameter);
            names = getValues(_realName);

            if (names.size() == 0) {
                names = getValues(_urlName);
            }

            if (names.size() != 0 && params.size() < names.size()) {
                throw new JspException("Accessor parameter size should be greater then url name/real name size");
            }

            boolean useName;

            int size = params.size();
            for (int i = 0; i < size; i++) {
                String value = (String) params.get(i);
                useName = (i < names.size());
                if (useName) {
                    linkBuffer.append(names.get(i));
                } else {
                    linkBuffer.append(value);
                }

                linkBuffer.append('=');
                String data = (String) _dataAccessor.getValue(value);
                linkBuffer.append(data == null ? "" : data);

                if (i < (size - 1)) {
                    linkBuffer.append("&");
                }
            }

            //              if (_urlName==null) linkBuffer.append(_accessorParameter);
            //                                else linkBuffer.append(_urlName);
            //              linkBuffer.append('=');
            //              String data = (String)_dataAccessor.getValue(_accessorParameter);
            //              linkBuffer.append( data == null ? "" : data );

            //yllee end
        }

        // add urlSuffix
        if (_urlSuffix != null) {
            if (_urlSuffix.charAt(0) != '#')
                linkBuffer.append(linkBuffer.length() == 0 ? '?' : '&');
            linkBuffer.append(_urlSuffix);
        }

        String requestUri = _uri == null ? ((HttpServletRequest) pageContext.getRequest()).getRequestURI() : _uri;
        String cssClass = _class == null ? "" : "CLASS=\"" + _class + "\" ";
        String onClick = _onClick == null ? "" : "onClick=\"" + _onClick + "\" ";
        String url = requestUri + linkBuffer.toString();
        url = ((HttpServletResponse) pageContext.getResponse()).encodeURL(url);

        hrefBuffer.append("<A ");
        hrefBuffer.append(cssClass);
        hrefBuffer.append(onClick);
        hrefBuffer.append("HREF=\"");
        hrefBuffer.append(url);
        //hrefBuffer.append(requestUri);
        //hrefBuffer.append(linkBuffer.toString());

        hrefBuffer.append("\"");
        if (_extraAttributes != null) {
            hrefBuffer.append(" ").append(_extraAttributes);
        }
        hrefBuffer.append(">");
        hrefBuffer.append(bodyContent.getString());
        hrefBuffer.append("</A>");
        try {
            pageContext.getOut().write(hrefBuffer.toString());
        } catch (IOException ioe) {
            throw new JspException("IOException: " + ioe.getMessage());
        }
        return EVAL_PAGE;
    }

    /**
     * returns the set of values in a String that is delimited by commas.
     * e.g. a,b,c would returns {"a","b","c"}
     * If two consecutive command is found, it would return an empty String.
     * e.g. a,,c would returns {"a","","c"}
     * Note that if the String is ",", {"",""} would be returned.
     * @param str the string to be tokenized
     * @return the set of values in a List
     */
    private static List getValues(String str) {
        ArrayList l = new ArrayList();

        if (str == null) {
            return l;
        }

        if (str.length() == 1 && str.equals(",")) {
            l.add("");
            return l;
        }

        if (str.startsWith(",")) {
            l.add("");
        }

        StringTokenizer st = new StringTokenizer(str, ",", true);
        boolean isToken = false;
        while (st.hasMoreTokens()) {
            String value = st.nextToken();
            if (value.equals(",")) {
                if (isToken) {
                    l.add("");
                } else {
                    isToken = true;
                }
            } else {
                l.add(value);
                isToken = false;
            }
        }
        if (isToken) {
            l.add("");
        }
        return l;
    }

    public int doStartTag() throws JspException {

        //get the data accessor
        Tag tag = this;
        _dataAccessor = null;
        while (_dataAccessor == null && tag != null) {
            if (tag instanceof NextTag && ((NextTag) tag).getIterator() instanceof WidgetDataIteratorTag) {
                _dataAccessor = (WidgetDataAccessor) ((NextTag) tag).getObject();
            } else if (tag instanceof WidgetDataTag) {
                _dataAccessor = ((WidgetDataTag) tag).getDataAccessor();
            } else
                tag = tag.getParent();
        }
        return EVAL_BODY_TAG;
    }

    /**
     * @return a string of hidden form elements with the specified changes and exclustions
     */
    protected Map getRequestParams() {
        Map map = _gesture.getRequestParams();
        map.remove(UserGesture.TARGET_VIEW_REQUEST_PARAM_NAME);
        map.remove(UserGesture.ACTION_REQUEST_PARAM_NAME);
        if (_view != null) {
            map.put(UserGesture.VIEW_REQUEST_PARAM_NAME, _view);
        }
        if (_targetView != null)
            map.put(UserGesture.TARGET_VIEW_REQUEST_PARAM_NAME, _targetView);
        if (_baseView != null)
            map.put(UserGesture.BASE_VIEW_REQUEST_PARAM_NAME, _baseView);
        if (_action != null)
            map.put(UserGesture.ACTION_REQUEST_PARAM_NAME, _action);
        return map;
    }

    public String getUrlSuffix() {
        return _urlSuffix;
    }

    public void setUrlSuffix(String s) {
        _urlSuffix = s;
    }

    public String getCssClass() {
        return _class;
    }

    public String getUri() {
        return _uri;
    }

    public void setUri(String uri) {
        this._uri = uri;
    }

    /**
     * sets the name of the parameter which will be in the url link
     * eg. XXX in http://localhost/webdb/servlet/AddressBook?XXX=YYY
     * If there is multiple parameters specified, it will be in the
     * form of a comma-delimited string eg. X,Y,Z where X, Y and Z
     * are url parameters
     * @param the url parameter name
     */
    public void setRealName(String realName) {
        _realName = realName;
    }

    /**
     * gets the name of the url parameter
     * @return the url name(comma-delimited if multiple)
     */
    public String getRealName() {
        return _realName;
    }

    public String getAction() {
        return _action;
    }

    public void setAction(String s) {
        _action = s;
    }

    public String getAccessorParameter() {
        return _accessorParameter;
    }

    public void setAccessorParameter(String s) {
        _accessorParameter = s;
    }

    /**
     * @deprecated  replaced by {@link #getRealName()}
     */
    public String getUrlName() {
        return _urlName;
    }

    /**
     * @deprecated  replace by {@link #setRealName(String)}
     */
    public void setUrlName(String s) {
        _urlName = s;
    }

    public String getOnClick() {
        return _onClick;
    }

    public void setOnClick(String s) {
        _onClick = s;
    }

    public String getTargetView() {
        return _targetView;
    }

    public String getView() {
        return _view;
    }

    public String getBaseView() {
        return _baseView;
    }

    public void setCssClass(String cssClass) {
        _class = cssClass;
    }

    public void setPageContext(PageContext pc) {
        super.setPageContext(pc);
        ServletRequest request = pc.getRequest();
        _gesture = (UserGesture) request.getAttribute(UserGesture.REQUEST_PARAM_NAME);
        _gestureResponse = (UserGestureResponse) request.getAttribute(UserGestureResponse.REQUEST_PARAM_NAME);
    }

    public void setBaseView(String baseView) {
        _baseView = baseView;
    }

    public void setTargetView(String targetView) {
        _targetView = targetView;
    }

    public void setView(String view) {
        _view = view;
    }

    public void setExtraAttributes(String s) {
        _extraAttributes = s;
    }

    public String getExtraAttributes() {
        return _extraAttributes;
    }

    public void release() {
        super.release();
        _baseView = null;
        _view = null;
        _targetView = null;
        _class = null;
        _action = null;
        _accessorParameter = null;
        _dataAccessor = null;
        _urlName = null;
    }
}
