/*
 * WidgetDataIteratorTag.java
 * Copyright (C) 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.view;

import java.util.Iterator;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.BodyTagSupport;

import com.eXtropia.app.UserGesture;
import com.eXtropia.app.UserGestureResponse;

/**
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */
public class WidgetDataIteratorTag extends BodyTagSupport implements Iterator {
    protected WidgetDataAccessor _widgetDataAccessor;
    protected boolean _nextCalled = false;

    public int doStartTag() throws JspException {
        return EVAL_BODY_TAG;
    }

    public int doAfterBody() throws JspException {
        if (hasNext()) {
            _nextCalled = false;
            return EVAL_BODY_TAG;
        } else {
            return SKIP_BODY;
        }
    }

    public int doEndTag() throws JspException {
        try {
            if (bodyContent != null) {
                bodyContent.writeOut(bodyContent.getEnclosingWriter());
            }
        } catch (Exception e) {
            throw new JspException("IO Error: " + e.getMessage());
        }
        return EVAL_PAGE;
    }

    public boolean hasNext() {
        return _widgetDataAccessor.hasNext();
    }

    public Object next() {
        _nextCalled = true;
        _widgetDataAccessor.next();
        return _widgetDataAccessor;
    }

    public void remove() {
        throw new UnsupportedOperationException();
    }

    public void setAccessor(String accessor) {
        try {
            _widgetDataAccessor = (WidgetDataAccessor) Class.forName(accessor).newInstance();
            UserGesture g = (UserGesture) pageContext.getRequest().getAttribute(UserGesture.REQUEST_PARAM_NAME);
            _widgetDataAccessor.setUserGesture(g);
            UserGestureResponse gr = (UserGestureResponse) pageContext.getRequest().getAttribute(UserGestureResponse.REQUEST_PARAM_NAME);
            _widgetDataAccessor.setUserGestureResponse(gr);
            _widgetDataAccessor.setServletRequest(pageContext.getRequest());
        } catch (Exception e) {
            e.printStackTrace();
            throw new IllegalArgumentException("Failed to instantiate accessor" + e.getMessage());
        }
    }
}
