/*
 * WidgetIteratorTag.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.view;

import java.util.Iterator;
import java.util.NoSuchElementException;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.BodyTagSupport;

/**
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */
public class WidgetIteratorTag extends BodyTagSupport implements Iterator {
    protected boolean _nextCalled = false;

    protected int _currentWidget = -1;
    protected View _view;
    protected String[] _widgetOrder;

    public WidgetIteratorTag() {

    }

    public int doStartTag() throws JspException {
        try {
            _nextCalled = false;
            _currentWidget = -1;
            _widgetOrder = null;
            _view = (View) pageContext.getRequest().getAttribute(ViewManager.VIEW_ATTRIBUTE_NAME);
            //System.out.println("View: " + _view.getName());
            _view.printWidgetNames();
            _widgetOrder = _view.getWidgetOrder();
            //System.out.println("Widget order: " + _widgetOrder.length);
            //for(int i=0;i<_widgetOrder.length;i++)
            //{
            //	System.out.println("Widget [ " + i + "]: " + _widgetOrder[i]);
            //}
        } catch (Exception e) {
            throw new JspException("Exception in doStartTag(): " + e.getMessage());
        }
        if (hasNext())
            return EVAL_BODY_TAG;
        else
            return SKIP_BODY;
    }

    public int doAfterBody() throws JspException {
        if (!_nextCalled) {
            throw new JspException("No NextTag withion WidgetIteratorTag");
        }
        if (hasNext()) {
            _nextCalled = false;
            return EVAL_BODY_TAG;
        } else {
            return SKIP_BODY;
        }
    }

    public int doEndTag() throws JspException {
        try {
            if (bodyContent != null) {
                bodyContent.writeOut(bodyContent.getEnclosingWriter());
            }
        } catch (Exception e) {
            throw new JspException("IO Error: " + e.getMessage());
        }
        return EVAL_PAGE;
    }

    public java.lang.Object next() {
        _nextCalled = true;
        if (hasNext()) {
            //System.out.println("being asked for next one: " + _view.getWidget(_widgetOrder[++_currentWidget]).getName());
            return _view.getWidget(_widgetOrder[++_currentWidget]);
        } else
            throw new NoSuchElementException();
    }

    public void remove() {
        //I don't think so Mr View
        throw new UnsupportedOperationException();
    }

    public boolean hasNext() {
        return _currentWidget < _widgetOrder.length - 1;
    }

    public void release() {
        super.release();
        _nextCalled = false;
        _currentWidget = -1;
        _view = null;
        _widgetOrder = null;
    }
}