/*
 * WidgetTag.java
 * Copyright (C) 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.view;

import java.io.IOException;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.Tag;
import javax.servlet.jsp.tagext.TagSupport;

import com.eXtropia.app.UserGestureResponse;
import com.eXtropia.app.servlet.ActionManager;
import com.eXtropia.app.servlet.ServletUtils;
import com.eXtropia.datahandler.DataHandlerException;
import com.eXtropia.view.widget.Widget;

/**
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */
public class WidgetTag extends TagSupport {
    public static final String LABEL = "label";
    public static final String NAME = "name";
    public static final String ERROR = "error";

    protected String _property;
    protected String _id;
    //anderson - additional attrs
    protected String _additionalAttrs;
    //anderson - real name
    protected String _realName;

    protected Widget _widget;
    protected WidgetDataAccessor _dataAccessor;

    public WidgetTag() {

    }

    public String getId() {
        return _id;
    }

    public void setId(String id) {
        _id = id;
    }

    //anderson - additional attrs
    public String getAdditionalAttrs() {
        return _additionalAttrs;
    }

    //anderson - additional attrs
    public void setAdditionalAttrs(String attrs) {
        this._additionalAttrs = attrs;
    }

    //anderson - real name
    public String getRealName() {
        return _realName;
    }

    //anderson - real name
    public void setRealName(String name) {
        this._realName = name;
    }

    public String getProperty() {
        return _property;
    }

    public void setProperty(String property) {
        _property = property;
    }

    public int doStartTag() throws JspException {
        Tag tag = this;
        //get the widget
        if (_id == null) {
            _widget = null; // To satisfy JRun 3.1
            while (_widget == null && tag != null) {
                if (tag instanceof NextTag && ((NextTag) tag).getIterator() instanceof WidgetIteratorTag) {
                    _widget = (Widget) ((NextTag) tag).getObject();
                } else
                    tag = tag.getParent();
            }
        } else {
            View view = (View) pageContext.getRequest().getAttribute(ViewManager.VIEW_ATTRIBUTE_NAME);
            _widget = view.getWidget(_id);
        }

        if (_widget == null)
            throw new JspException("No Widget for WidgetTag" + (_id != null ? ": " + _id : ""));

        //anderson - additional attrs
        _widget.setAdditionalAttrs(_additionalAttrs);
        //anderson - real name
        _widget.setRealName(_realName);
        //get the data accessor
        tag = this;
        // we start off with a null data accessor if this is truly the start tag.
        // added to fix jrun 3.1 problem (GB)
        _dataAccessor = null;
        while (_dataAccessor == null && tag != null) {
            if (tag instanceof NextTag && ((NextTag) tag).getIterator() instanceof WidgetDataIteratorTag) {
                _dataAccessor = (WidgetDataAccessor) ((NextTag) tag).getObject();
            } else if (tag instanceof WidgetDataTag) {
                _dataAccessor = ((WidgetDataTag) tag).getDataAccessor();
            } else
                tag = tag.getParent();
        }

        return SKIP_BODY;
    }

    public int doEndTag() throws JspException {
        Locale locale = (Locale) pageContext.getSession().getAttribute(ServletUtils.SESSION_ATTRIBUTE_LOCALE);
        String value = "";
        if (_property != null) {
            if (_property.equals(LABEL)) {
                value = _widget.getLabel(locale);
                if (value == null) {
                    value = _widget.getName();
                }
            } else if (_property.equals(ERROR)) {
                //get the datahandlerexception for this widget
                value = getDataHandlerError(locale);
            } else if (_property.equals(NAME)) {
                value = _widget.getName();
            }
        } else {
            value = getWidgetValue(locale);
        }
        try {
            pageContext.getOut().write(value);
        } catch (IOException e) {
            throw new JspException("IOException: " + e);
        }
        return EVAL_PAGE;
    }

    protected String getWidgetValue(Locale locale) {
        if (_dataAccessor != null) {
            String idSuffix = _dataAccessor.getIdSuffix();
            try {
                if (idSuffix == null)
                    return _widget.toString(locale, _dataAccessor.getValue(_widget.getId()), pageContext.getRequest());
                else
                    return _widget.toString(locale, _dataAccessor.getValue(_widget.getId()), pageContext.getRequest(), idSuffix);
            } catch (Exception e) {
                //System.out.println("Exception whilst displaying widget: " + _id + ": " + e.getMessage());
                e.printStackTrace();
                return "";
            }
        } else
            return _widget.toString(locale, null, pageContext.getRequest());
    }

    public void release() {
        super.release();
        _property = null;
        _id = null;
        _widget = null;
        _dataAccessor = null;
    }

    protected String getDataHandlerError(Locale locale) {
        UserGestureResponse gestureResponse = (UserGestureResponse) pageContext.getRequest().getAttribute(UserGestureResponse.REQUEST_PARAM_NAME);
        DataHandlerException topLevelError = gestureResponse.getDataHandlerException();
        String error = "";
        if (topLevelError != null) {
            DataHandlerException firstError = topLevelError.getFirstError(ActionManager.WIDGET_RULE_NAME + "." + _widget.getId());
            if (firstError != null) {
                View view = (View) pageContext.getRequest().getAttribute(ViewManager.VIEW_ATTRIBUTE_NAME);
                ResourceBundle bundle = ResourceBundle.getBundle(view.getResourceLocation(), locale);
                error = firstError.getResourceName();
                try {
                    error = bundle.getString(error);
                } catch (MissingResourceException mre) {
                }
            }
        }
        return error;
    }
}
