/*
 * PlainText.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.view.widget;

import java.text.DecimalFormat;
import java.text.MessageFormat;
import java.text.SimpleDateFormat;
import java.util.Locale;
import java.util.ResourceBundle;

import javax.servlet.ServletRequest;

import com.eXtropia.config.Config;
import com.eXtropia.config.ConfigException;

/**
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 0.1
 */
public class PlainText extends Widget {
    public static final String CFG_PREFIX = "Prefix";
    public static final String CFG_SUFFIX = "Suffix";
    public static final String WIDGET_RESOURCE_PREFIX = ".prefix";
    public static final String WIDGET_RESOURCE_SUFFIX = ".suffix";
    public static final String CFG_FORMAT = "Format";
    protected String _format;

    public void configure(Config config) throws ConfigException {
        super.configureWidgetBase(config);
        if (config.hasChild(CFG_FORMAT))
            _format = config.getChild(CFG_FORMAT).getValue();
    }

    public String toString(Locale locale, Object value, ServletRequest request, String idSuffix) {
        return toString(locale, value, request);
    }

    public String toString(Locale locale, Object value, ServletRequest request) {
        String prefix = null, suffix = null;
        try {
            ResourceBundle bundle = ResourceBundle.getBundle(_resourceLocation, locale);
            try {
                prefix = bundle.getString(_resourceBase + WIDGET_RESOURCE_PREFIX);
            } catch (Exception e) {
            }
            try {
                suffix = bundle.getString(_resourceBase + WIDGET_RESOURCE_SUFFIX);
            } catch (Exception e) {
            }
        } catch (Exception e) {
        }

        //value= selectValueToUse(value, request);
        if (value == null && _default)
            value = getDefaultValue(locale);
        String newValueString = (value == null ? "" : value.toString());
        if (_format != null && value != null && !value.equals("")) {
            Object newValue = value;
            try {
                if (_format.indexOf("number") > -1) {
                    newValue = (new DecimalFormat()).parse(value.toString());
                }
                if (_format.indexOf("date") > -1 || _format.indexOf("time") > -1) {
                    //anderson - the value obtained might be a Date object already
                    if (value instanceof String)
                        newValue = (new SimpleDateFormat()).parse(value.toString());
                }
            } catch (java.text.ParseException e) {
                e.printStackTrace();
                newValue = null;
            }
            if (newValue != null) {
                newValueString = MessageFormat.format("{0," + _format + "}", new Object[] { newValue });
            }
            value = newValueString;
        }

        if (value == null)
            return "";
        else {
            StringBuffer buffer = new StringBuffer();
            if (prefix != null)
                buffer.append(prefix);
            buffer.append(value.toString());
            if (suffix != null)
                buffer.append(suffix);
            return buffer.toString();
        }
    }
}
