/* 
 * ComboInput.java
 * Copyright (C) 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.view.widget.html;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import javax.servlet.ServletRequest;

import com.eXtropia.config.Config;
import com.eXtropia.config.ConfigException;
import com.eXtropia.util.HtmlEncoder;
import com.eXtropia.view.widget.Widget;

/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */
public class ComboInput extends Widget {
    public static final String CFG_ALLOW_NULL = "AllowNull";
    public static final String CFG_OPTIONS = "Options";
    public static final String CFG_OPTION = "Option";
    public static final String RESOURCE_OPTION_EXTENSION = ".option.";

    protected boolean _allowNull = false;
    protected List _options = new ArrayList();

    public ComboInput() {

    }

    public void configure(Config config) throws ConfigException {
        super.configure(config);
        if (config.hasChild(CFG_ALLOW_NULL))
            setAllowNull(config.getChild(CFG_ALLOW_NULL).getBooleanValue());
        if (config.hasChild(CFG_OPTIONS)) {
            Config[] children = config.getChild(CFG_OPTIONS).getChildren();
            for (int i = 0; i < children.length; i++) {
                if (children[i].getName().equals(CFG_OPTION))
                    _options.add(children[i].getValue());
                else
                    throw new ConfigException("Only " + CFG_OPTION + " configuration is allowed within a " + CFG_OPTIONS + " for widget " + _name);
            }
        } else
            throw new ConfigException("No options specified for widget " + _name);
    }

    public void setAllowNull(boolean allowNull) {
        _allowNull = allowNull;
    }

    public String toString(Locale locale, Object value, ServletRequest request) {
        return toString(locale, value, request, null);
    }

    public String toString(Locale locale, Object value, ServletRequest request, String idSuffix) {
        StringBuffer buffer = new StringBuffer(256);
        String strValue = "";
        String currentOption;
        ResourceBundle bundle = ResourceBundle.getBundle(_resourceLocation, locale);
        value = selectValueToUse(value, request);

        if (value != null)
            strValue = value.toString();

        buffer.append("<SELECT NAME=\"");
        buffer.append(_id);
        if (idSuffix != null)
            buffer.append(idSuffix);
        buffer.append("\">");
        for (Iterator i = _options.iterator(); i.hasNext();) {
            currentOption = i.next().toString();
            buffer.append("<OPTION VALUE=\"");
            buffer.append(HtmlEncoder.encode(currentOption));
            buffer.append("\"");
            buffer.append(currentOption.equals(strValue) ? " SELECTED>" : ">");
            try {
                buffer.append(HtmlEncoder.encode(bundle.getString(_resourceBase + RESOURCE_OPTION_EXTENSION + currentOption)));
            } catch (MissingResourceException mre) {
                buffer.append(HtmlEncoder.encode(currentOption));
            }
        }
        buffer.append("</SELECT>");
        return buffer.toString();
    }

}