/* 
 * DateDisplay.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.view.widget.html;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import javax.servlet.ServletRequest;

import com.eXtropia.config.Config;
import com.eXtropia.config.ConfigException;
import com.eXtropia.view.widget.Widget;

/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */
public class DateDisplay extends Widget {
    public static final String CFG_FORMAT = "Format";
    public static final String CFG_TYPE = "Type";
    public static final String CFG_STYLE_DATE = "DateStyle";
    public static final String CFG_STYLE_TIME = "TimeStyle";

    public static final String CFG_TYPE_DATE = "date";
    public static final String CFG_TYPE_TIME = "time";

    public static final String CFG_STYLE_SHORT = "short";
    public static final String CFG_STYLE_MEDIUM = "medium";
    public static final String CFG_STYLE_DEFAULT = "default";
    public static final String CFG_STYLE_LONG = "long";

    private static final int TYPE_DATE = 1;
    private static final int TYPE_TIME = 2;
    private static final int TYPE_DATE_TIME = 0;

    private int _type = TYPE_DATE_TIME;
    private int _dateStyle = DateFormat.DEFAULT;
    private int _timeStyle = DateFormat.DEFAULT;

    protected String _format;
    //DateFormat _format= DateFormat.getInstance().getNumberFormat().get;

    /** Creates new DateDisplay */
    public DateDisplay() {

    }

    public void configure(Config config) throws ConfigException {
        super.configure(config);
        if (config.hasChild(CFG_FORMAT)) {
            setFormat(config.getChild(CFG_FORMAT).getValue());
        } else {
            if (config.hasChild(CFG_TYPE_DATE)) {
                String type = config.getChild(CFG_TYPE).getValue().toLowerCase();
                if (type.equals(CFG_TYPE_DATE))
                    _type = TYPE_DATE;
                else if (type.equals(CFG_TYPE_TIME))
                    _type = TYPE_TIME;
                else
                    _type = TYPE_DATE_TIME;
            }
            //get out the styles
            if (config.hasChild(CFG_STYLE_DATE))
                _dateStyle = getStyle(config.getChild(CFG_STYLE_DATE).getValue());
            if (config.hasChild(CFG_STYLE_TIME))
                _timeStyle = getStyle(config.getChild(CFG_STYLE_TIME).getValue());
        }
    }

    private int getStyle(String style) {
        style = style.toLowerCase();
        int intStyle;
        if (style.equals(CFG_STYLE_SHORT))
            intStyle = DateFormat.SHORT;
        else if (style.equals(CFG_STYLE_MEDIUM))
            intStyle = DateFormat.MEDIUM;
        else if (style.equals(CFG_STYLE_LONG))
            intStyle = DateFormat.LONG;
        else
            intStyle = DateFormat.DEFAULT;
        return intStyle;
    }

    public void setFormat(String format) {
        _format = format;
    }

    public String toString(Locale locale, Object value, ServletRequest request, String idSuffix) {
        return toString(locale, value, request);
    }

    public String toString(Locale locale, Object value, ServletRequest request) {
        value = selectValueToUse(value, request);
        if (value == null)
            return "No Date Specified";
        if (_format == null) {
            DateFormat df;
            switch (_type) {
                case TYPE_DATE :
                    df = DateFormat.getDateInstance(_dateStyle, locale);
                    break;
                case TYPE_TIME :
                    df = DateFormat.getTimeInstance(_timeStyle, locale);
                    break;
                default :
                    df = DateFormat.getDateTimeInstance(_dateStyle, _timeStyle, locale);
                    break;
            }
            return df.format((Date) value);
        } else
            return new SimpleDateFormat(_format, locale).format((Date) value);
    }
}