/*
 * DropDownAccessor.java
 * Copyright (C) 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.view.widget.html;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import java.util.StringTokenizer;

import javax.servlet.ServletRequest;

import com.eXtropia.config.Config;
import com.eXtropia.config.ConfigException;
import com.eXtropia.util.HtmlEncoder;
import com.eXtropia.view.WidgetDataAccessor;

/**
 *
 * @author  Gunther Birznieks (gunther.birznieks@eXtropia.com)
 * @version 1.0
 */
public class DropDownAccessor extends ComboInput {
    public static final String CFG_ALLOW_NULL = "AllowNull";
    public static final String CFG_DROP_DOWN_ACCESSOR = "DisplayValueAccessor";
    public static final String CFG_VALUE_ID = "ValueId";
    public static final String CFG_DISPLAY_ID = "DisplayId";
    public static final String CFG_RESOURCE_FILE = "ResourceFile";

    public static final String RESOURCE_OPTION_EXTENSION = ".option.";

    public static final String DEFAULT_VALUE = "DefaultValue";
    public static final String ORDER = "Order";

    protected List _options = new ArrayList();
    protected List _display = new ArrayList();
    protected String _valueId;
    protected String _displayId;
    protected String _resourceFile;
    protected Config _accessConfig;
    String defaultValue;
    private Map[] _data;
    private Map ordering;

    public DropDownAccessor() {

    }

    public void configure(Config config) throws ConfigException {
        // We want to call the widget's configure method but we don't want
        // the restrictions of the immediate superclass...
        this.configureWidgetBase(config);

        if (config.hasChild(CFG_DROP_DOWN_ACCESSOR)) {
            _accessConfig = config.getChild(CFG_DROP_DOWN_ACCESSOR);
            if (_accessConfig.hasChild(CFG_VALUE_ID)) {
                _valueId = _accessConfig.getChild(CFG_VALUE_ID).getValue();
                if (_accessConfig.hasChild(CFG_DISPLAY_ID)) {
                    _displayId = _accessConfig.getChild(CFG_DISPLAY_ID).getValue();
                }
                if (_accessConfig.hasChild(CFG_RESOURCE_FILE)) {
                    _resourceFile = _accessConfig.getChild(CFG_RESOURCE_FILE).getValue();
                    loadValues(_resourceFile);
                }
            } else {
                throw new ConfigException("No value id specified for the accessor in the drop down widget " + _name);
            }
        } else if (config.hasChild(CFG_RESOURCE_FILE)) {
            _valueId = "name";
            _displayId = "value";
            _resourceFile = config.getChild(CFG_RESOURCE_FILE).getValue();
            loadValues(_resourceFile);

        } else
            throw new ConfigException("No accessor specified for this drop down widget " + _name);
    }

    public String toString(Locale locale, Object value, ServletRequest request) {
        return toString(locale, value, request, null);
    }

    public String toString(Locale locale, Object value, ServletRequest request, String idSuffix) {
        // Get the data accessor options filled...

        WidgetDataAccessor dropDownAccessor = null;
        if (_valueId != null) {
            // We have to first create the accessor
            try {
                String accessorClassName = "com.eXtropia.view.DefaultWidgetDataAccessor";
                if (_accessConfig != null)
                    accessorClassName = _accessConfig.getType();
                dropDownAccessor = (WidgetDataAccessor) Class.forName(accessorClassName).newInstance();
                if (_resourceFile != null)
                    dropDownAccessor.setResource(_data);
            } catch (Exception e) {
                e.printStackTrace();
            } catch (Throwable e) {
                e.printStackTrace();
            }
            dropDownAccessor.setServletRequest(request);
        }

        StringBuffer buffer = new StringBuffer(256);
        String strValue = "";
        String currentOption;
        ResourceBundle bundle = ResourceBundle.getBundle(_resourceLocation, locale);
        value = selectValueToUse(value, request);

        if (value != null) {
            strValue = value.toString();
        } else if (_default) {
            strValue = (String) getDefaultValue(locale);
        } else if (defaultValue != null && defaultValue.length() > 0) {
            strValue = defaultValue;
        }

        buffer.append("<SELECT NAME=\"");
        //anderson - real name
        if (_realName != null)
            buffer.append(_realName);
        else
            buffer.append(_id);
        if (idSuffix != null)
            buffer.append(idSuffix);
        //anderson - additional attrs
        buffer.append("\"");
        if (_additionalAttrs != null)
            buffer.append(" " + _additionalAttrs);
        buffer.append(">\n");
        //buffer.append("\">\n");
        for (; dropDownAccessor.hasNext();) {
            dropDownAccessor.next();
            currentOption = dropDownAccessor.getValue(_valueId).toString();
            buffer.append("<OPTION VALUE=\"");
            buffer.append(HtmlEncoder.encode((currentOption)));
            buffer.append("\"");
            buffer.append(currentOption.equals(strValue) ? " SELECTED>" : ">");
            if (_displayId != null) {
                buffer.append((String) dropDownAccessor.getValue(_displayId));
            } else {
                try {
                    buffer.append(HtmlEncoder.encode((bundle.getString(_resourceBase + RESOURCE_OPTION_EXTENSION + currentOption))));
                } catch (MissingResourceException mre) {
                    buffer.append(currentOption);
                }
            }
            buffer.append("\n");
        }
        buffer.append("</SELECT>\n");
        return buffer.toString();
    }

    private void loadValues(String resource) {
        try {
            ResourceBundle bundle = ResourceBundle.getBundle(resource);
            try {
                defaultValue = bundle.getString(DEFAULT_VALUE);
            } catch (java.util.MissingResourceException e) {
                System.err.println("NO default value");
            }
            try {
                String str = bundle.getString(ORDER);
                ordering = getOrdering(str);
            } catch (MissingResourceException e) {
            }

            Map data = null;
            ArrayList datalist = new ArrayList();
            String key = null;
            for (Enumeration e = bundle.getKeys(); e.hasMoreElements();) {
                key = (String) e.nextElement();
                if (!key.equals(CFG_VALUE_ID) && !key.equals(CFG_DISPLAY_ID) && !key.equals(DEFAULT_VALUE) && !key.equals(ORDER)) {
                    data = new HashMap();
                    data.put(_valueId, key);
                    data.put(_displayId, bundle.getString(key));
                    datalist.add(data);
                }
            }
            _data = new HashMap[datalist.size()];
            datalist.toArray(_data);

            if (ordering != null && ordering.size() == _data.length) {
                Map[] sorted = new Map[_data.length];
                for (int i = 0; i < _data.length; i++) {
                    sorted[((Integer) ordering.get(_data[i].get(_valueId))).intValue()] = _data[i];
                }
                _data = sorted;
            } else {
                Arrays.sort(_data, new java.util.Comparator() {
                    public int compare(Object o1, Object o2) {
                        //                        if (defaultValue!=null) {
                        //                            if ((((HashMap)o1).get(_valueId)).equals(defaultValue)) return -1;
                        //                            if ((((HashMap)o2).get(_valueId)).equals(defaultValue)) return 1;
                        //                        }
                        String word1 = (String) ((HashMap) o1).get(_displayId);
                        String word2 = (String) ((HashMap) o2).get(_displayId);
                        return word1.compareTo(word2);
                    }
                });
            }
        } catch (Exception e) {
            e.printStackTrace();
            System.err.println("DataAccessor Exception during loading properties : " + e.getMessage());
        }
    }

    private Map getOrdering(String str) {
        Map map = new HashMap();
        StringTokenizer st = new StringTokenizer(str, ",");
        int order = 0;

        while (st.hasMoreTokens()) {
            map.put(st.nextToken(), new Integer(order++));
        }
        return map;
    }

}
