/*
 * HtmlInput.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.view.widget.html;

import java.text.DecimalFormat;
import java.text.MessageFormat;
import java.text.SimpleDateFormat;
import java.util.Locale;
import java.util.Vector;

import javax.servlet.ServletRequest;

import com.eXtropia.config.Config;
import com.eXtropia.config.ConfigException;
import com.eXtropia.datahandler.DataHandler;
import com.eXtropia.view.widget.Widget;

/**
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 0.1
 */
public abstract class HtmlInput extends Widget {
    public static final String CFG_FORMAT = "Format";
    protected String _type;
    protected Attribute[] _attributes;

    protected String _format;

    protected HtmlInput(String type) {
        _type = type;
    }

    public void configure(Config config) throws ConfigException {
        super.configure(config);
        Config[] children = config.getChildren();
        Vector attributes = new Vector();
        String childName;

        for (int i = 0; i < children.length; i++) {
            childName = children[i].getName();
            if (!(childName.equals(CFG_ID) || childName.equals(CFG_NAME) || childName.equals(DataHandler.CFG_BASE) || childName.equals(CFG_REQUIRED) || childName.equals(CFG_DEFAULT)))
                attributes.addElement(new Attribute(children[i].getName(), children[i].getValue()));
        }
        _attributes = (Attribute[]) attributes.toArray(new Attribute[attributes.size()]);

        if (config.hasChild(CFG_FORMAT))
            _format = config.getChild(CFG_FORMAT).getValue();
    }

    public String getType() {
        return _type;
    }

    public String toString(Locale locale, Object value, ServletRequest request) {
        return toString(locale, value, request, null);
    }

    public String toString(Locale locale, Object value, ServletRequest request, String idSuffix) {
        value = selectValueToUse(value, request);
        if (value == null && _default)
            value = getDefaultValue(locale);

        StringBuffer buffer = new StringBuffer(50);
        buffer.append("<INPUT TYPE=\"");
        buffer.append(_type);
        buffer.append("\" NAME=\"");
        //anderson - real name
        if (_realName != null)
            buffer.append(_realName);
        else
            buffer.append(_id);

        if (idSuffix != null)
            buffer.append(idSuffix);
        buffer.append("\" VALUE=\"");
        String newValueString = (value == null ? "" : value.toString());
        if (_format != null && value != null) {
            Object newValue = value;
            try {
                if (_format.indexOf("number") > -1) {
                    //anderson - try to solve the class cast exception for BigDecimal...
                    if (value instanceof String)
                        newValue = (new DecimalFormat()).parse((String) value);
                    else
                        newValue = value;
                }
                if (_format.indexOf("date") > -1 || _format.indexOf("time") > -1) {
                    //anderson - the value obtained might be a Date object already
                    if (value instanceof String)
                        newValue = (new SimpleDateFormat()).parse((String) value);
                }
            } catch (java.text.ParseException e) {
                //e.printStackTrace();
                newValue = null;
            }
            if (newValue != null) {
                newValueString = MessageFormat.format("{0," + _format + "}", new Object[] { newValue });
            }
        }
        buffer.append((newValueString == null ? "" : encode(newValueString.toString())) + "\"");
        appendAttributes(buffer);
        if (_additionalAttrs != null)
            buffer.append(" " + _additionalAttrs);
        buffer.append('>');
        return buffer.toString();
    }

    private static final String encode(String str) {
        StringBuffer sb = new StringBuffer();

        int len = str.length();
        for (int i = 0; i < len; i++) {
            char c = str.charAt(i);
            if (c == '"') {
                sb.append("&quot;");
            } else {
                sb.append(c);
            }
        }
        return sb.toString();
    }

    protected void appendAttributes(StringBuffer buffer) {
        String key;
        for (int i = 0; i < _attributes.length; i++) {
            buffer.append(' ');
            buffer.append(_attributes[i].key);
            buffer.append("=\"");
            buffer.append(_attributes[i].value);
            buffer.append('"');
        }
    }

    protected class Attribute {
        String key;
        String value;

        public Attribute(String k, String v) {
            key = k;
            value = v;
        }
    }
}
