/*
 * HtmlPlainText.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.view.widget.html;

import java.text.DecimalFormat;
import java.text.MessageFormat;
import java.text.SimpleDateFormat;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.ResourceBundle;

import javax.servlet.ServletRequest;

import com.eXtropia.config.Config;
import com.eXtropia.config.ConfigException;
import com.eXtropia.view.widget.Widget;

/**
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 0.1
 */
public class HtmlPlainText extends Widget {
    public static final String CFG_PREFIX = "Prefix";
    public static final String CFG_SUFFIX = "Suffix";
    public static final String CFG_FORMAT = "Format";
    public static final String CFG_RESOURCE_FILE = "ResourceFile";
    public static final String CFG_BLANK_DISPLAY_STRING = "BlankDisplayString";
    public static final String WIDGET_RESOURCE_PREFIX = ".prefix";
    public static final String WIDGET_RESOURCE_SUFFIX = ".suffix";

    protected String _blankDisplayString = " ";
    protected String _format;
    private Map map = new HashMap();

    public void configure(Config config) throws ConfigException {
        super.configure(config);
        if (config.hasChild(CFG_BLANK_DISPLAY_STRING))
            _blankDisplayString = config.getChild(CFG_BLANK_DISPLAY_STRING).getValue();
        if (config.hasChild(CFG_FORMAT))
            _format = config.getChild(CFG_FORMAT).getValue();
        if (config.hasChild(CFG_RESOURCE_FILE)) {
            loadValues(config.getChild(CFG_RESOURCE_FILE).getValue());
        }
    }

    private void loadValues(String resourceFile) {
        ResourceBundle bundle = ResourceBundle.getBundle(resourceFile);
        for (Enumeration e = bundle.getKeys(); e.hasMoreElements();) {
            String key = (String) e.nextElement();
            if (!DropDownAccessor.DEFAULT_VALUE.equals(key) && !DropDownAccessor.ORDER.equals(key)) {
                map.put(key, bundle.getString(key));
            }
        }

    }

    public String toString(Locale locale, Object value, ServletRequest request, String idSuffix) {
        return toString(locale, value, request);
    }

    public String toString(Locale locale, Object value, ServletRequest request) {
        String prefix = null, suffix = null;
        try {
            ResourceBundle bundle = ResourceBundle.getBundle(_resourceLocation, locale);
            try {
                prefix = bundle.getString(_resourceBase + WIDGET_RESOURCE_PREFIX);
            } catch (Exception e) {
            }
            try {
                suffix = bundle.getString(_resourceBase + WIDGET_RESOURCE_SUFFIX);
            } catch (Exception e) {
            }
        } catch (Exception e) {
        }

        //value= selectValueToUse(value, request);
        if (value == null && _default)
            value = getDefaultValue(locale);

        String stringValue = null;
        if (value != null && value instanceof String)
            stringValue = (String) value;

        if (stringValue != null && !map.isEmpty()) {
            return com.eXtropia.util.HtmlEncoder.encode((String) map.get(value.toString().trim()));
        }

        if (value == null || (stringValue != null && stringValue.trim().length() == 0)) {
            return com.eXtropia.util.HtmlEncoder.encode(_blankDisplayString.toString());
        } else {
            StringBuffer buffer = new StringBuffer();
            String newValueString = (value == null ? "" : value.toString());
            if (_format != null && value != null && !value.equals("")) {
                try {
                    if (_format.indexOf("number") > -1) {
                        // the value obtained might be a number already
                        if (value instanceof String) {
                            value = (new DecimalFormat()).parse((String) value);
                        }
                    }
                    if (_format.indexOf("date") > -1 || _format.indexOf("time") > -1) {
                        //anderson - the value obtained might be a Date object already
                        if (value instanceof String)
                            value = (new SimpleDateFormat()).parse((String) value);
                    }
                } catch (java.text.ParseException e) {
                    e.printStackTrace();
                }
                newValueString = MessageFormat.format("{0," + _format + "}", new Object[] { value });
            }
            if (prefix != null)
                buffer.append(prefix);
            buffer.append(newValueString);
            if (suffix != null)
                buffer.append(suffix);
            return com.eXtropia.util.HtmlEncoder.encode(buffer.toString());
        }
    }

}
