/*
 * HtmlPlainTextAccessor.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.view.widget.html;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.ResourceBundle;

import javax.servlet.ServletRequest;

import com.eXtropia.config.Config;
import com.eXtropia.config.ConfigException;
import com.eXtropia.view.WidgetDataAccessor;

/**
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 0.1
 */
public class HtmlPlainTextAccessor extends HtmlPlainText {

    public static final String CFG_DISPLAY_VALUE_ACCESSOR = "DisplayValueAccessor";
    public static final String CFG_VALUE_ID = "ValueId";
    public static final String CFG_DISPLAY_ID = "DisplayId";
    public static final String CFG_RESOURCE_FILE = "ResourceFile";

    protected String _valueId;
    protected String _displayId;
    protected Config _accessConfig;
    protected String _resourceFile;
    String defaultValue;
    private Map[] _data;
    public final String DEFAULT_VALUE = "DefaultValue";

    public void configure(Config config) throws ConfigException {
        super.configure(config);

        if (config.hasChild(CFG_DISPLAY_VALUE_ACCESSOR)) {
            _accessConfig = config.getChild(CFG_DISPLAY_VALUE_ACCESSOR);
            if (_accessConfig.hasChild(CFG_VALUE_ID)) {
                _valueId = _accessConfig.getChild(CFG_VALUE_ID).getValue();
                if (_accessConfig.hasChild(CFG_DISPLAY_ID)) {
                    _displayId = _accessConfig.getChild(CFG_DISPLAY_ID).getValue();
                }
                if (_accessConfig.hasChild(CFG_RESOURCE_FILE)) {
                    _resourceFile = _accessConfig.getChild(CFG_RESOURCE_FILE).getValue();
                    System.out.println("HTMLPlainTextAccessor = " + _resourceFile);
                    loadValues(_resourceFile);
                }
            } else {
                throw new ConfigException("No value id specified for the accessor in the html plain text widget " + _name);
            }
        } else
            throw new ConfigException("No accessor specified for this html plain text widget " + _name);

    }

    public String toString(Locale locale, Object value, ServletRequest request, String idSuffix) {
        return toString(locale, value, request);
    }

    public String toString(Locale locale, Object value, ServletRequest request) {
        Object newValue = value;
        // Get the data accessor options filled...

        WidgetDataAccessor displayValueAccessor = null;
        if (_valueId != null && _displayId != null) {
            // We have to first create the accessor
            try {
                displayValueAccessor = (WidgetDataAccessor) Class.forName(_accessConfig.getType()).newInstance();
                if (_resourceFile != null)
                    displayValueAccessor.setResource(_data);
            } catch (Exception e) {
                e.printStackTrace();
            }
            displayValueAccessor.setServletRequest(request);
        }

        // OK lets get the real value
        if (displayValueAccessor != null) {
            while (displayValueAccessor.hasNext()) {
                displayValueAccessor.next();
                String compareValue = (String) displayValueAccessor.getValue(_valueId);
                if (compareValue.equals(value)) {
                    newValue = (String) displayValueAccessor.getValue(_displayId);
                }
            }
        }

        return super.toString(locale, newValue, request);

    }

    private void loadValues(String resource) {
        try {

            ResourceBundle bundle = ResourceBundle.getBundle(resource);
            try {
                defaultValue = bundle.getString(DEFAULT_VALUE);
            } catch (java.util.MissingResourceException e) {
                defaultValue = null;
                System.err.println("NO default value");
            }
            Map data = null;
            ArrayList datalist = new ArrayList();
            String key = null;
            for (Enumeration e = bundle.getKeys(); e.hasMoreElements();) {
                key = (String) e.nextElement();
                if (!key.equals(CFG_VALUE_ID) && !key.equals(CFG_DISPLAY_ID) && !key.equals(DEFAULT_VALUE)) {
                    data = new HashMap();
                    data.put(_valueId, key);
                    data.put(_displayId, bundle.getString(key));
                    datalist.add(data);
                }
            }
            _data = new HashMap[datalist.size()];
            datalist.toArray(_data);
            Arrays.sort(_data, new java.util.Comparator() {
                public int compare(Object o1, Object o2) {
                    if (defaultValue != null) {
                        if (((String) ((HashMap) o1).get(_valueId)).equals(defaultValue))
                            return -1;
                        if (((String) ((HashMap) o2).get(_valueId)).equals(defaultValue))
                            return 1;
                    }
                    String word1 = (String) ((HashMap) o1).get(_displayId);
                    String word2 = (String) ((HashMap) o2).get(_displayId);
                    return word1.compareTo(word2);
                }
            });
        } catch (Exception e) {
            System.err.println("DataAccessor Exception during loading properties : " + e.getMessage());
        }
    }

}