/* 
 * MultiTextInput.java
 * Copyright (C) 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.view.widget.html;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import javax.servlet.ServletRequest;

import com.eXtropia.config.Config;
import com.eXtropia.config.ConfigException;
import com.eXtropia.view.widget.Widget;

/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */
public class MultiTextInput extends Widget {
    public static final String CFG_INPUTS = "Inputs";
    public static final String CFG_INPUT = "Input";
    public static final String CFG_INPUT_SIZE = "Size";
    public static final String CFG_INPUT_MAXLENGTH = "MaxLength";

    public static final String RESOURCE_INPUT_SEPARATOR = ".separator";

    private Input[] _inputs;

    public MultiTextInput() {
    }

    protected void appendInput(Input input, int position, Object value, String idSuffix, StringBuffer buffer) {
        buffer.append("<INPUT TYPE=\"TEXT\" NAME=\"");
        buffer.append(_id + (idSuffix == null ? "" : idSuffix) + "_" + position);
        buffer.append("\" VALUE=\"");
        buffer.append(value == null ? "" : value.toString());
        buffer.append("\"");
        if (input.size > 0) {
            buffer.append(" SIZE=\"");
            buffer.append(input.size);
            buffer.append("\"");
        }
        if (input.maxlength > 0) {
            buffer.append(" MAXLENGTH=\"");
            buffer.append(input.maxlength);
            buffer.append("\"");
        }
        buffer.append(">");
    }

    public void configure(Config config) throws ConfigException {
        super.configure(config);
        List list = new ArrayList();
        if (config.hasChild(CFG_INPUTS)) {
            Config[] children = config.getChild(CFG_INPUTS).getChildren();
            int size;
            int maxlength;
            for (int i = 0; i < children.length; i++) {
                if (children[i].getName().equals(CFG_INPUT)) {
                    size = maxlength = 0;
                    try {

                        size = children[i].getChild(CFG_INPUT_SIZE).getIntValue();
                    } catch (Exception e) {
                    }
                    try {
                        maxlength = children[i].getChild(CFG_INPUT_MAXLENGTH).getIntValue();
                    } catch (Exception e) {
                    }

                    list.add(new Input(size, maxlength));
                } else
                    throw new ConfigException(CFG_INPUTS + " may only contain " + CFG_INPUT + " in widget " + _name);
            }
            _inputs = (Input[]) list.toArray(new Input[list.size()]);
        } else
            throw new ConfigException("No " + CFG_INPUTS + " specified in widget " + _name);

    }

    protected Object selectValueToUse(int i, Object value, ServletRequest request) {
        Object requestValue = request.getAttribute(_id + "_" + i);
        if (requestValue == null)
            requestValue = request.getParameter(_id + "_" + i);
        if (requestValue != null || value == null)
            return requestValue;
        else {
            //should be an object array
            try {
                Object[] objects = (Object[]) value;
                return objects[i];
            } catch (Exception e) {
                throw new IllegalArgumentException("MultiTextWidget " + _id + " expects an Object[] to be passed in the value");
            }
        }
    }

    public void setInputs(Input[] inputs) {
        _inputs = inputs;
    }

    public String toString(Locale locale, Object value, ServletRequest request) {
        return toString(locale, value, request, null);
    }

    public String toString(Locale locale, Object value, ServletRequest request, String idSuffix) {
        StringBuffer buffer = new StringBuffer(256);
        ResourceBundle bundle = ResourceBundle.getBundle(_resourceLocation, locale);
        String inputSeparator;
        try {
            inputSeparator = bundle.getString(_resourceBase + RESOURCE_INPUT_SEPARATOR);
        } catch (MissingResourceException mre) {
            inputSeparator = "";
        }
        for (int i = 0; i < _inputs.length; i++) {
            if (i > 0)
                buffer.append(inputSeparator);
            appendInput(_inputs[i], i, selectValueToUse(i, value, request), idSuffix, buffer);
        }
        return buffer.toString();
    }

    public String[] getSubmitIds() {
        if (_submitIds == null) {
            String[] ids = new String[_inputs.length];
            for (int i = 0; i < ids.length; i++)
                ids[i] = _id + "_" + i;
            _submitIds = ids;
        }
        return _submitIds;
    }

    public class Input {
        int size;
        int maxlength;

        public Input() {
        }

        public Input(int s, int m) {
            size = s;
            maxlength = m;
        }
    }
}