/*
 * RadioButton.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.view.widget.html;

import java.util.Locale;
import java.util.Vector;

import javax.servlet.ServletRequest;

import com.eXtropia.config.Config;
import com.eXtropia.config.ConfigException;
import com.eXtropia.datahandler.DataHandler;

/**
 *
 * @author  Gunther Birznieks (gunther.birznieks@eXtropia.com)
 * @version 1.0
 */
public class RadioButton extends HtmlInput {
    public static final String CFG_VALUE = "Value";
    public static final String CFG_CHECKED_BY_DEFAULT = "CheckedByDefault";

    protected String _configValue = null;
    protected boolean _checkedByDefault = false;

    /** Creates new RadioButton */
    public RadioButton() {
        super("RADIO");
    }

    public void configure(Config config) throws ConfigException {
        super.configure(config);
        if (config.hasChild(CFG_VALUE)) {
            setConfigValue(config.getChild(CFG_VALUE).getValue());
        } else {
            throw new ConfigException("No value specified for widget " + _name);
        }
        if (config.hasChild(CFG_CHECKED_BY_DEFAULT)) {
            _checkedByDefault = config.getChild(CFG_CHECKED_BY_DEFAULT).getBooleanValue();
        }

        // hack to remove double value and CheckedByDefault to be generated
        // e.g.
        //  <INPUT TYPE="RADIO" NAME="provide_guarantor" VALUE="Y" Value="Y" CheckedByDefault="true">
        Config[] children = config.getChildren();
        Vector attributes = new Vector();
        for (int i = 0; i < children.length; i++) {
            String childName = children[i].getName();
            if (!(childName.equals(CFG_ID) || childName.equals(CFG_NAME) || childName.equals(DataHandler.CFG_BASE) || childName.equals(CFG_REQUIRED) || childName.equals(CFG_DEFAULT) || childName.equals(CFG_VALUE) || childName.equals(CFG_CHECKED_BY_DEFAULT))) {
                attributes.addElement(new Attribute(children[i].getName(), children[i].getValue()));
            }
        }
        _attributes = (Attribute[]) attributes.toArray(new Attribute[attributes.size()]);
    }

    public void setConfigValue(String s) {
        _configValue = s;
    }

    public String getConfigValue() {
        return _configValue;
    }

    public String toString(Locale locale, Object value, ServletRequest request, String idSuffix) {
        value = selectValueToUse(value, request);
        if (value == null && _default)
            value = getDefaultValue(locale);

        StringBuffer buffer = new StringBuffer(50);
        buffer.append("<INPUT TYPE=\"");
        buffer.append(_type);
        buffer.append("\" NAME=\"");
        buffer.append(_id);
        buffer.append("\" VALUE=\"");
        //buffer.append((value == null?"":value.toString()) + "\"");
        buffer.append(getConfigValue() + "\"");

        if ((value == null && _checkedByDefault) || (value != null && value.equals(getConfigValue())) || (idSuffix != null && idSuffix.equals("true"))) { // what's is it doing here???
            buffer.append(" CHECKED");
        }

        appendAttributes(buffer);
        buffer.append('>');
        return buffer.toString();
    }
}