/* 
 * Event.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.webcal;

import java.util.Date;

import com.eXtropia.auth.User;

/** 
 * An object respresenting an Event that occurs on all days between the specified start and end dates
 * <P>
 * Any time component specified in the start date and end date should not be relied upon i.e. 
 * an <CODE>Event</CODE> should not have hours, minutes or seconds specified in the 
 * {@link Task#_startDate} or {@link Task#_endDate}
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */
public class Event extends Task {
    /**
     * The physical location of the <CODE>Event</CODE>
     */
    public String _location;

    public Event(User owner, String subject, Date startDate, Date endDate) {
        super(owner, subject, startDate);
        if (!endDate.before(startDate)) {
            _endDate = endDate;
        } else {
            throw new IllegalArgumentException("The endDate should be before the startDate");
        }
    }

    /**
     * Creates an Event with the specified parameters.
     * @param owner the owner of the <CODE>Event</CODE>
     * @param subject the subject of the <CODE>event</CODE>
     * @param startDate the first day the event occurs on
     * @param endDate the last day the event occurs on
     * @throws java.lang.IllegalArgumentException if the <CODE>endDate</CODE> is before the <CODE>startDate</CODE>
     */
    public Event(Object id, User owner, String subject, Date startDate, Date endDate) {
        super(id, owner, subject, startDate);
        if (!endDate.before(startDate)) {
            _endDate = endDate;
        } else {
            throw new IllegalArgumentException("The endDate should be before the startDate");
        }
    }

    /**
     * @returns the {@link #_location} of the <CODE>Event</CODE>
     */
    public String getLocation() {
        return _location;
    }

    /**
     * Sets the {@link #_location} of the <CODE>Event</CODE>
     * @param location The new {@link #_location} of the <CODE>Event</CODE>.
     */
    public void setLocation(String location) {
        _location = location;
    }
}