/* 
 * RecurringHoliday.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.webcal;

import java.util.Date;

import com.eXtropia.auth.User;
import com.eXtropia.webcal.recurrence.RecurrencePattern;
import com.eXtropia.webcal.recurrence.Recurring;

/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */
public class RecurringHoliday extends Holiday implements Recurring {
    /**
     * The recurrence pattern of the <CODE>RecurringHoliday</CODE>
     */
    public RecurrencePattern _recurrencePattern;

    /**
     * Creates a new RecurringHoliday with the specified recurrence pattern
     * @param owner The owner of the <CODE>RecurringEvent</CODE>
     * @param subject The subject of the <CODE>RecurringEvent</CODE>
     * @param startDate The Start Date of the first instance of the <CODE>RecurringEvent</CODE>
     * @param endDate The End Date of the first instance of the <CODE>RecurringEvent</CODE>
     * @param recurrencePattern The recurrence Pattern of the <CODE>RecurringEvent</CODE>
     */
    public RecurringHoliday(Object id, User owner, String subject, Date startDate, Date endDate, RecurrencePattern pattern) {
        super(id, owner, subject, startDate, endDate);
        _recurrencePattern = pattern;
    }

    public RecurringHoliday(User owner, String subject, Date startDate, Date endDate, RecurrencePattern pattern) {
        super(owner, subject, startDate, endDate);
        _recurrencePattern = pattern;
    }

    public RecurrencePattern getRecurrencePattern() {
        return _recurrencePattern;
    }

    public void setRecurrencePattern(RecurrencePattern recurrencePattern) {
        _recurrencePattern = recurrencePattern;
    }

    public EventOccurrence[] getOccurrences(Date startDate, Date endDate) {
        return _recurrencePattern.getOccurrences(this, startDate, endDate);
    }
}