/* 
 * Utilities.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.webcal;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import com.eXtropia.webcal.recurrence.Recurring;

/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */
public class Utilities {
    /** Creates new Utilities */
    public Utilities() {

    }

    public static EventOccurrence[] generateEventOccurrences(Task[] tasks, Date startDate, Date endDate) {
        List eventOccurrenceArrays = new ArrayList();
        int totalOccurrences = 0;
        EventOccurrence[] occurrences;
        //expand out the tasks into event occurrences
        for (int i = 0; i < tasks.length; i++) {
            if (tasks[i] instanceof Event) {
                //then we need may need to expand it out
                if (tasks[i] instanceof Recurring) {
                    occurrences = ((Recurring) tasks[i]).getOccurrences(startDate, endDate);
                } else {
                    occurrences = new EventOccurrence[] { new EventOccurrence((Event) tasks[i], tasks[i].getStartDate(), tasks[i].getEndDate())};
                }
                eventOccurrenceArrays.add(occurrences);
                totalOccurrences += occurrences.length;
            }
        }
        EventOccurrence[] allOccurrences = new EventOccurrence[totalOccurrences];
        //now we have an event occurrence for all, need to draft all arrays into a single and then sort it
        int currentPosition = 0;
        for (Iterator listIter = eventOccurrenceArrays.iterator(); listIter.hasNext();) {
            occurrences = (EventOccurrence[]) listIter.next();
            System.arraycopy(occurrences, 0, allOccurrences, currentPosition, occurrences.length);
            currentPosition += occurrences.length;
        }
        Arrays.sort(allOccurrences);
        return allOccurrences;
    }

    public static Date parseDate(String date) {
        if (date != null && date.length() == 10) {
            try {
                int day = Integer.parseInt(date.substring(8, 10));
                int month = Integer.parseInt(date.substring(5, 7));
                int year = Integer.parseInt(date.substring(0, 4));
                Calendar cal = Calendar.getInstance();
                cal.clear();
                cal.set(Calendar.YEAR, year);
                cal.set(Calendar.MONTH, month - 1);
                cal.set(Calendar.DATE, day);
                return cal.getTime();
            } catch (Exception e) {
                return null;
            }
        }
        return null;
    }

    public static Date clearTime(Date date) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);

        int year = cal.get(Calendar.YEAR);
        int month = cal.get(Calendar.MONTH);
        int day = cal.get(Calendar.DATE);
        cal.clear();
        cal.set(Calendar.YEAR, year);
        cal.set(Calendar.MONTH, month);
        cal.set(Calendar.DATE, day);
        return cal.getTime();
    }
}