/* 
 * AddModify.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.webcal.action;

import java.util.Date;
import java.util.Map;

import javax.servlet.ServletRequest;

import com.eXtropia.app.UserGesture;
import com.eXtropia.app.UserGestureResponse;
import com.eXtropia.app.servlet.Views;
import com.eXtropia.diagnostics.LogEvent;
import com.eXtropia.util.ExtropiaException;
import com.eXtropia.webcal.Appointment;
import com.eXtropia.webcal.Event;
import com.eXtropia.webcal.RecurringAppointment;
import com.eXtropia.webcal.RecurringEvent;
import com.eXtropia.webcal.Task;
import com.eXtropia.webcal.datasource.DataSourceColumns;
import com.eXtropia.webcal.recurrence.Daily;
import com.eXtropia.webcal.recurrence.Monthly;
import com.eXtropia.webcal.recurrence.RecurrencePattern;
import com.eXtropia.webcal.recurrence.Yearly;

/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */
public class AddModify extends WebCalAction {
    public AddModify() {
    }

    public UserGestureResponse execute(UserGesture g, ServletRequest request) throws ExtropiaException {
        WebCalUserGesture gesture = (WebCalUserGesture) g;
        Map validatedData = gesture.getData();
        /*Map.Entry entry;
        for(Iterator iter= validatedData.entrySet().iterator();iter.hasNext();)
        {
        	entry= (Map.Entry)iter.next();
        	System.out.println(entry.getKey().toString() + " => " + (entry.getValue()==null?"null":entry.getValue().toString()));
        }*/

        boolean allDayEvent = ((Boolean) validatedData.get(UrlParams.ALL_DAY_EVENT)).booleanValue();
        String subject = (String) validatedData.get(UrlParams.SUBJECT);
        String description = (String) validatedData.get(UrlParams.DESCRIPTION);
        String location = (String) validatedData.get(UrlParams.LOCATION);
        Date startDate = (Date) validatedData.get(UrlParams.START_DATE);
        Date endDate = (Date) validatedData.get(UrlParams.END_DATE);
        Integer status = (Integer) validatedData.get(UrlParams.STATUS);
        Integer priority = (Integer) validatedData.get(UrlParams.PRIORITY);
        Integer recurrencePattern = (Integer) validatedData.get(UrlParams.RECURRENCE);
        Date recurUntil = (Date) validatedData.get(UrlParams.RECUR_UNTIL);

        Event event;
        RecurrencePattern pattern = generateRecurrencePattern(recurrencePattern, recurUntil);

        if (gesture.getId() == null) //it's an add
            {
            _doctor.log(new LogEvent(this, LogEvent.INFO, "User " + gesture.getUser().getId() + " adding event: " + subject));

            if (allDayEvent) {
                if (pattern == null)
                    event = new Event(gesture.getUser(), subject, startDate, endDate);
                else
                    event = new RecurringEvent(gesture.getUser(), subject, startDate, endDate, pattern);
            } else {
                if (pattern == null)
                    event = new Appointment(gesture.getUser(), subject, startDate, endDate);
                else
                    event = new RecurringAppointment(gesture.getUser(), subject, startDate, endDate, pattern);
            }
        } else //it's a modify
            {
            if (allDayEvent) {
                if (pattern == null)
                    event = new Event(gesture.getId(), gesture.getUser(), subject, startDate, endDate);
                else
                    event = new RecurringEvent(gesture.getId(), gesture.getUser(), subject, startDate, endDate, pattern);
            } else {
                if (pattern == null)
                    event = new Appointment(gesture.getId(), gesture.getUser(), subject, startDate, endDate);
                else
                    event = new RecurringAppointment(gesture.getId(), gesture.getUser(), subject, startDate, endDate, pattern);
            }
            //load up the old event to check that they are the owner
            Task task = _dataSource.loadById(gesture.getId());

            if (task == null)
                return new WebCalUserGestureResponse(Views.APPLICATION_ERROR, new ExtropiaException("Can't locate the event to modify"));

            if (!gesture.getUser().getId().equals(task.getOwner().getId())) {
                _doctor.log(new LogEvent(this, LogEvent.WARN, "User " + gesture.getUser().getId() + " is not allowed to modify a task owned by:  " + task.getOwner().getId()));
                return new WebCalUserGestureResponse(Views.UNAUTHORIZED);
            }
            _doctor.log(new LogEvent(this, LogEvent.INFO, "User " + gesture.getUser().getId() + " modifying event: " + task.getId()));
        }
        event.setDescription(description);
        event.setLocation(location);
        if (priority != null)
            event.setPriority(priority.intValue());
        if (status != null)
            event.setStatus(status.intValue());
        _dataSource.saveTask(event);
        return null;
    }

    protected RecurrencePattern generateRecurrencePattern(Integer recurrencePattern, Date recurUntil) {
        if (recurrencePattern == null || recurrencePattern.intValue() == DataSourceColumns.RECURRENCE_NONE)
            return null;
        else {
            RecurrencePattern pattern = null;
            switch (recurrencePattern.intValue()) {
                case DataSourceColumns.RECURRENCE_DAILY :
                    pattern = new Daily(1);
                    break;
                case DataSourceColumns.RECURRENCE_WEEKLY :
                    pattern = new Daily(7);
                    break;
                case DataSourceColumns.RECURRENCE_MONTHLY :
                    pattern = new Monthly(1);
                    break;
                case DataSourceColumns.RECURRENCE_YEARLY :
                    pattern = new Yearly(1);
                    break;
            }
            if (pattern != null && recurUntil != null)
                pattern.setRecurUntil(recurUntil);
            return pattern;
        }
    }
}