/* 
 * AddModifyTask.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.webcal.action;

import java.util.Date;
import java.util.Map;

import javax.servlet.ServletRequest;

import com.eXtropia.app.UserGesture;
import com.eXtropia.app.UserGestureResponse;
import com.eXtropia.app.servlet.Views;
import com.eXtropia.diagnostics.LogEvent;
import com.eXtropia.util.ExtropiaException;
import com.eXtropia.webcal.Task;

/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */
public class AddModifyTask extends WebCalAction {
    public AddModifyTask() {
    }

    public UserGestureResponse execute(UserGesture g, ServletRequest request) throws ExtropiaException {
        WebCalUserGesture gesture = (WebCalUserGesture) g;
        Map validatedData = gesture.getData();
        /*Map.Entry entry;
        for(Iterator iter= validatedData.entrySet().iterator();iter.hasNext();)
        {
        	entry= (Map.Entry)iter.next();
        	System.out.println(entry.getKey().toString() + " => " + (entry.getValue()==null?"null":entry.getValue().toString()));
        }*/

        String subject = (String) validatedData.get(UrlParams.SUBJECT);
        String description = (String) validatedData.get(UrlParams.DESCRIPTION);
        Date startDate = (Date) validatedData.get(UrlParams.START_DATE);
        Date endDate = (Date) validatedData.get(UrlParams.END_DATE);
        Integer status = (Integer) validatedData.get(UrlParams.STATUS);
        Integer priority = (Integer) validatedData.get(UrlParams.PRIORITY);

        Task task;
        if (gesture.getId() == null) //it's an add
            {
            task = new Task(gesture.getUser(), subject, startDate);
            _doctor.log(new LogEvent(this, LogEvent.INFO, "User " + gesture.getUser().getId() + " adding task: " + subject));
        } else //it's a modify
            {
            task = new Task(gesture.getId(), gesture.getUser(), subject, startDate);
            Task origTask = _dataSource.loadById(gesture.getId());

            if (origTask == null)
                return new WebCalUserGestureResponse(Views.APPLICATION_ERROR, new ExtropiaException("Can't locate the task to modify"));

            if (!gesture.getUser().getId().equals(origTask.getOwner().getId())) {
                _doctor.log(new LogEvent(this, LogEvent.WARN, "User " + gesture.getUser().getId() + " is not allowed to modify a task owned by:  " + origTask.getOwner().getId()));
                return new WebCalUserGestureResponse(Views.UNAUTHORIZED);
            }
            _doctor.log(new LogEvent(this, LogEvent.INFO, "User " + gesture.getUser().getId() + " modifying event: " + task.getId()));
        }
        task.setEndDate(endDate);
        task.setDescription(description);
        if (priority != null)
            task.setPriority(priority.intValue());
        if (status != null)
            task.setStatus(status.intValue());
        _dataSource.saveTask(task);

        return null;
    }
}