/* 
 * LoadDateRange.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.webcal.action;

import java.util.Calendar;
import java.util.Date;

import javax.servlet.ServletRequest;

import com.eXtropia.app.UserGesture;
import com.eXtropia.app.UserGestureResponse;
import com.eXtropia.auth.User;
import com.eXtropia.diagnostics.TraceEvent;
import com.eXtropia.util.ExtropiaException;
import com.eXtropia.webcal.Task;
import com.eXtropia.webcal.Utilities;
import com.eXtropia.webcal.view.WebCalViews;

/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */
public class LoadDateRange extends WebCalAction {
    public LoadDateRange() {
    }

    public UserGestureResponse execute(UserGesture g, ServletRequest request) throws ExtropiaException {
        WebCalUserGesture gesture = (WebCalUserGesture) g;
        WebCalUserGestureResponse response = new WebCalUserGestureResponse();
        Date startDate = null, endDate = null;
        //determine the data range to select
        if (gesture.getView().equals(WebCalViews.DAY)) {
            startDate = gesture.getFocusDate();
            Calendar cal = Calendar.getInstance();
            cal.setTime(startDate);
            cal.add(Calendar.DATE, 1);
            cal.add(Calendar.SECOND, -1);
            endDate = cal.getTime();
        } else if (gesture.getView().equals(WebCalViews.WEEK)) {
            Calendar cal = Calendar.getInstance();
            cal.setTime(gesture.getFocusDate());
            int firstDay = cal.getFirstDayOfWeek();
            int focusDateDayOfWeek = cal.get(Calendar.DAY_OF_WEEK);
            if (focusDateDayOfWeek < firstDay)
                focusDateDayOfWeek += 7;
            cal.add(Calendar.DATE, firstDay - focusDateDayOfWeek);
            startDate = cal.getTime();
            cal.add(Calendar.DATE, 7);
            cal.add(Calendar.SECOND, -1);
            endDate = cal.getTime();
        } else if (gesture.getView().equals(WebCalViews.MONTH)) {
            //need to determine when the month starts so we can get the days prior to the start of the month
            //but in the same week and the days after the end of the month within the same week
            Calendar cal = Calendar.getInstance();
            cal.setTime(gesture.getFocusDate());
            cal.set(Calendar.DATE, 1);
            Date firstDayOfMonth = cal.getTime();

            int firstDayOfWeek = cal.getFirstDayOfWeek();
            int myFirstDay = cal.get(Calendar.DAY_OF_WEEK);
            int daysFromLastMonth = myFirstDay - firstDayOfWeek;
            if (daysFromLastMonth < 0)
                daysFromLastMonth += 7;
            cal.add(Calendar.DATE, 0 - daysFromLastMonth);
            startDate = cal.getTime();

            cal.setTime(firstDayOfMonth);
            cal.add(Calendar.MONTH, 1);
            cal.add(Calendar.DATE, -1);
            //now the cal object points to the last day in the month we wish to view

            int myLastDay = cal.get(Calendar.DAY_OF_WEEK);
            int daysFromNextMonth = firstDayOfWeek - myLastDay;
            if (daysFromNextMonth <= 0)
                daysFromNextMonth += 7;

            cal.add(Calendar.DATE, daysFromNextMonth);
            cal.add(Calendar.SECOND, -1);
            endDate = cal.getTime();
        } else {
            throw new ExtropiaException("Can't recognize a date range to select from the specified view: " + gesture.getView());
        }
        if (_doctor.doTrace(TraceEvent.INFO))
            _doctor.trace(new TraceEvent(this, TraceEvent.INFO, "Loading events between: " + startDate + " and " + endDate));
        User[] users = new User[2];
        users[0] = gesture.getUser();
        users[1] = _globalUser;
        Task[] tasks = _dataSource.load(users, startDate, endDate);
        response.setTasks(tasks);
        response.setEventOccurrences(Utilities.generateEventOccurrences(tasks, startDate, endDate), startDate, endDate);
        response.setView(g.getView());
        return response;
    }
}