/* 
 * WebCalActionManager.java
 * Copyright (C) 2000 -2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.webcal.action;

import com.eXtropia.app.servlet.ActionManager;
import com.eXtropia.auth.AuthManager;
import com.eXtropia.auth.User;
import com.eXtropia.config.Config;
import com.eXtropia.config.ConfigException;
import com.eXtropia.view.ViewManager;
import com.eXtropia.webcal.datasource.TaskDataSource;
import com.eXtropia.webcal.view.WebCalViewAttributes;

/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */
public class WebCalActionManager extends ActionManager {
    public static final String CFG_GLOBAL_USER = "GlobalUid";
    public static final String CFG_START_OF_DAY = "StartOfDay";

    protected TaskDataSource _dataSource;
    protected User _globalUser;

    public WebCalActionManager(String appName, String diagBase, AuthManager authManager, ViewManager viewManager) {
        super(appName, diagBase, authManager, viewManager);
    }

    public void configureConcrete(Config config) throws ConfigException {
        if (config.hasChild(CFG_GLOBAL_USER)) {
            try {
                _globalUser = _authManager.getUser(config.getChild(CFG_GLOBAL_USER).getValue());
                if (_globalUser == null)
                    throw new Exception("Global User is not registered");
                _viewManager.addViewAttribute(WebCalViewAttributes.GLOBAL_USER, _globalUser);
            } catch (Exception e) {
                throw new ConfigException("Failed to get global user from AuthManager: " + e.getMessage());
            }
        } else
            throw new ConfigException("No " + CFG_GLOBAL_USER + " specified in " + CFG_BASE);

        //create the data source
        _dataSource = new TaskDataSource(_appName, _appInstance, _authManager);
        if (config.hasChild(TaskDataSource.CFG_BASE)) {
            _dataSource.configure(config.getChild(TaskDataSource.CFG_BASE));
        } else
            throw new ConfigException("No " + TaskDataSource.CFG_BASE + " specified in the " + CFG_BASE + " Config");

        if (config.hasChild(CFG_START_OF_DAY))
            _viewManager.addViewAttribute(WebCalViewAttributes.START_OF_DAY, new Integer(config.getChild(CFG_START_OF_DAY).getIntValue()));
        else
            _viewManager.addViewAttribute(WebCalViewAttributes.START_OF_DAY, DEFAULT_START_OF_DAY);
    }

    public com.eXtropia.app.servlet.Action createAction(Config config) throws ConfigException {
        com.eXtropia.app.servlet.Action action = super.createAction(config);
        if (action instanceof WebCalAction) {
            WebCalAction webCalAction = (WebCalAction) action;
            webCalAction.setGlobalUser(_globalUser);
            webCalAction.setTaskDataSource(_dataSource);
        }
        return action;
    }
}