/* 
 * WebCalUserGesture.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.webcal.action;

import java.util.Calendar;
import java.util.Date;
import java.util.Map;

import com.eXtropia.app.UserGesture;

/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */
public class WebCalUserGesture extends UserGesture {
    public static final String PARAM_FOCUS_DATE = "fd";
    public static final String PARAM_TASK_SORT_FIELD = "taskSf";
    public static final String PARAM_SEARCH_SORT_FIELD = "srchSf";
    public static final String PARAM_SORT_ASCENDING = "sa";
    public static final String PARAM_TASK_ID = "id";
    public static final String PARAM_SEARCH_KEYWORD = "sk";
    public static final String PARAM_SEARCH_TYPES = "st";
    public static final String PARAM_HOLIDAY_YEAR = "hy";

    private String _fd;
    private Date _focusDate;
    private Date _today;
    private String _taskSf;
    private String _srchSf;
    private boolean _sa = true;
    private String _id;
    private String _sk;
    private int _st;
    private int _hy;

    public Date getFocusDate() {
        if (_focusDate == null) {
            Calendar cal = Calendar.getInstance();
            int year = cal.get(Calendar.YEAR);
            int month = cal.get(Calendar.MONTH);
            int day = cal.get(Calendar.DATE);
            cal.clear();
            cal.set(Calendar.YEAR, year);
            cal.set(Calendar.MONTH, month);
            cal.set(Calendar.DATE, day);
            _focusDate = cal.getTime();
        }
        return _focusDate;
    }

    public String getId() {
        return _id;
    }

    public int getHy() {
        if (_hy == 0) {
            Calendar cal = Calendar.getInstance();
            _hy = cal.get(Calendar.YEAR);
        }
        return _hy;
    }

    public String getSk() {
        return _sk;
    }

    public int getSt() {
        return _st;
    }

    public boolean getSa() {
        return _sa;
    }

    public String getSrchSf() {
        return _srchSf;
    }

    public String getTaskSf() {
        return _taskSf;
    }

    public void setFd(String date) {
        if (date != null && date.length() == 10) {
            try {
                int day = Integer.parseInt(date.substring(8, 10));
                int month = Integer.parseInt(date.substring(5, 7));
                int year = Integer.parseInt(date.substring(0, 4));
                Calendar cal = Calendar.getInstance();
                cal.clear();
                cal.set(Calendar.YEAR, year);
                cal.set(Calendar.MONTH, month - 1);
                cal.set(Calendar.DATE, day);
                _fd = date;
                _focusDate = cal.getTime();
            } catch (Exception e) {
                e.printStackTrace();
                _focusDate = null;
            }
        }
    }

    public void setId(String id) {
        _id = id;
    }

    public void setHy(int holidayYear) {
        //System.out.println("Setting holiday year: " + holidayYear);
        _hy = holidayYear;
    }

    public void setSk(String searchKeyword) {
        _sk = searchKeyword;
    }

    public void setSt(int searchTypes) {
        _st = searchTypes;
    }

    public void setSa(boolean sortAscending) {
        _sa = sortAscending;
    }

    public void setSrchSf(String searchSortField) {
        _srchSf = searchSortField;
    }

    public void setTaskSf(String taskSortField) {
        _taskSf = taskSortField;
    }

    public Map getRequestParams() {
        Map map = super.getRequestParams();
        if (_fd != null)
            map.put(PARAM_FOCUS_DATE, _fd);
        if (!_sa)
            map.put(PARAM_SORT_ASCENDING, new Boolean(_sa));
        if (_taskSf != null)
            map.put(PARAM_TASK_SORT_FIELD, _taskSf);
        if (_sk != null)
            map.put(PARAM_SEARCH_KEYWORD, _sk);
        if (_st != 0)
            map.put(PARAM_SEARCH_TYPES, new Integer(_st));
        if (_hy != 0)
            map.put(PARAM_HOLIDAY_YEAR, new Integer(_hy));
        return map;
    }
}