/* 
 * AllDayEventHandler.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.webcal.datahandler;

import java.util.Calendar;
import java.util.Date;

import com.eXtropia.datahandler.DataHandler;
import com.eXtropia.datahandler.DataHandlerException;
import com.eXtropia.webcal.Utilities;

/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */
public class AllDayEventHandler extends DataHandler {
    public static final int ALL_DAY_INDEX = 0;
    public static final int START_DATE_INDEX = 1;
    public static final int END_DATE_INDEX = 2;

    /** Creates new AllDayEventHandler */
    public AllDayEventHandler() {
        super("alldayevent");
    }

    public Object validate(Object[] values) throws DataHandlerException {
        boolean allDayEvent = Boolean.valueOf(values[ALL_DAY_INDEX].toString()).booleanValue();
        if (allDayEvent) {
            Date startDate = (Date) values[START_DATE_INDEX];
            Date endDate = (Date) values[END_DATE_INDEX];

            if (startDate != null)
                startDate.setTime(Utilities.clearTime(startDate).getTime());

            if (endDate != null) {
                Calendar cal = Calendar.getInstance();
                cal.setTime(Utilities.clearTime(endDate));
                cal.add(Calendar.SECOND, 1); //if the same day, needs to end after the start
                //cal.add(Calendar.SECOND, -1);
                endDate.setTime(cal.getTime().getTime());
            }
        }
        return null;
    }
}