/* 
 * RecurrenceHandler.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.webcal.datahandler;

import java.util.Calendar;
import java.util.Date;

import com.eXtropia.datahandler.DataHandler;
import com.eXtropia.datahandler.DataHandlerException;
import com.eXtropia.webcal.Utilities;
import com.eXtropia.webcal.action.UrlParams;
import com.eXtropia.webcal.datasource.DataSourceColumns;

/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */
public class RecurrenceHandler extends DataHandler {
    public static final String ERROR_DATE_NO_RECURRENCE = "datenorecurrence";
    public static final String ERROR_UNTIL_BEFORE_END = "until";

    /** Creates new RecurrenceHandler */
    public RecurrenceHandler() {
        super("recurrence");
        super.setIds(new String[] { UrlParams.RECURRENCE, UrlParams.RECUR_UNTIL, UrlParams.END_DATE });
    }

    public void setIds(String[] ids) {
        throw new UnsupportedOperationException("This datahandler has fixed ids");
    }

    public Object validate(Object[] values) throws DataHandlerException {
        Integer recurrence = (Integer) values[0];
        Date recurUntil = (Date) values[1];
        if (recurUntil != null) {
            Date newRecurUntil = Utilities.clearTime(recurUntil);
            Calendar cal = Calendar.getInstance();
            cal.setTime(newRecurUntil);
            cal.add(Calendar.DATE, 1);
            cal.add(Calendar.SECOND, -1);
            recurUntil.setTime(cal.getTime().getTime());
        }
        Date endDate = (Date) values[2];
        if (recurrence != null && recurrence.intValue() == DataSourceColumns.RECURRENCE_NONE && recurUntil != null) {
            throw new DataHandlerException(_name, ERROR_DATE_NO_RECURRENCE);
        }
        if (recurUntil != null && endDate != null) {
            if (!recurUntil.after(endDate)) {
                throw new DataHandlerException(_name, ERROR_UNTIL_BEFORE_END);
            }
        }
        return null;
    }
}