/* 
 * TaskFactory.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.webcal.datasource;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.sql.RowSet;

import com.eXtropia.auth.AuthManager;
import com.eXtropia.auth.User;
import com.eXtropia.util.ExtropiaException;
import com.eXtropia.webcal.Appointment;
import com.eXtropia.webcal.Event;
import com.eXtropia.webcal.Holiday;
import com.eXtropia.webcal.RecurringAppointment;
import com.eXtropia.webcal.RecurringEvent;
import com.eXtropia.webcal.RecurringHoliday;
import com.eXtropia.webcal.Task;
import com.eXtropia.webcal.recurrence.Daily;
import com.eXtropia.webcal.recurrence.Monthly;
import com.eXtropia.webcal.recurrence.RecurrencePattern;
import com.eXtropia.webcal.recurrence.Recurring;
import com.eXtropia.webcal.recurrence.Yearly;

/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */
public class TaskFactory {
    /** Creates new TaskFactory */
    public TaskFactory() {

    }

    private static void getEventExtras(Event event, RowSet rowSet, AuthManager authManager) throws SQLException {
        event.setDescription(rowSet.getString(DataSourceColumns.DESCRIPTION));
        event.setStatus(rowSet.getInt(DataSourceColumns.STATUS));
        event.setPriority(rowSet.getInt(DataSourceColumns.PRIORITY));
        try {
            event.setLastModifiedBy(authManager.getUser(rowSet.getObject(DataSourceColumns.LAST_MOD_BY)));
        } catch (ExtropiaException e) {
        }
        event.setLastModified(rowSet.getDate(DataSourceColumns.LAST_MOD));
        event.setLocation(rowSet.getString(DataSourceColumns.LOCATION));
    }

    static Task getTask(RowSet rowSet, AuthManager authManager) throws ExtropiaException {
        try {
            if (rowSet.next())
                return realGetTask(rowSet, authManager);
            else
                return null;
        } catch (SQLException se) {
            throw new ExtropiaException("Exception getting Task from row set: " + se.getMessage());
        }
    }

    private static Task realGetTask(RowSet rowSet, AuthManager authManager) throws ExtropiaException {
        try {
            switch (rowSet.getInt(DataSourceColumns.TYPE)) {
                case DataSourceColumns.TYPE_TASK :
                    return generateTask(rowSet, authManager);
                case DataSourceColumns.TYPE_EVENT :
                    return generateEvent(rowSet, authManager);
                case DataSourceColumns.TYPE_APPOINTMENT :
                    return generateAppointment(rowSet, authManager);
                case DataSourceColumns.TYPE_HOLIDAY :
                    return generateHoliday(rowSet, authManager);
                default :
                    return null;
            }
        } catch (SQLException e) {
            throw new ExtropiaException("Exception determining type of Task: " + e.getMessage());
        }
    }

    static Task[] getTasks(RowSet rowSet, AuthManager authManager) throws ExtropiaException {
        List tasks = new ArrayList();
        Task task;
        try {
            while (rowSet.next()) {
                task = realGetTask(rowSet, authManager);
                if (task != null)
                    tasks.add(task);
            }
            return (Task[]) tasks.toArray(new Task[tasks.size()]);
        } catch (SQLException se) {
            throw new ExtropiaException("Exception constructing tasks: " + se.getMessage());
        }
    }

    private static Appointment generateAppointment(RowSet rowSet, AuthManager authManager) throws ExtropiaException {
        try {
            Object id = rowSet.getObject(DataSourceColumns.ID);
            Object userName = rowSet.getObject(DataSourceColumns.OWNER);
            User owner = authManager.getUser(userName);
            String subject = rowSet.getString(DataSourceColumns.SUBJECT);
            Date startDate = rowSet.getDate(DataSourceColumns.START_DATE);
            Date endDate = rowSet.getDate(DataSourceColumns.END_DATE);

            RecurrencePattern pattern = generateRecurrencePattern(rowSet);
            Appointment appointment;
            if (pattern != null)
                appointment = new RecurringAppointment(id, owner, subject, startDate, endDate, pattern);
            else
                appointment = new Appointment(id, owner, subject, startDate, endDate);

            getEventExtras(appointment, rowSet, authManager);
            return appointment;
        } catch (SQLException se) {
            throw new ExtropiaException("Exception constructing Task from row : " + se.getMessage());
        }
    }

    private static Event generateEvent(RowSet rowSet, AuthManager authManager) throws ExtropiaException {
        try {
            Object id = rowSet.getObject(DataSourceColumns.ID);
            Object userName = rowSet.getObject(DataSourceColumns.OWNER);
            User owner = authManager.getUser(userName);
            String subject = rowSet.getString(DataSourceColumns.SUBJECT);
            Date startDate = rowSet.getDate(DataSourceColumns.START_DATE);
            Date endDate = rowSet.getDate(DataSourceColumns.END_DATE);

            RecurrencePattern pattern = generateRecurrencePattern(rowSet);

            Event event;
            if (pattern != null)
                event = new RecurringEvent(id, owner, subject, startDate, endDate, pattern);
            else
                event = new Event(id, owner, subject, startDate, endDate);

            getEventExtras(event, rowSet, authManager);

            return event;
        } catch (SQLException se) {
            throw new ExtropiaException("Exception constructing Task from row : " + se.getMessage());
        }
    }

    private static Holiday generateHoliday(RowSet rowSet, AuthManager authManager) throws ExtropiaException {
        try {
            Object id = rowSet.getObject(DataSourceColumns.ID);
            Object userName = rowSet.getObject(DataSourceColumns.OWNER);
            User owner = authManager.getUser(userName);
            String subject = rowSet.getString(DataSourceColumns.SUBJECT);
            Date startDate = rowSet.getDate(DataSourceColumns.START_DATE);
            Date endDate = rowSet.getDate(DataSourceColumns.END_DATE);

            RecurrencePattern pattern = generateRecurrencePattern(rowSet);

            Holiday holiday;
            if (pattern != null)
                holiday = new RecurringHoliday(id, owner, subject, startDate, endDate, pattern);
            else
                holiday = new Holiday(id, owner, subject, startDate, endDate);

            getEventExtras(holiday, rowSet, authManager);

            return holiday;
        } catch (SQLException se) {
            throw new ExtropiaException("Exception constructing Task from row : " + se.getMessage());
        }
    }

    private static Task generateTask(RowSet rowSet, AuthManager authManager) throws ExtropiaException {
        try {
            Object id = rowSet.getObject(DataSourceColumns.ID);
            Object userName = rowSet.getObject(DataSourceColumns.OWNER);
            User owner = authManager.getUser(userName);
            String subject = rowSet.getString(DataSourceColumns.SUBJECT);
            Date startDate = rowSet.getDate(DataSourceColumns.START_DATE);

            String description = rowSet.getString(DataSourceColumns.DESCRIPTION);
            Date endDate = rowSet.getDate(DataSourceColumns.END_DATE);
            int status = rowSet.getInt(DataSourceColumns.STATUS);
            int priority = rowSet.getInt(DataSourceColumns.PRIORITY);
            Object lastModUser = rowSet.getObject(DataSourceColumns.OWNER);
            User lastModBy = authManager.getUser(lastModUser);
            Date lastModified = rowSet.getDate(DataSourceColumns.LAST_MOD);

            Task task = new Task(id, owner, subject, startDate);
            if (description != null)
                task.setDescription(description);
            if (endDate != null)
                task.setEndDate(endDate);
            task.setStatus(status);
            task.setPriority(priority);
            task.setLastModifiedBy(lastModBy);
            if (lastModified != null)
                task.setLastModified(lastModified);
            return task;
        } catch (SQLException se) {
            throw new ExtropiaException("Exception constructing Task from row : " + se.getMessage());
        }
    }

    static RecurrencePattern generateRecurrencePattern(RowSet rowSet) throws ExtropiaException {
        try {
            int recurrence = rowSet.getInt(DataSourceColumns.RECURRENCE);
            int interval = rowSet.getInt(DataSourceColumns.RECURRENCE_INTERVAL);

            RecurrencePattern pattern;
            switch (recurrence) {
                case DataSourceColumns.RECURRENCE_DAILY :
                    pattern = new Daily(interval);
                    break;
                case DataSourceColumns.RECURRENCE_MONTHLY :
                    pattern = new Monthly(interval);
                    break;
                case DataSourceColumns.RECURRENCE_YEARLY :
                    pattern = new Yearly(interval);
                    break;
                case DataSourceColumns.RECURRENCE_NONE :
                    return null;
                default :
                    throw new ExtropiaException("Unknown RecurrenceType specified in row: " + recurrence);
            }
            Date recurUntil = rowSet.getDate(DataSourceColumns.RECURRENCE_UNTIL_DATE);
            if (recurUntil != null)
                pattern.setRecurUntil(recurUntil);
            return pattern;
        } catch (SQLException se) {
            throw new ExtropiaException("Exception constructing RecurrencePattern from row : " + se.getMessage());
        }
    }

    static Map generateUpdateHash(Task task) {
        Map updates = new HashMap();
        updates.put(DataSourceColumns.TYPE, new Integer(DataSourceColumns.TYPE_TASK));
        updates.put(DataSourceColumns.OWNER, task.getOwner().getId());
        updates.put(DataSourceColumns.START_DATE, task.getStartDate());
        updates.put(DataSourceColumns.END_DATE, task.getEndDate());
        updates.put(DataSourceColumns.SUBJECT, task.getSubject());
        updates.put(DataSourceColumns.DESCRIPTION, task.getDescription());
        updates.put(DataSourceColumns.STATUS, new Integer(task.getStatus()));
        updates.put(DataSourceColumns.PRIORITY, new Integer(task.getPriority()));
        updates.put(DataSourceColumns.LAST_MOD_BY, task.getLastModifiedBy().getId());
        updates.put(DataSourceColumns.LAST_MOD, task.getLastModified());

        if (task instanceof Event) {
            if (task instanceof Appointment) {
                updates.put(DataSourceColumns.TYPE, new Integer(DataSourceColumns.TYPE_APPOINTMENT));
            } else if (task instanceof Holiday) {
                updates.put(DataSourceColumns.TYPE, new Integer(DataSourceColumns.TYPE_HOLIDAY));
            } else {
                updates.put(DataSourceColumns.TYPE, new Integer(DataSourceColumns.TYPE_EVENT));
            }
            Event event = (Event) task;
            updates.put(DataSourceColumns.LOCATION, (event).getLocation());
            if (event instanceof Recurring) {
                RecurrencePattern pattern = ((Recurring) event).getRecurrencePattern();
                if (pattern instanceof Daily)
                    updates.put(DataSourceColumns.RECURRENCE, new Integer(DataSourceColumns.RECURRENCE_DAILY));
                else if (pattern instanceof Monthly)
                    updates.put(DataSourceColumns.RECURRENCE, new Integer(DataSourceColumns.RECURRENCE_MONTHLY));
                else if (pattern instanceof Yearly)
                    updates.put(DataSourceColumns.RECURRENCE, new Integer(DataSourceColumns.RECURRENCE_YEARLY));
                updates.put(DataSourceColumns.RECURRENCE_UNTIL_DATE, pattern.getRecurUntil());
                updates.put(DataSourceColumns.RECURRENCE_INTERVAL, new Integer(pattern.getInterval()));
            } else {
                updates.put(DataSourceColumns.RECURRENCE, new Integer(DataSourceColumns.RECURRENCE_NONE));
                updates.put(DataSourceColumns.RECURRENCE_UNTIL_DATE, null);
                updates.put(DataSourceColumns.RECURRENCE_INTERVAL, null);
            }
        }
        return updates;
    }
}