/* 
 * Daily.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.webcal.recurrence;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import com.eXtropia.webcal.Event;
import com.eXtropia.webcal.EventOccurrence;

/**
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */
public class Daily extends RecurrencePattern {
    private static final long ONE_DAY = 1000L * 60 * 60 * 24;

    public Daily(int interval) {
        super(interval);
    }

    protected Date getFirstStartDate(Date windowStart, Date start) {
        if (start.compareTo(windowStart) > -1) {
            //the first occurrence is within the window
            return start;
        } else {
            long windowStartLong = windowStart.getTime();
            long startLong = start.getTime();
            long intervalLong = ONE_DAY * _interval;
            int intervalsBetween = (int) ((windowStartLong - startLong) / intervalLong);

            //be safe for long running appointment: better to iterate a couple of extra times 
            //than miss one
            intervalsBetween -= 2;
            if (intervalsBetween > 0) {
                Calendar cal = Calendar.getInstance();
                cal.setTime(start);
                cal.add(Calendar.DATE, intervalsBetween * _interval);
                return cal.getTime();
            } else {
                return start;
            }
        }
    }

    public EventOccurrence[] getOccurrences(Event event, Date startDate, Date endDate) {
        List eventOccurrences = new ArrayList();
        Calendar occurrenceCal = Calendar.getInstance();
        Date windowEndDate = endDate;
        EventOccurrence occurrence;

        if (_recurUntil != null && _recurUntil.compareTo(endDate) < 0)
            windowEndDate = _recurUntil;

        //occurrenceCal.setTime(event.getStartDate());
        occurrenceCal.setTime(getFirstStartDate(startDate, event.getStartDate()));

        while (occurrenceCal.getTime().compareTo(windowEndDate) < 1) {
            if (occurrenceCal.getTime().compareTo(startDate) > -1)
                eventOccurrences.add(new EventOccurrence(event, occurrenceCal.getTime(), getEndDate(event, occurrenceCal.getTime())));
            occurrenceCal.add(Calendar.DATE, _interval);
        }
        return (EventOccurrence[]) eventOccurrences.toArray(new EventOccurrence[eventOccurrences.size()]);
    }
}