/* 
 * DayOfMonth.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.webcal.recurrence;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import com.eXtropia.webcal.Event;
import com.eXtropia.webcal.EventOccurrence;

/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */
public abstract class DayOfMonth extends RecurrencePattern {
    /** Creates new DayOfMonth */
    public DayOfMonth(int interval) {
        super(interval);
    }

    protected abstract int getSkipCount(Date windowStart, Date start);

    public EventOccurrence[] getOccurrences(Event event, Date startDate, Date endDate, int intervalField) {
        List eventOccurrences = new ArrayList();
        Calendar occurrenceCal = Calendar.getInstance();
        Date windowEndDate = endDate;
        EventOccurrence occurrence;

        if (_recurUntil != null && _recurUntil.compareTo(endDate) < 0)
            windowEndDate = _recurUntil;

        int occurrences = getSkipCount(startDate, event.getStartDate());
        occurrenceCal.setTime(event.getStartDate());

        while (occurrenceCal.getTime().compareTo(windowEndDate) < 1) {
            if (occurrenceCal.getTime().compareTo(startDate) > -1)
                eventOccurrences.add(new EventOccurrence(event, occurrenceCal.getTime(), getEndDate(event, occurrenceCal.getTime())));
            occurrenceCal.setTime(event.getStartDate());
            occurrenceCal.add(intervalField, ++occurrences * _interval);
        }
        return (EventOccurrence[]) eventOccurrences.toArray(new EventOccurrence[eventOccurrences.size()]);
    }
}