/* 
 * RecurrencePattern.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.webcal.recurrence;

import java.util.Calendar;
import java.util.Date;

import com.eXtropia.webcal.Event;
import com.eXtropia.webcal.EventOccurrence;
import com.eXtropia.webcal.Utilities;

/**
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */
public abstract class RecurrencePattern {
    protected Date _recurUntil;
    protected int _interval;
    private int _daysDifferent = -1;

    public RecurrencePattern(int interval) {
        _interval = interval;
    }

    /**
     * Gets a list of the occurrences of the event, given a window of dates
     * @param event The event for which to get the occurrence list
     * @param windowStartDate the first date that an occurrence may start on
     * @param windowStartDate the last date that an occurrence may start on
     * @return an array of the occurrences in the window
     */
    public abstract EventOccurrence[] getOccurrences(Event event, Date startDate, Date endDate);

    protected Date getEndDate(Event event, Date currentStartDate) {
        if (currentStartDate.compareTo(event.getStartDate()) == 0) {
            return event.getEndDate();
        } else {
            if (_daysDifferent < 0) {
                //calculate the number of days different the original start date and 
                //end date are
                Calendar startDate = Calendar.getInstance();
                startDate.setTime(Utilities.clearTime(event.getStartDate()));
                Calendar endDate = Calendar.getInstance();
                endDate.setTime(Utilities.clearTime(event.getEndDate()));
                /*startDate.set(Calendar.HOUR, 0);
                startDate.set(Calendar.MINUTE, 0);
                startDate.set(Calendar.SECOND, 0);
                endDate.set(Calendar.HOUR, 0);
                endDate.set(Calendar.MINUTE, 0);
                endDate.set(Calendar.SECOND, 0);*/

                while (!startDate.after(endDate)) {
                    startDate.add(Calendar.DATE, 1);
                    _daysDifferent++;
                }
            }
            //as we know the days different we can clear the current start time
            //add the appropriate days, set time according to the end date
            Calendar origEndDate = Calendar.getInstance();
            origEndDate.setTime(event.getEndDate());

            Calendar endDate = Calendar.getInstance();
            endDate.setTime(Utilities.clearTime(currentStartDate));
            endDate.add(Calendar.DATE, _daysDifferent);
            endDate.set(Calendar.HOUR_OF_DAY, origEndDate.get(Calendar.HOUR_OF_DAY));
            endDate.set(Calendar.MINUTE, origEndDate.get(Calendar.MINUTE));
            endDate.set(Calendar.SECOND, origEndDate.get(Calendar.SECOND));
            return endDate.getTime();
        }
    }

    /**
     * @return The interval of the pattern
     */
    public int getInterval() {
        return _interval;
    }

    /**
     * @return The last day the recurrence starts to occur
     */
    public Date getRecurUntil() {
        return _recurUntil;
    }

    /**
     * @return The last day the recurrence starts to occur
     */
    public void setRecurUntil(Date recurUntil) {
        _recurUntil = recurUntil;
    }
}