/* 
 * Yearly.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.webcal.recurrence;

import java.util.Calendar;
import java.util.Date;

import com.eXtropia.webcal.Event;
import com.eXtropia.webcal.EventOccurrence;

/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */
public class Yearly extends DayOfMonth {
    /** Creates new Yearly */
    public Yearly(int interval) {
        super(interval);
    }

    protected int getSkipCount(Date windowStart, Date start) {
        if (start.compareTo(windowStart) > -1) {
            //the first occurrence is within the window
            return 0;
        } else {
            //determine the number of months between the two start dates
            Calendar windowStartCal = Calendar.getInstance();
            windowStartCal.setTime(windowStart);
            Calendar startCal = Calendar.getInstance();
            startCal.setTime(start);

            int yearDiff = windowStartCal.get(Calendar.YEAR) - startCal.get(Calendar.YEAR);

            int intervalsBetween = yearDiff / _interval;
            intervalsBetween -= 2; //subtract two for safety (better than missing one)
            if (intervalsBetween > 0) {
                return intervalsBetween;
            } else {
                return 0;
            }
        }
    }

    public EventOccurrence[] getOccurrences(Event event, Date startDate, Date endDate) {
        return getOccurrences(event, startDate, endDate, Calendar.YEAR);
    }
}