/* 
 * DayViewGridGenerator.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.webcal.view;

import java.util.Calendar;
import java.util.Date;

import com.eXtropia.webcal.Appointment;
import com.eXtropia.webcal.EventOccurrence;
import com.eXtropia.webcal.Utilities;
/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */
public class DayViewGridGenerator {
    public static final EventOccurrenceTableCell MARKER = new EventOccurrenceTableCell(null);

    /** Creates new DayViewGridGenerator */
    public DayViewGridGenerator() {

    }

    protected static EventOccurrenceTableCell[][] realGenerateGrid(Date date, int columns, EventOccurrence[] occurrences) {
        EventOccurrenceTableCell[][] grid = new EventOccurrenceTableCell[columns][24];
        Date startOfDay = Utilities.clearTime(date);
        Calendar cal = Calendar.getInstance();
        cal.setTime(startOfDay);
        cal.add(Calendar.DATE, 1);
        Date startOfNextDay = cal.getTime();

        EventOccurrence occurrence;
        Appointment appointment;
        EventOccurrenceTableCell tableCellToDraw;
        int startHour, endHour, columnToUse, gridSize = 0, lastColAvailable;
        boolean foundCol;
        for (int i = 0; i < occurrences.length; i++) {
            occurrence = occurrences[i];
            if (occurrence.getEvent() instanceof Appointment) {
                appointment = (Appointment) occurrence.getEvent();

                //determine the first row the appointment should appear in
                if (occurrence.getStartDate().compareTo(startOfDay) == -1) {
                    startHour = 0;
                } else {
                    cal.setTime(occurrence.getStartDate());
                    startHour = cal.get(Calendar.HOUR_OF_DAY);
                }
                //determine the last row the appointment should appear in
                if (occurrence.getEndDate().compareTo(startOfNextDay) > -1) {
                    endHour = 24;
                } else {
                    cal.setTime(occurrence.getEndDate());
                    endHour = cal.get(Calendar.HOUR_OF_DAY);
                    if (cal.get(Calendar.MINUTE) > 0)
                        endHour++;
                }
                //System.out.println(occurrence.getEvent().getSubject() + ")" + startHour + " => " + endHour);

                //now locate a suitable column for the appointment
                columnToUse = -1;
                for (int col = 0; col < grid.length; col++) //foreach column
                    {
                    foundCol = true;
                    for (int row = startHour; row < endHour; row++) {
                        if (grid[col][row] != null) {
                            foundCol = false;
                            continue;
                        }
                    }
                    if (foundCol) {
                        columnToUse = col;
                        break;
                    }
                }

                if (columnToUse < 0) //then the grid wasn't big enough for the number of concurrent appointments
                    return realGenerateGrid(date, columns + 1, occurrences);
                else {
                    tableCellToDraw = new EventOccurrenceTableCell(occurrence);
                    tableCellToDraw.setRowspan(endHour - startHour);
                    //mark the column with this this appointment so it doesn't get used again
                    grid[columnToUse][startHour] = tableCellToDraw;
                    for (int row = startHour + 1; row < endHour; row++) {
                        grid[columnToUse][row] = MARKER;
                    }
                    if (columnToUse > gridSize)
                        gridSize = columnToUse;
                }
            }
        }
        gridSize++;
        //now we have a grid and also a gridSize
        //printGrid(grid);
        //go through the grid and determine how many columns each appointment can occupy
        for (int row = 0; row < 24; row++) {
            for (int col = 0; col < gridSize; col++) {
                if (grid[col][row] != null && grid[col][row].getOccurrence() != null) {
                    startHour = row;
                    endHour = startHour + grid[col][row].getRowspan();
                    lastColAvailable = gridSize;
                    for (int i = startHour; lastColAvailable > 1 && i < endHour; i++) {
                        for (int j = col + 1; j < gridSize; j++) {
                            if (grid[j][i] != null && j < lastColAvailable) {
                                //then this is first column used up in this hour
                                lastColAvailable = j;
                                break;
                            }
                        }
                    }
                    grid[col][row].setColspan(lastColAvailable - col);
                    //System.out.println(grid[col][row].getOccurrence().getEvent().getSubject() + " rowspan: " + grid[col][row].getRowspan() + ", colspan: " + grid[col][row].getColspan());
                    for (int i = startHour; i < endHour; i++) {
                        for (int j = col; j < lastColAvailable; j++) {
                            if (i == startHour && j == col)
                                continue;
                            grid[j][i] = MARKER;
                        }
                    }
                }
            }
        }
        //printGrid(grid);
        //now go through the grid, hour by hour and fill in any spare slots with markers
        //also, since we only draw each appointment in one place, we need to remove all markers that are for
        //existing events i.e. the only markers left should be those for blank cells
        for (int row = 0; row < 24; row++) {
            for (int col = 0; col < gridSize; col++) {
                if (grid[col][row] == null) {
                    lastColAvailable = gridSize;
                    for (int j = col + 1; j < gridSize; j++) {
                        if (grid[j][row] != null && j < lastColAvailable) {
                            lastColAvailable = j;
                            break;
                        }
                    }
                    grid[col][row] = new EventOccurrenceTableCell(null);
                    grid[col][row].setColspan(lastColAvailable - col);
                    col += lastColAvailable - col - 1;
                } else if (grid[col][row] == MARKER) {
                    grid[col][row] = null;
                }
            }
        }
        //printGrid(grid);
        return grid;
    }

    public static EventOccurrenceTableCell[][] generateGrid(Date date, EventOccurrence[] occurrences) {
        return realGenerateGrid(date, 1, occurrences);
    }

    /*public static void main(String [] args)
    {
    	Date date= Utilities.clearTime(new Date());
    	Date startDate, endDate;
    	Calendar cal= Calendar.getInstance();
    	cal.setTime(date);
    
    	cal.set(Calendar.HOUR_OF_DAY, 8);
    	startDate= cal.getTime();
    	cal.set(Calendar.HOUR_OF_DAY, 9);
    	endDate= cal.getTime();
    	Appointment appt1= new Appointment(null, "1", startDate, endDate);
    	EventOccurrence o1= new EventOccurrence(appt1, startDate, endDate);
    
    	cal.set(Calendar.HOUR_OF_DAY, 9);
    	startDate= cal.getTime();
    	cal.set(Calendar.HOUR_OF_DAY, 12);
    	endDate= cal.getTime();
    	Appointment appt2= new Appointment(null, "2", startDate, endDate);
    	EventOccurrence o2= new EventOccurrence(appt2, startDate, endDate);
    
    	cal.set(Calendar.HOUR_OF_DAY, 9);
    	startDate= cal.getTime();
    	cal.set(Calendar.HOUR_OF_DAY, 13);
    	cal.set(Calendar.MINUTE, 15);
    	endDate= cal.getTime();
    	Appointment appt3= new Appointment(null, "3", startDate, endDate);
    	EventOccurrence o3= new EventOccurrence(appt3, startDate, endDate);
    
    	cal.set(Calendar.MINUTE, 0);
    	cal.set(Calendar.HOUR_OF_DAY, 11);
    	startDate= cal.getTime();
    	cal.set(Calendar.HOUR_OF_DAY, 15);
    	endDate= cal.getTime();
    	Appointment appt4= new Appointment(null, "4", startDate, endDate);				
    	EventOccurrence o4= new EventOccurrence(appt4, startDate, endDate);
    
    	cal.set(Calendar.HOUR_OF_DAY, 14);
    	startDate= cal.getTime();
    	cal.set(Calendar.HOUR_OF_DAY, 17);
    	endDate= cal.getTime();
    	Appointment appt5= new Appointment(null, "5", startDate, endDate);
    	EventOccurrence o5= new EventOccurrence(appt5, startDate, endDate);
    	
    	generateGrid(date, new EventOccurrence[] {o1, o2, o3, o4, o5});
    }
    
    public static void printGrid(EventOccurrenceTableCell [][] grid)
    {
    	NumberFormat nf= new DecimalFormat("00");
    	for(int row= 0; row<grid[0].length;row++)
    	{
    		System.out.print(nf.format(row) + "|");
    		for(int col=0;col<grid.length;col++)
    		{
    			if(grid[col][row] == null)
    				System.out.print("      ");
    			else if(grid[col][row] == MARKER)
    				System.out.print("EM    ");
    			else if(grid[col][row].getOccurrence() == null)
    				System.out.print("   " + grid[col][row].getColspan() + "x" + grid[col][row].getRowspan());
    			else
    				System.out.print("E" + grid[col][row].getOccurrence().getEvent().getSubject() + " " + grid[col][row].getColspan() + "x" + grid[col][row].getRowspan());
    			System.out.print("|");
    		}
    		System.out.println("");
    	}
    }*/
}