/* 
 * EventOccurrenceIteratorTag.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.webcal.view;

import java.util.Date;
import java.util.Iterator;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.BodyTagSupport;

import com.eXtropia.webcal.Appointment;
import com.eXtropia.webcal.Event;
import com.eXtropia.webcal.EventOccurrence;
import com.eXtropia.webcal.Holiday;
import com.eXtropia.webcal.RecurringAppointment;
import com.eXtropia.webcal.RecurringEvent;
import com.eXtropia.webcal.RecurringHoliday;
import com.eXtropia.webcal.action.WebCalUserGestureResponse;

/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */
public class EventOccurrenceIteratorTag extends BodyTagSupport implements Iterator {
    public static final String TYPE_APPT = "appointment";
    public static final String TYPE_EVENT = "event";
    public static final String TYPE_HOLIDAY = "holiday";

    protected Date _startDate, _endDate;
    protected EventOccurrence[] _eventOccurrences;
    protected int _index = -1;
    protected boolean _fetched = true;

    protected Class[] _comparators = null;

    /** Creates new EventOccurrenceIteratorTag */
    public EventOccurrenceIteratorTag() {
    }

    public int doStartTag() throws JspException {
        WebCalUserGestureResponse gestureResponse = (WebCalUserGestureResponse) pageContext.getRequest().getAttribute(WebCalUserGestureResponse.REQUEST_PARAM_NAME);

        if (gestureResponse == null || gestureResponse.getEventOccurrences() == null)
            throw new JspException("No event occurrences to iterate over");
        _eventOccurrences = gestureResponse.getEventOccurrences();
        if (hasNext())
            return EVAL_BODY_TAG;
        else
            return SKIP_BODY;
    }

    public int doEndTag() throws JspException {
        try {
            if (bodyContent != null) {
                bodyContent.writeOut(bodyContent.getEnclosingWriter());
            }
        } catch (Exception e) {
            throw new JspException("IO Error: " + e.getMessage());
        }
        return EVAL_PAGE;
    }

    public int doAfterBody() throws JspException {
        if (!_fetched) {
            return SKIP_BODY;
        }
        if (hasNext()) {
            return EVAL_BODY_TAG;
        } else {
            return SKIP_BODY;
        }
    }

    public boolean hasNext() {
        if (_fetched) {
            boolean dateBoundsOk;
            EventOccurrence currentEventOccurrence;
            while (_fetched && ++_index < _eventOccurrences.length) {
                //if(_eventOccurrences[_index].getEvent().getId().toString().equals("120"))
                //System.out.println("Trying to get: " + _startDate + ", " + _endDate + ":" + _eventOccurrences[_index].getEvent().getId() + " " + _eventOccurrences[_index].getStartDate() + ", " + _eventOccurrences[_index].getEndDate());
                dateBoundsOk = true;
                currentEventOccurrence = _eventOccurrences[_index];
                //this is the next one if within the bounds of the dates (if any)
                if (_startDate != null && currentEventOccurrence.getEndDate().compareTo(_startDate) != 1)
                    dateBoundsOk = false;
                if (_endDate != null && currentEventOccurrence.getStartDate().compareTo(_endDate) > -1) {
                    dateBoundsOk = false;
                    _index = _eventOccurrences.length;
                }
                _fetched = !dateBoundsOk;
                if (!_fetched && _comparators != null) // then we also need to check the class is correct
                    {
                    boolean matchedClass = false;
                    for (int i = 0; i < _comparators.length; i++) {
                        if (_eventOccurrences[_index].getEvent().getClass().equals(_comparators[i])) {
                            matchedClass = true;
                            break;
                        }
                    }
                    _fetched = !matchedClass;
                }
            }
        }
        return !_fetched;
    }

    public Object next() {
        _fetched = true;
        return _eventOccurrences[_index];
    }

    /**
     * Not supported as views should not change the model
     */
    public void remove() {
        throw new UnsupportedOperationException();
    }

    public void setEndDate(Date date) {
        _endDate = date;
    }

    /*public void setEndDate(String date)
    {
    	_endDate= Utilities.parseDate(date);
    	if(_endDate == null)
    		throw new IllegalArgumentException("Start Date is not of the right format");
    }*/

    public void setStartDate(Date date) {
        _startDate = (Date) date;
    }

    /*public void setStartDate(String date)
    {
    	_startDate= Utilities.parseDate(date);
    	if(_startDate == null)
    		throw new IllegalArgumentException("Start Date is not of the right format");
    }*/

    public void setType(String type) {
        if (type.equals(TYPE_APPT)) {
            _comparators = new Class[] { Appointment.class, RecurringAppointment.class };
        } else if (type.equals(TYPE_EVENT)) {
            _comparators = new Class[] { Event.class, RecurringEvent.class };
        } else if (type.equals(TYPE_HOLIDAY)) {
            _comparators = new Class[] { Holiday.class, RecurringHoliday.class };
        } else {
            throw new IllegalArgumentException("Unrecognised type: " + type);
        }
    }

    public void release() {
        super.release();
        _startDate = null;
        _endDate = null;
        _eventOccurrences = null;
        _index = -1;
        _fetched = true;
    }
}