/* 
 * EventWidgetTag.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.webcal.view;

import java.io.IOException;
import java.util.Locale;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.Tag;
import javax.servlet.jsp.tagext.TagSupport;

import com.eXtropia.app.servlet.ServletUtils;
import com.eXtropia.view.NextTag;
import com.eXtropia.view.View;
import com.eXtropia.view.ViewManager;
import com.eXtropia.view.WidgetIteratorTag;
import com.eXtropia.view.widget.Widget;
import com.eXtropia.webcal.EventOccurrence;

/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */
public class EventWidgetTag extends TagSupport {
    protected static final int LOCATION = 1901043637; /* location */
    protected static final int SUBJECT = -1867885268; /* subject */
    protected static final int DESCRIPTION = -1724546052; /* description */
    protected static final int STARTDATE = -2128825584; /* startdate */
    protected static final int ENDDATE = -1606774007; /* enddate */
    protected static final int STARTTIME = -2128341457; /* starttime */
    protected static final int ENDTIME = -1606289880; /* endtime */
    protected static final int STATUS = -892481550; /* status */
    protected static final int PRIORITY = -1165461084; /* priority */

    public static final String LABEL = "label";
    public static final String NAME = "name";
    public static final String ID = "id";

    private String _property;
    private EventOccurrence _eventOccurrence;
    private String _id;
    private Widget _widget;

    public EventWidgetTag() {

    }

    public String getId() {
        return _id;
    }

    public void setId(String id) {
        _id = id;
    }

    public String getProperty() {
        return _property;
    }

    public void setProperty(String property) {
        _property = property;
    }

    public int doStartTag() throws JspException {
        Tag tag = this;
        //select out the id if not specified
        if (_id == null) {
            while (_widget == null && tag != null) {
                if (tag instanceof NextTag && ((NextTag) tag).getIterator() instanceof WidgetIteratorTag) {
                    _widget = (Widget) ((NextTag) tag).getObject();
                } else
                    tag = tag.getParent();
            }
        } else {
            View view = (View) pageContext.getRequest().getAttribute(ViewManager.VIEW_ATTRIBUTE_NAME);
            _widget = view.getWidget(_id);
        }

        if (_widget == null)
            throw new JspException("Undefined widget: " + _id);

        //now find the current event
        tag = this;
        while (_eventOccurrence == null && tag != null) {
            if (tag instanceof NextTag && ((NextTag) tag).getIterator() instanceof EventOccurrenceIteratorTag) {
                _eventOccurrence = (EventOccurrence) ((NextTag) tag).getObject();
            } else
                tag = tag.getParent();
        }
        return SKIP_BODY;
    }

    public int doEndTag() throws JspException {
        Locale locale = (Locale) pageContext.getSession().getAttribute(ServletUtils.SESSION_ATTRIBUTE_LOCALE);
        if (locale == null)
            throw new JspException("no locale specified in the session");
        String value = "";
        if (_property != null) {
            if (_property.equals(LABEL)) {
                value = _widget.getLabel(locale);
                if (value == null) {
                    value = _widget.getName();
                }
            } else if (_property.equals(NAME)) {
                value = _widget.getName();
            }
        } else {
            Object eventValue = null;
            if (_eventOccurrence != null)
                eventValue = getEventValue(_id);
            value = _widget.toString(locale, eventValue, pageContext.getRequest());
        }

        try {
            pageContext.getOut().write(value);
        } catch (IOException e) {
            throw new JspException("IOException: " + e);
        }
        return EVAL_PAGE;
    }

    protected Object getEventValue(String id) {
        switch (id.hashCode()) {
            case SUBJECT :
                return _eventOccurrence.getEvent().getSubject();
            case DESCRIPTION :
                return _eventOccurrence.getEvent().getDescription();
            case LOCATION :
                return _eventOccurrence.getEvent().getLocation();
            case STARTDATE :
                return _eventOccurrence.getStartDate();
            case ENDDATE :
                return _eventOccurrence.getEndDate();
            case STARTTIME :
                return _eventOccurrence.getStartDate();
            case ENDTIME :
                return _eventOccurrence.getEndDate();
            case STATUS :
                return new Integer(_eventOccurrence.getEvent().getStatus());
            case PRIORITY :
                //System.out.println("The priority of the event is: " + _eventOccurrence.getEvent().getPriority());
                return new Integer(_eventOccurrence.getEvent().getPriority());
            default :
                throw new RuntimeException("Invalid id: " + id);
        }
    }

    public void release() {
        _eventOccurrence = null;
        _property = null;
        _id = null;
        _widget = null;
    }

    public void setEventOccurrence(EventOccurrence eventOccurrence) {
        _eventOccurrence = eventOccurrence;
    }
}