/* 
 * TaskWidgetTag.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.webcal.view;

import java.io.IOException;
import java.util.Locale;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.Tag;
import javax.servlet.jsp.tagext.TagSupport;

import com.eXtropia.app.servlet.ServletUtils;
import com.eXtropia.view.NextTag;
import com.eXtropia.view.View;
import com.eXtropia.view.ViewManager;
import com.eXtropia.view.widget.Widget;
import com.eXtropia.webcal.Event;
import com.eXtropia.webcal.datasource.DataSourceColumns;
import com.eXtropia.webcal.recurrence.Daily;
import com.eXtropia.webcal.recurrence.Monthly;
import com.eXtropia.webcal.recurrence.Recurring;
import com.eXtropia.webcal.recurrence.Yearly;

/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */
public class RecurrencePatternWidgetTag extends TagSupport {
    public static final int RECURRENCE = 1778391856; /* recurrence */
    public static final int RECURUNTIL = 1781436497; /* recuruntil */

    public static final String LABEL = "label";
    public static final String NAME = "name";
    public static final String ID = "id";

    private String _property;
    private Event _event;
    private String _id;
    private Widget _widget;

    public RecurrencePatternWidgetTag() {

    }

    public String getId() {
        return _id;
    }

    public void setEvent(Event event) {
        _event = event;
    }

    public void setId(String id) {
        _id = id;
    }

    public String getProperty() {
        return _property;
    }

    public void setProperty(String property) {
        _property = property;
    }

    public int doStartTag() throws JspException {
        Tag tag = this;
        //select out the id if not specified
        _widget = null;
        if (_id != null) {
            View view = (View) pageContext.getRequest().getAttribute(ViewManager.VIEW_ATTRIBUTE_NAME);
            _widget = view.getWidget(_id);
        }

        if (_widget == null)
            throw new JspException("Undefined widget: " + _id);

        //now find the current Task
        tag = this;
        while (_event == null && tag != null) {
            if (tag instanceof NextTag && (((NextTag) tag).getIterator() instanceof TaskIteratorTag))
                _event = (Event) ((NextTag) tag).getObject();
            else
                tag = tag.getParent();
        }
        return SKIP_BODY;
    }

    public int doEndTag() throws JspException {
        Locale locale = (Locale) pageContext.getSession().getAttribute(ServletUtils.SESSION_ATTRIBUTE_LOCALE);
        if (locale == null)
            throw new JspException("no locale specified in the session");
        String value = "";
        if (_property != null) {
            if (_property.equals(LABEL)) {
                value = _widget.getLabel(locale);
                if (value == null) {
                    value = _widget.getName();
                }
            } else if (_property.equals(NAME)) {
                value = _widget.getName();
            }
        } else {
            Object eventValue = null;
            if (_event != null) {
                eventValue = getEventValue(_id);
            }
            value = _widget.toString(locale, eventValue, pageContext.getRequest());
        }

        try {
            pageContext.getOut().write(value);
        } catch (IOException e) {
            throw new JspException("IOException: " + e);
        }
        return EVAL_PAGE;
    }

    protected Object getEventValue(String id) {
        boolean isRecurring = false;
        Recurring recurring = null;
        if (_event instanceof Recurring) {
            isRecurring = true;
            recurring = (Recurring) _event;
        }

        switch (id.hashCode()) {

            case RECURRENCE :
                int result = DataSourceColumns.RECURRENCE_NONE;
                if (!isRecurring) {
                    result = DataSourceColumns.RECURRENCE_NONE;
                } else if (recurring.getRecurrencePattern() instanceof Daily) {
                    if (recurring.getRecurrencePattern().getInterval() == 7)
                        result = DataSourceColumns.RECURRENCE_WEEKLY;
                    else
                        result = DataSourceColumns.RECURRENCE_DAILY;
                } else if (recurring.getRecurrencePattern() instanceof Monthly)
                    result = DataSourceColumns.RECURRENCE_MONTHLY;
                else if (recurring.getRecurrencePattern() instanceof Yearly)
                    result = DataSourceColumns.RECURRENCE_YEARLY;
                else
                    result = DataSourceColumns.RECURRENCE_NONE;
                return new Integer(result);
            case RECURUNTIL :
                if (isRecurring) {
                    return recurring.getRecurrencePattern().getRecurUntil();
                } else {
                    return null;
                }
            default :
                throw new RuntimeException("Invalid id: " + id);
        }
    }

    public void release() {
        super.release();
        _property = null;
        _event = null;
        _id = null;
        _widget = null;
    }
}