/* 
 * EventOccurrenceIteratorTag.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.webcal.view;

import java.util.Iterator;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.BodyTagSupport;

import com.eXtropia.webcal.Appointment;
import com.eXtropia.webcal.Event;
import com.eXtropia.webcal.Holiday;
import com.eXtropia.webcal.Task;
import com.eXtropia.webcal.action.WebCalUserGestureResponse;

/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */
public class TaskIteratorTag extends BodyTagSupport implements Iterator {
    public static final String TYPE_TASK = "task";
    public static final String TYPE_APPT = "appointment";
    public static final String TYPE_EVENT = "event";
    public static final String TYPE_HOLIDAY = "holiday";

    protected Task[] _tasks;
    protected int _index = -1;
    protected boolean _fetched = true;

    protected boolean _comparing;
    protected int _comparator;

    /** Creates new EventOccurrenceIteratorTag */
    public TaskIteratorTag() {

    }

    public int doAfterBody() throws JspException {
        if (!_fetched) {
            throw new JspException("No NextTag in the TaskIteratorTag");
        }
        if (hasNext()) {
            return EVAL_BODY_TAG;
        } else {
            return SKIP_BODY;
        }
    }

    public int doEndTag() throws JspException {
        try {
            if (bodyContent != null) {
                bodyContent.writeOut(bodyContent.getEnclosingWriter());
            }
        } catch (Exception e) {
            throw new JspException("IO Error: " + e.getMessage());
        }
        return EVAL_PAGE;
    }

    public int doStartTag() throws JspException {
        WebCalUserGestureResponse gestureResponse = (WebCalUserGestureResponse) pageContext.getRequest().getAttribute(WebCalUserGestureResponse.REQUEST_PARAM_NAME);

        _tasks = gestureResponse.getTasks();
        return EVAL_BODY_TAG;
    }

    public boolean hasNext() {
        if (_fetched && _tasks != null) {
            Task currentTask;
            while (_fetched && ++_index < _tasks.length) {
                currentTask = _tasks[_index];
                if (!_comparing || currentTask.getClass().getName().hashCode() == _comparator)
                    _fetched = false;
            }
        }
        return !_fetched;
    }

    public Object next() {
        _fetched = true;
        return _tasks[_index];
    }

    /**
     * Not supported as views should not change the model
     */
    public void remove() {
        throw new UnsupportedOperationException();
    }

    public void setType(String type) {
        if (type.equals(TYPE_APPT)) {
            _comparator = Appointment.class.getName().hashCode();
        } else if (type.equals(TYPE_EVENT)) {
            _comparator = Event.class.getName().hashCode();
        } else if (type.equals(TYPE_HOLIDAY)) {
            _comparator = Holiday.class.getName().hashCode();
        } else if (type.equals(TYPE_TASK)) {
            _comparator = Task.class.getName().hashCode();
        } else {
            throw new IllegalArgumentException("Unrecognised type: " + type);
        }
        _comparing = true;
    }

    public void release() {
        super.release();
        _tasks = null;
        _index = -1;
        _fetched = true;

        _comparing = false;
        _comparator = 0;
    }
}