/* 
 * PriorityWidget.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.webcal.view.widget;

import java.util.Locale;
import java.util.ResourceBundle;

import javax.servlet.ServletRequest;

import com.eXtropia.config.Config;
import com.eXtropia.config.ConfigException;
import com.eXtropia.datahandler.DataHandler;
import com.eXtropia.datahandler.handler.IntegerHandler;
import com.eXtropia.view.widget.Widget;
import com.eXtropia.webcal.Task;

/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */
public class PriorityWidget extends Widget {
    public static final String CFG_EDITABLE = "Editable";

    public static final String LOW = "low";
    public static final String NORMAL = "normal";
    public static final String HIGH = "high";

    public static final String[] PRIORITIES = new String[] { LOW, NORMAL, HIGH };
    public static final int[] INT_PRIORITIES = new int[] { Task.Priority.LOW, Task.Priority.NORMAL, Task.Priority.HIGH };

    protected boolean _editable;

    /** Creates new PriorityWidget */
    public PriorityWidget() {
    }

    public void configure(Config config) throws ConfigException {
        super.configure(config);
        if (config.hasChild(CFG_EDITABLE))
            setEditable(Boolean.valueOf(config.getChild(CFG_EDITABLE).getValue()).booleanValue());
        DataHandler handler = new IntegerHandler();
        handler.setIds(new String[] { _id });
        addDataHandler(handler);
    }

    public void setEditable(boolean editable) {
        _editable = editable;
    }

    public String toString(Locale locale, Object value, ServletRequest request) {
        return _editable ? toEditableString(locale, value, request) : toNonEditableString(locale, value, request);
    }

    public String toNonEditableString(Locale locale, Object value, ServletRequest request) {
        if (!(value instanceof Integer)) {
            try {
                value = new Integer(value.toString());
            } catch (Exception e) {
                value = null;
            }
        }
        int priority = Task.Priority.NORMAL;
        if (value != null)
            priority = ((Integer) value).intValue();
        String resourceName;
        String res;
        switch (priority) {
            case Task.Priority.LOW :
                resourceName = _resourceBase + "." + LOW;
                res = LOW;
                break;
            case Task.Priority.HIGH :
                resourceName = _resourceBase + "." + HIGH;
                res = HIGH;
                break;
            default :
                resourceName = _resourceBase + "." + NORMAL;
                res = NORMAL;
                break;
        }
        ResourceBundle bundle = ResourceBundle.getBundle(_resourceLocation, locale);
        try {
            res = bundle.getString(resourceName);
        } catch (Exception e) {
        } //just leave as the string LOW, NORMAL or HIGH
        return res;
    }

    protected String toEditableString(Locale locale, Object value, ServletRequest request) {
        value = selectValueToUse(value, request);
        if (!(value instanceof Integer)) {
            try {
                value = new Integer(value.toString());
            } catch (Exception e) {
                value = null;
            }
        }
        int priority = Task.Priority.NORMAL;
        if (value != null)
            priority = ((Integer) value).intValue();
        StringBuffer buffer = new StringBuffer();
        buffer.append("<SELECT name=\"");
        buffer.append(_id);
        buffer.append("\">");

        ResourceBundle bundle = ResourceBundle.getBundle(_resourceLocation, locale);

        for (int i = 0; i < PRIORITIES.length; i++) {
            buffer.append("<OPTION value=\"");
            buffer.append(INT_PRIORITIES[i]);
            buffer.append('\"');
            if (priority == INT_PRIORITIES[i]) {
                buffer.append(" SELECTED");
            }
            buffer.append('>');
            value = PRIORITIES[i];
            try {
                value = bundle.getString(_resourceBase + "." + PRIORITIES[i]);
            } catch (Exception e) {
            }
            buffer.append(value);
        }
        buffer.append("</SELECT>");
        return buffer.toString();
    }

    public String toString(Locale locale, Object value, ServletRequest request, String idSuffix) {
        return toString(locale, value, request);
    }
}