/* 
 * RecursWidget.java
 * Copyright (C) 2000 - 2001  eXtropia.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.eXtropia.webcal.view.widget;

import java.util.Locale;
import java.util.ResourceBundle;

import javax.servlet.ServletRequest;

import com.eXtropia.config.Config;
import com.eXtropia.config.ConfigException;
import com.eXtropia.datahandler.DataHandler;
import com.eXtropia.datahandler.handler.IntegerHandler;
import com.eXtropia.view.widget.Widget;
import com.eXtropia.webcal.datasource.DataSourceColumns;

/** 
 *
 * @author  Tim Parker (tim.parker@eXtropia.com)
 * @version 1.0
 */
public class RecursWidget extends Widget {
    public static final String CFG_EDITABLE = "Editable";
    public static final String CFG_IMAGE = "Image";

    public static final String NONE = "none";
    public static final String DAILY = "daily";
    public static final String WEEKLY = "weekly";
    public static final String MONTHLY = "monthly";
    public static final String YEARLY = "yearly";

    public static final String[] RECURRENCES = new String[] { NONE, DAILY, WEEKLY, MONTHLY, YEARLY };
    public static final int[] INT_RECURRENCES = new int[] { DataSourceColumns.RECURRENCE_NONE, DataSourceColumns.RECURRENCE_DAILY, DataSourceColumns.RECURRENCE_WEEKLY, DataSourceColumns.RECURRENCE_MONTHLY, DataSourceColumns.RECURRENCE_YEARLY };

    protected boolean _editable;
    protected String _image;

    /** Creates new PriorityWidget */
    public RecursWidget() {
    }

    public void configure(Config config) throws ConfigException {
        super.configure(config);
        if (config.hasChild(CFG_EDITABLE))
            setEditable(config.getChild(CFG_EDITABLE).getBooleanValue());
        if (config.hasChild(CFG_IMAGE))
            _image = config.getChild(CFG_IMAGE).getValue();
        DataHandler handler = new IntegerHandler();
        handler.setIds(new String[] { _id });
        addDataHandler(handler);
    }

    public void setEditable(boolean editable) {
        _editable = editable;
    }

    public String toString(Locale locale, Object value, ServletRequest request) {
        return _editable ? toEditableString(locale, value, request) : toNonEditableString(locale, value, request);
    }

    public String toNonEditableString(Locale locale, Object value, ServletRequest request) {
        if (!(value instanceof Integer)) {
            try {
                value = new Integer(value.toString());
            } catch (Exception e) {
                value = null;
            }
        }
        int recurrence = DataSourceColumns.RECURRENCE_NONE;
        if (value != null)
            recurrence = ((Integer) value).intValue();
        String resourceName;
        String res;
        switch (recurrence) {
            case DataSourceColumns.RECURRENCE_DAILY :
                resourceName = _resourceBase + "." + DAILY;
                res = DAILY;
                break;
            case DataSourceColumns.RECURRENCE_WEEKLY :
                resourceName = _resourceBase + "." + WEEKLY;
                res = WEEKLY;
                break;
            case DataSourceColumns.RECURRENCE_MONTHLY :
                resourceName = _resourceBase + "." + MONTHLY;
                res = MONTHLY;
                break;
            case DataSourceColumns.RECURRENCE_YEARLY :
                resourceName = _resourceBase + "." + YEARLY;
                res = YEARLY;
                break;
            default :
                resourceName = _resourceBase + "." + NONE;
                res = NONE;
                break;
        }
        ResourceBundle bundle = ResourceBundle.getBundle(_resourceLocation, locale);
        try {
            res = bundle.getString(resourceName);
        } catch (Exception e) {
        } //just leave as the string NONE, DAILY etc.
        if (_image != null) {
            if (recurrence != DataSourceColumns.RECURRENCE_NONE)
                res = "<IMG SRC=\"" + _image + "\" ALT=\"" + getLabel(locale) + " " + res + "\" ALIGN=\"ABSMIDDLE\" BORDER=\"0\">";
            else
                res = "";
        }
        return res;
    }

    protected String toEditableString(Locale locale, Object value, ServletRequest request) {
        value = selectValueToUse(value, request);
        if (!(value instanceof Integer)) {
            try {
                value = new Integer(value.toString());
            } catch (Exception e) {
                value = null;
            }
        }
        int recurrence = DataSourceColumns.RECURRENCE_NONE;
        if (value != null)
            recurrence = ((Integer) value).intValue();
        StringBuffer buffer = new StringBuffer();
        buffer.append("<SELECT name=\"");
        buffer.append(_id);
        buffer.append("\">");

        ResourceBundle bundle = ResourceBundle.getBundle(_resourceLocation, locale);

        for (int i = 0; i < RECURRENCES.length; i++) {
            buffer.append("<OPTION value=\"");
            buffer.append(INT_RECURRENCES[i]);
            buffer.append('\"');
            if (recurrence == INT_RECURRENCES[i]) {
                buffer.append(" SELECTED");
            }
            buffer.append('>');
            value = RECURRENCES[i];
            try {
                value = bundle.getString(_resourceBase + "." + RECURRENCES[i]);
            } catch (Exception e) {
            }
            buffer.append(value);
        }
        buffer.append("</SELECT>");
        return buffer.toString();
    }

    public String toString(Locale locale, Object value, ServletRequest request, String idSuffix) {
        return toString(locale, value, request);
    }
}